@@ -1,5 +1,5 @@
-#ifndef lint
-static char sccsid[] = "@(#)ant.c	3.11 96/09/20 xlockmore";
+#if !defined( lint ) && !defined( SABER )
+static const char sccsid[] = "@(#)ant.c	4.00 97/01/01 xlockmore";
 
 #endif
 
@@ -13,10 +13,13 @@ static char sccsid[] = "@(#)ant.c	3.11 96/09/20 xlockmore";
  * See xlock.c for copying information.
  *
  * Revision History:
+ * 01-Jan-97: Updated ant.c to handle more kinds of ants.  Thanks to
+ *            J Austin David <Austin.David@tlogic.com>.  Check it out in
+ *            java at http://havoc.gtf.gatech.edu/austin  He thought up the
+ *            new Ladder ant.
  * 04-Apr-96: -neighbors 6 runtime-time option added for hexagonal ants
  *            (bees), coded from an idea of Jim Propp's in Science News,
  *            Oct 28, 1995 VOL. 148 page 287
- *            -gridsize <int> runtime option added,  170 is the default
  * 20-Sep-95: Memory leak in ant fixed.  Now random colors.
  * 05-Sep-95: Coded from A.K. Dewdney's "Computer Recreations", Scientific
  *            American Magazine" Sep 1989 pp 180-183, Mar 1990 p 121
@@ -27,7 +30,7 @@ static char sccsid[] = "@(#)ant.c	3.11 96/09/20 xlockmore";
 
 #include "xlock.h"
 
-#define BITS(n,w,h)\
+#define ANTBITS(n,w,h)\
   ap->pixmaps[ap->init_bits++]=\
   XCreatePixmapFromBitmapData(display,window,(char *)n,w,h,1,0,1)
 
@@ -42,7 +45,6 @@ static char sccsid[] = "@(#)ant.c	3.11 96/09/20 xlockmore";
 #define DEF_TRUCHET  "False"
 
 extern int  neighbors;
-extern int  gridsize;
 static Bool truchet;
 
 static XrmOptionDescRec opts[] =
@@ -68,6 +70,10 @@ ModeSpecOpt ant_opts =
 #define MINANTS 1
 #define PATTERNSIZE 8
 #define REDRAWSTEP 2000		/* How much tape to draw per cycle */
+#define MINGRIDSIZE 24
+#define MINSIZE 1
+#define ANGLES 360
+
 static unsigned char patterns[COLORS - 1][PATTERNSIZE] =
 {
 	{0x11, 0x22, 0x11, 0x22, 0x11, 0x22, 0x11, 0x22},	/* grey+white | stripe */
@@ -88,7 +94,8 @@ typedef struct {
 
 typedef struct {
 	int         col, row;
-	unsigned char direction, state;
+	short       direction;
+	unsigned char state;
 } antstruct;
 
 typedef struct {
@@ -108,7 +115,7 @@ typedef struct {
 	unsigned char *truchet_state;
 	antstruct  *ants;
 	unsigned char colors[COLORS - 1];
-	GC          stippled_GC;
+	GC          stippledGC;
 	Pixmap      pixmaps[COLORS - 1];
 	XPoint      hexagonList[7];
 } antfarmstruct;
@@ -124,22 +131,19 @@ static XPoint hexagonUnit[7] =
 	{1, -1}
 };
 
-#define N 0
-#define NE 0
-#define E 1
-#define SE 2
-#define S 3
-#define SW 3
-#define W 4
-#define NW 5
-
 /* Relative ant moves */
-#define F 0
-#define R 1
-#define HR 2
-#define B 3
-#define HL 4
-#define L 5
+#define FS 0			/* Step */
+#define RS 1			/* Turn right, then step */
+#define HRS 2			/* Turn hard right, then step */
+#define BS 3			/* Turn back, then step */
+#define HLS 4			/* Turn hard left, then step */
+#define LS 5			/* Turn left, then step */
+#define SF 6			/* Step */
+#define SR 7			/* Step then turn right */
+#define SHR 8			/* Step then turn hard right */
+#define SB 9			/* Step then turn back */
+#define SHL 10			/* Step then turn hard left */
+#define SL 11			/* Step then turn left */
 
 static antfarmstruct *antfarms = NULL;
 
@@ -151,25 +155,38 @@ static antfarmstruct *antfarms = NULL;
 /* TURK'S 1100 SYMMETRIC ANT  all even run 1's and 0's are symmetric */
 /* other examples 1001, 110011, 110000, 1001101 */
 /* TURK'S 1101 ANT Builder after 250,000 (388p) */
+/* Once saw a chess horse type builder (i.e. non-45 degree builder) */
 
 /* BEE ONLY */
 /* All alternating 10 appear symmetric, no proof (i.e. 10, 1010, etc) */
 /* Even runs of 0's and 1's are also symmetric */
-/* I have yet to see Hexagonal builders. */
+/* I have seen Hexagonal builders but they are more rare. */
 
 static unsigned char tables[][3 * COLORS * STATES + 2] =
 {
+#if 0
+  /* Here just so you can figure out notation */
+	{			/* Langton's ant */
+		2, 1,
+		1, LS, 0, 0, RS, 0
+	},
+#else
   /* First 2 numbers are the size (ncolors, nstates) */
 	{			/* SPIRALING PATTERN */
 		2, 2,
-		1, L, 0, 0, F, 1,
-		1, R, 0, 1, R, 0
+		1, LS, 0, 0, FS, 1,
+		1, RS, 0, 1, RS, 0
 	},
 	{			/* SQUARE (HEXAGON) BUILDER */
 		2, 2,
-		1, L, 0, 0, F, 1,
-		0, R, 0, 1, R, 0
+		1, LS, 0, 0, FS, 1,
+		0, RS, 0, 1, RS, 0
+	},
+	{			/* LADDER BUILDER */
+		4, 1,
+		1, SR, 0, 2, SL, 0, 3, RS, 0, 0, LS, 0
 	}
+#endif
 };
 
 #define NTABLES   (sizeof tables / sizeof tables[0])
@@ -184,12 +201,17 @@ fillcell(ModeInfo * mi, int col, int row, int stipple)
 
 		ap->hexagonList[0].x = ap->xb + ccol * ap->xs;
 		ap->hexagonList[0].y = ap->yb + crow * ap->ys;
-		XFillPolygon(MI_DISPLAY(mi), MI_WINDOW(mi),
-			     (stipple) ? ap->stippled_GC : MI_GC(mi),
-			     ap->hexagonList, 6, Convex, CoordModePrevious);
+		if (ap->xs == 1 && ap->ys == 1)
+			XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi),
+				       (stipple) ? ap->stippledGC : MI_GC(mi),
+			   ap->hexagonList[0].x, ap->hexagonList[0].y, 1, 1);
+		else
+			XFillPolygon(MI_DISPLAY(mi), MI_WINDOW(mi),
+				     (stipple) ? ap->stippledGC : MI_GC(mi),
+			      ap->hexagonList, 6, Convex, CoordModePrevious);
 	} else {
 		XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi),
-			       (stipple) ? ap->stippled_GC : MI_GC(mi),
+			       (stipple) ? ap->stippledGC : MI_GC(mi),
 		ap->xb + ap->xs * col, ap->yb + ap->ys * row, ap->xs, ap->ys);
 	}
 }
@@ -205,8 +227,8 @@ truchetcell(ModeInfo * mi, int col, int row, int truchetstate)
 		XPoint      hex, hex2;
 
 		/* Very crude approx of Sqrt 3, so it will not cause drawing errors. */
-		hex.x = ap->xb + ccol * ap->xs - (int) ((double) ap->xs * 1.5 / 2.0);
-		hex.y = ap->yb + crow * ap->ys - (int) ((double) ap->ys * 1.5 / 2.0);
+		hex.x = ap->xb + ccol * ap->xs - (int) ((double) ap->xs * 1.6 / 2.0);
+		hex.y = ap->yb + crow * ap->ys - (int) ((double) ap->ys * 1.6 / 2.0);
 		for (side = 0; side < 6; side++) {
 			if (side > 0) {
 				hex.x += ap->hexagonList[side].x;
@@ -214,13 +236,11 @@ truchetcell(ModeInfo * mi, int col, int row, int truchetstate)
 			}
 			hex2.x = hex.x + ap->hexagonList[side + 1].x / 2;
 			hex2.y = hex.y + ap->hexagonList[side + 1].y / 2;
-			if (side >= 1 && side < 4)	/* Fudge correction */
-				hex2.y += 1;
 			if (truchetstate == side % 3)
 				/* Crude approx of 120 deg, so it will not cause drawing errors. */
 				XDrawArc(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
-				  hex2.x, hex2.y,
-					(int) ((double) ap->xs * 1.5), (int) ((double) ap->ys * 1.5),
+					 hex2.x, hex2.y,
+					 (int) ((double) ap->xs * 1.5), (int) ((double) ap->ys * 1.5),
 				  ((555 - (side * 60)) % 360) * 64, 90 * 64);
 		}
 	} else {
@@ -270,7 +290,7 @@ drawcell(ModeInfo * mi, int col, int row, unsigned char color)
 		gcv.stipple = ap->pixmaps[color - 1];
 		gcv.foreground = MI_WIN_WHITE_PIXEL(mi);
 		gcv.background = MI_WIN_BLACK_PIXEL(mi);
-		XChangeGC(MI_DISPLAY(mi), ap->stippled_GC,
+		XChangeGC(MI_DISPLAY(mi), ap->stippledGC,
 			  GCStipple | GCForeground | GCBackground, &gcv);
 		fillcell(mi, col, row, 1);
 	}
@@ -307,30 +327,29 @@ draw_anant(ModeInfo * mi, int col, int row)
 
 			XSetForeground(display, MI_GC(mi), MI_WIN_BLACK_PIXEL(mi));
 			switch (direction) {
-				case E:
+				case 0:
 					XDrawPoint(display, window, MI_GC(mi),
 					    ap->xb + ap->xs - 1, ap->yb + 1);
 					XDrawPoint(display, window, MI_GC(mi),
 						   ap->xb + ap->xs - 1, ap->yb + ap->ys - 2);
 					break;
-				case W:
+				case 180:
 					XDrawPoint(display, window, MI_GC(mi), ap->xb, ap->yb + 1);
 					XDrawPoint(display, window, MI_GC(mi), ap->xb, ap->yb + ap->ys - 2);
 					break;
-#ifdef BEE
-#else
-				case N:
-					XDrawPoint(display, window, MI_GC(mi), ap->xb + 1, ap->yb);
-					XDrawPoint(display, window, MI_GC(mi),
+					if (neighbors == 4) {
+				case 90:
+						XDrawPoint(display, window, MI_GC(mi), ap->xb + 1, ap->yb);
+						XDrawPoint(display, window, MI_GC(mi),
 						ap->xb + ap->xs - 2, ap->yb);
-					break;
-				case S:
-					XDrawPoint(display, window, MI_GC(mi),
-					    ap->xb + 1, ap->yb + ap->ys - 1);
-					XDrawPoint(display, window, MI_GC(mi),
-						   ap->xb + ap->xs - 2, ap->yb + ap->ys - 1);
-					break;
-#endif
+						break;
+				case 270:
+						XDrawPoint(display, window, MI_GC(mi),
+							   ap->xb + 1, ap->yb + ap->ys - 1);
+						XDrawPoint(display, window, MI_GC(mi),
+							   ap->xb + ap->xs - 2, ap->yb + ap->ys - 1);
+						break;
+					}	/* else BEE */
 				default:
 			}
 		}
@@ -351,7 +370,7 @@ RandomSoup(mi)
 			ap->old[col + mrow] = (unsigned char) NRAND((int) ap->ncolors);
 			drawcell(mi, col, row, ap->old[col + mrow]);
 		}
-		mrow += gridsize;
+		mrow += ap->nrows;
 	}
 }
 
@@ -378,6 +397,7 @@ GetTable(ModeInfo * mi, int i)
 	}
 	ap->truchet = False;
 }
+
 static void
 GetTurk(ModeInfo * mi, int i)
 {
@@ -395,7 +415,7 @@ GetTurk(ModeInfo * mi, int i)
 	total = ap->ncolors * ap->nstates;
 	for (j = 0; j < total; j++) {
 		ap->machine[j].color = (j + 1) % total;
-		ap->machine[j].direction = (power2 & number) ? R : L;
+		ap->machine[j].direction = (power2 & number) ? RS : LS;
 		ap->machine[j].next = 0;
 		power2 >>= 1;
 	}
@@ -410,6 +430,7 @@ init_ant(ModeInfo * mi)
 {
 	Display    *display = MI_DISPLAY(mi);
 	Window      window = MI_WINDOW(mi);
+	int         size = MI_SIZE(mi);
 	XGCValues   gcv;
 	antfarmstruct *ap;
 	int         col, row, i, dir;
@@ -420,11 +441,12 @@ init_ant(ModeInfo * mi)
 			return;
 	}
 	ap = &antfarms[MI_SCREEN(mi)];
+	ap->redrawing = 0;
 	if ((MI_NPIXELS(mi) <= 2) && (ap->init_bits == 0)) {
 		gcv.fill_style = FillOpaqueStippled;
-		ap->stippled_GC = XCreateGC(display, window, GCFillStyle, &gcv);
+		ap->stippledGC = XCreateGC(display, window, GCFillStyle, &gcv);
 		for (i = 0; i < COLORS - 1; i++)
-			BITS(patterns[i], PATTERNSIZE, PATTERNSIZE);
+			ANTBITS(patterns[i], PATTERNSIZE, PATTERNSIZE);
 	}
 	ap->generation = 0;
 	ap->n = MI_BATCHCOUNT(mi);
@@ -447,8 +469,6 @@ init_ant(ModeInfo * mi)
 		ap->neighbors = 6;
 	else
 		ap->neighbors = (LRAND() & 1) ? 4 : 6;
-	if (gridsize < 4)
-		gridsize = 4;
 
 	if (ap->neighbors == 6) {
 		int         nccols, ncrows;
@@ -457,31 +477,43 @@ init_ant(ModeInfo * mi)
 			ap->width = 2;
 		if (ap->height < 4)
 			ap->height = 4;
-		nccols = MIN(ap->width, 2 * (gridsize + 1));
-		ncrows = MIN(ap->height, 2 * (gridsize + 2));
-		ap->ncols = nccols >> 1;
-		ap->nrows = (ncrows >> 2) << 1;
-		nccols = ap->ncols << 1;
-		ncrows = ap->nrows << 1;
-		ap->xs = ap->width / nccols;
-		if (ap->xs < 2)
-			ap->xs = 2;
-		ap->ys = ap->height / ncrows;
-		if (ap->ys < 2)
-			ap->ys = 2;
+		if (size < -MINSIZE)
+			ap->ys = NRAND(MIN(-size, MAX(MINSIZE, MIN(ap->width, ap->height) /
+				      MINGRIDSIZE)) - MINSIZE + 1) + MINSIZE;
+		else if (size < MINSIZE) {
+			if (!size)
+				ap->ys = MAX(MINSIZE, MIN(ap->width, ap->height) / MINGRIDSIZE);
+			else
+				ap->ys = MINSIZE;
+		} else
+			ap->ys = MIN(size, MAX(MINSIZE, MIN(ap->width, ap->height) /
+					       MINGRIDSIZE));
+		ap->xs = ap->ys;
+		nccols = MAX(ap->width / ap->xs - 2, 2);
+		ncrows = MAX(ap->height / ap->ys - 1, 2);
+		ap->ncols = nccols / 2;
+		ap->nrows = 2 * (ncrows / 4);
 		ap->xb = (ap->width - ap->xs * nccols) / 2 + ap->xs;
 		ap->yb = (ap->height - ap->ys * ncrows) / 2 + ap->ys;
-		ap->ncols -= 1;
-		ap->nrows -= 2;
 		for (i = 0; i < 7; i++) {
 			ap->hexagonList[i].x = (ap->xs - 1) * hexagonUnit[i].x;
 			ap->hexagonList[i].y = (ap->ys - 1) * hexagonUnit[i].y / 2;
 		}
 	} else {
-		ap->ncols = MIN(ap->width, gridsize);
-		ap->nrows = MIN(ap->height, gridsize);
-		ap->xs = ap->width / ap->ncols;
-		ap->ys = ap->height / ap->nrows;
+		if (size < -MINSIZE)
+			ap->ys = NRAND(MIN(-size, MAX(MINSIZE, MIN(ap->width, ap->height) /
+				      MINGRIDSIZE)) - MINSIZE + 1) + MINSIZE;
+		else if (size < MINSIZE) {
+			if (!size)
+				ap->ys = MAX(MINSIZE, MIN(ap->width, ap->height) / MINGRIDSIZE);
+			else
+				ap->ys = MINSIZE;
+		} else
+			ap->ys = MIN(size, MAX(MINSIZE, MIN(ap->width, ap->height) /
+					       MINGRIDSIZE));
+		ap->xs = ap->ys;
+		ap->ncols = MAX(ap->width / ap->xs, 2);
+		ap->nrows = MAX(ap->height / ap->ys, 2);
 		ap->xb = (ap->width - ap->xs * ap->ncols) / 2;
 		ap->yb = (ap->height - ap->ys * ap->nrows) / 2;
 	}
@@ -495,9 +527,8 @@ init_ant(ModeInfo * mi)
 		for (i = 0; i < (int) ap->ncolors; i++)
 			ap->colors[i] = (NRAND(MI_NPIXELS(mi)) +
 			     i * MI_NPIXELS(mi)) / ((int) (ap->ncolors - 1));
-	if (ap->ants != NULL)
-		(void) free((char *) ap->ants);
-	ap->ants = (antstruct *) malloc(ap->n * sizeof (antstruct));
+	if (ap->ants == NULL)
+		ap->ants = (antstruct *) malloc(ap->n * sizeof (antstruct));
 	if (ap->tape != NULL)
 		(void) free((char *) ap->tape);
 	ap->tape = (unsigned char *)
@@ -510,7 +541,7 @@ init_ant(ModeInfo * mi)
 
 	col = ap->ncols / 2;
 	row = ap->nrows / 2;
-	dir = NRAND(ap->neighbors);
+	dir = NRAND(ap->neighbors) * ANGLES / ap->neighbors;
 	/* Have them all start in the same spot, why not? */
 	for (i = 0; i < ap->n; i++) {
 		ap->ants[i].col = col;
@@ -528,7 +559,8 @@ draw_ant(ModeInfo * mi)
 	antstruct  *anant;
 	statestruct *status;
 	int         i, state_pos, tape_pos;
-	unsigned char color, temp1, temp2, old_dir;
+	unsigned char color;
+	short       chg_dir, old_dir;
 
 	for (i = 0; i < ap->n; i++) {
 		anant = &ap->ants[i];
@@ -540,79 +572,77 @@ draw_ant(ModeInfo * mi)
 		ap->tape[tape_pos] = status->color;	/* write on tape */
 
 		/* Find direction of Bees or Ants. */
+		/* Translate relative direction to actual direction */
 		old_dir = anant->direction;
-		temp1 = (ap->neighbors == 4 && status->direction > 1) ?
-			(unsigned char) ((int) (status->direction + 1) / 2) : status->direction;
-		temp2 = (ap->neighbors == 4 && anant->direction > 1) ?
-			anant->direction - 1 : anant->direction;
-		anant->direction = (unsigned char) ((int) (temp1 + temp2) % ap->neighbors);
-		anant->direction = (ap->neighbors == 4 && anant->direction > 1) ?
-			anant->direction + 1 : anant->direction;
+		chg_dir = (ap->neighbors == 4 && (status->direction % 6) > 1) ?
+			(unsigned char) ((int) ((status->direction % 6) + 1) / 2) :
+			status->direction % 6;
+		chg_dir = (ANGLES - chg_dir * (ANGLES / ap->neighbors)) % ANGLES;
+		anant->direction = (chg_dir + old_dir) % ANGLES;
 		if (ap->truchet) {
 			int         a, b;
 
 			if (ap->neighbors == 4) {
-				a = ((old_dir > 1) ? old_dir - 1 : old_dir) / 2;
-				b = ((anant->direction > 1) ?
-				anant->direction - 1 : anant->direction) / 2;
+				a = old_dir / 180;
+				b = anant->direction / 180;
 				a = ((a && !b) || (b && !a));
 				drawtruchet(mi, anant->col, anant->row, status->color, a);
 			} else {
-				a = old_dir % 3;
-				b = anant->direction % 3;
-				a = 3 - (a + b);
+				a = (old_dir / 60) % 3;
+				b = (anant->direction / 60) % 3;
+				a = (a + b + 1) % 3;
 				drawtruchet(mi, anant->col, anant->row, status->color, a);
 			}
-			ap->truchet_state[tape_pos] = a;
+			ap->truchet_state[tape_pos] = a + 1;
 		}
 		anant->state = status->next;
 
 		/* If edge than wrap it */
 		if (ap->neighbors == 6) {
-			switch (anant->direction) {
-				case NE:
-					if (!(anant->row & 1))
-						anant->col = (anant->col + 1 == ap->ncols) ? 0 : anant->col + 1;
-					anant->row = (!anant->row) ? ap->nrows - 1 : anant->row - 1;
-					break;
-				case E:
+			switch ((status->direction < 6) ? anant->direction : old_dir) {
+				case 0:
 					anant->col = (anant->col + 1 == ap->ncols) ? 0 : anant->col + 1;
 					break;
-				case SE:
+				case 60:
 					if (!(anant->row & 1))
 						anant->col = (anant->col + 1 == ap->ncols) ? 0 : anant->col + 1;
-					anant->row = (anant->row + 1 == ap->nrows) ? 0 : anant->row + 1;
+					anant->row = (!anant->row) ? ap->nrows - 1 : anant->row - 1;
 					break;
-				case SW:
-					if (anant->row & 1)
-						anant->col = (!anant->col) ? ap->ncols - 1 : anant->col - 1;
-					anant->row = (anant->row + 1 == ap->nrows) ? 0 : anant->row + 1;
-					break;
-				case W:
-					anant->col = (!anant->col) ? ap->ncols - 1 : anant->col - 1;
-					break;
-				case NW:
+				case 120:
 					if (anant->row & 1)
 						anant->col = (!anant->col) ? ap->ncols - 1 : anant->col - 1;
 					anant->row = (!anant->row) ? ap->nrows - 1 : anant->row - 1;
 					break;
+				case 180:
+					anant->col = (!anant->col) ? ap->ncols - 1 : anant->col - 1;
+					break;
+				case 240:
+					if (anant->row & 1)
+						anant->col = (!anant->col) ? ap->ncols - 1 : anant->col - 1;
+					anant->row = (anant->row + 1 == ap->nrows) ? 0 : anant->row + 1;
+					break;
+				case 300:
+					if (!(anant->row & 1))
+						anant->col = (anant->col + 1 == ap->ncols) ? 0 : anant->col + 1;
+					anant->row = (anant->row + 1 == ap->nrows) ? 0 : anant->row + 1;
+					break;
 				default:
 					warning("%s: wrong direction %d\n", anant->direction);
 			}
 		} else {
-			switch (anant->direction) {
-				case N:
-					anant->row = (!anant->row) ? ap->nrows - 1 : anant->row - 1;
-					break;
-				case E:
+			switch ((status->direction < 6) ? anant->direction : old_dir) {
+				case 0:
 					anant->col = (anant->col + 1 == ap->ncols) ? 0 : anant->col + 1;
 					break;
-				case S:
-					anant->row = (anant->row + 1 == ap->nrows) ? 0 : anant->row + 1;
+				case 90:
+					anant->row = (!anant->row) ? ap->nrows - 1 : anant->row - 1;
 					break;
-				case W:
+				case 180:
 					anant->col = (!anant->col) ? ap->ncols - 1 : anant->col - 1;
 					break;
+				case 270:
+					anant->row = (anant->row + 1 == ap->nrows) ? 0 : anant->row + 1;
+					break;
 				default:
 					warning("%s: wrong direction %d\n", anant->direction);
 			}
@@ -622,15 +652,15 @@ draw_ant(ModeInfo * mi)
 	if (++ap->generation > MI_CYCLES(mi))
 		init_ant(mi);
 	if (ap->redrawing) {
-		int         i;
-
 		for (i = 0; i < REDRAWSTEP; i++) {
-			if (ap->tape[ap->redrawpos]) {
-				drawcell(mi, ap->redrawpos % ap->ncols, ap->redrawpos / ap->nrows,
+			if (ap->tape[ap->redrawpos] ||
+			 (ap->truchet && ap->truchet_state[ap->redrawpos])) {
+				drawcell(mi, ap->redrawpos % ap->ncols, ap->redrawpos / ap->ncols,
 					 ap->tape[ap->redrawpos]);
 				if (ap->truchet)
-					drawtruchet(mi, ap->redrawpos % ap->ncols, ap->redrawpos / ap->nrows,
-						    ap->tape[ap->redrawpos], ap->truchet_state[ap->redrawpos]);
+					drawtruchet(mi, ap->redrawpos % ap->ncols, ap->redrawpos / ap->ncols,
+						    ap->tape[ap->redrawpos],
+					ap->truchet_state[ap->redrawpos] - 1);
 			}
 			if (++(ap->redrawpos) >= ap->ncols * ap->nrows) {
 				ap->redrawing = 0;
@@ -649,8 +679,8 @@ release_ant(ModeInfo * mi)
 		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++) {
 			antfarmstruct *ap = &antfarms[screen];
 
-			if (ap->stippled_GC != NULL) {
-				XFreeGC(MI_DISPLAY(mi), ap->stippled_GC);
+			if (ap->stippledGC != NULL) {
+				XFreeGC(MI_DISPLAY(mi), ap->stippledGC);
 			}
 			if (ap->init_bits != 0) {
 				for (shade = 0; shade < COLORS - 1; shade++)
@@ -660,6 +690,8 @@ release_ant(ModeInfo * mi)
 				(void) free((void *) ap->tape);
 			if (ap->ants != NULL)
 				(void) free((void *) ap->ants);
+			if (ap->truchet_state != NULL)
+				(void) free((char *) ap->truchet_state);
 		}
 		(void) free((void *) antfarms);
 		antfarms = NULL;
