@@ -1,6 +1,6 @@
 
-#ifndef lint
-static char sccsid[] = "@(#)bug.c	3.11 96/09/20 xlockmore";
+#if !defined( lint ) && !defined( SABER )
+static const char sccsid[] = "@(#)bug.c	4.00 97/01/01 xlockmore";
 
 #endif
 
@@ -18,7 +18,11 @@ static char sccsid[] = "@(#)bug.c	3.11 96/09/20 xlockmore";
  *            also used wator.c as a guide.
  */
 
-#include <math.h>
+/*-
+ * Bugs learn to hunt bacteria (or die) in the Garden of Eden and outside.
+ * They start as jitterbugs and "evolve" to move straight (in the Garden
+ *  they may evolve to twirl around).
+ */
 #include "xlock.h"
 
 #define BACTERIA 0
@@ -33,8 +37,9 @@ static char sccsid[] = "@(#)bug.c	3.11 96/09/20 xlockmore";
 #define MATURE 800		/* Breeding time of bugs */
 #define MAXGENE 6
 #define MINGENE (-MAXGENE)
-
 #define REDRAWSTEP 2000		/* How many bacteria to draw per cycle */
+#define MINGRIDSIZE 24
+#define MINSIZE 1
 
 /* Relative bug moves */
 /* Forward, Right, Hard Right, Reverse, Hard Left, Left */
@@ -50,8 +55,6 @@ static char sccsid[] = "@(#)bug.c	3.11 96/09/20 xlockmore";
 ModeSpecOpt bug_opts =
 {0, NULL, 0, NULL, NULL};
 
-extern int  gridsize;
-
 /* Bug data */
 typedef struct {
 	char        direction;
@@ -397,6 +400,7 @@ void
 init_bug(ModeInfo * mi)
 {
 	bugfarmstruct *bp;
+	int         size = MI_SIZE(mi);
 	int         i = 0, j = 0, hcol, hrow, gene, colrow;
 	double      sum;
 	bugstruct   info;
@@ -407,6 +411,7 @@ init_bug(ModeInfo * mi)
 			return;
 	}
 	bp = &bugfarms[MI_SCREEN(mi)];
+	bp->redrawing = 0;
 	bp->generation = 0;
 	if (!bp->initialized) {	/* Genesis */
 		bp->initialized = 1;
@@ -421,23 +426,32 @@ init_bug(ModeInfo * mi)
 	} else			/* Apocolypse */
 		flush_buglist(bp);
 
-	bp->width = MI_WIN_WIDTH(mi) - 2;
+	bp->width = MI_WIN_WIDTH(mi);
 	bp->height = MI_WIN_HEIGHT(mi);
 	if (bp->width < 4)
 		bp->width = 4;
 	if (bp->height < 4)
 		bp->height = 4;
-	if (gridsize < 4)
-		gridsize = 4;
 
-	bp->nccols = MIN(bp->width, 2 * gridsize);
-	bp->ncrows = MIN(bp->height, 2 * gridsize);
+	if (size < -MINSIZE)
+		bp->ys = NRAND(MIN(-size, MAX(MINSIZE, MIN(bp->width, bp->height) /
+				      MINGRIDSIZE)) - MINSIZE + 1) + MINSIZE;
+	else if (size < MINSIZE) {
+		if (!size)
+			bp->ys = MAX(MINSIZE, MIN(bp->width, bp->height) / MINGRIDSIZE);
+		else
+			bp->ys = MINSIZE;
+	} else
+		bp->ys = MIN(size, MAX(MINSIZE, MIN(bp->width, bp->height) /
+				       MINGRIDSIZE));
+	bp->xs = bp->ys;
+	bp->nccols = MAX(bp->width / bp->xs - 1, 2);
+	bp->ncrows = MAX(bp->height / bp->ys, 2);
 	bp->nhcols = bp->nccols / 2;
 	bp->nhrows = bp->ncrows / 2;
-	if (!(bp->nhrows & 1)) {	/* Force odd rows  */
-		bp->ncrows -= 2;
-		bp->nhrows--;
-	}
+	bp->nhrows -= !(bp->nhrows & 1);	/* Must be odd */
+	bp->xb = (bp->width - bp->xs * bp->nccols) / 2;
+	bp->yb = (bp->height - bp->ys * bp->ncrows) / 2;
 	if (bp->arr != NULL)
 		(void) free((void *) bp->arr);
 	bp->arr = (BugList **) calloc(bp->nhcols * bp->nhrows, sizeof (BugList *));
@@ -445,16 +459,11 @@ init_bug(ModeInfo * mi)
 		(void) free((void *) bp->bacteria);
 	bp->bacteria = (char *) calloc(bp->nhcols * bp->nhrows, sizeof (char));
 
-	bp->xs = bp->width / bp->nccols;
-	bp->ys = bp->height / bp->ncrows;
-	bp->xb = (bp->width - bp->xs * bp->nccols) / 2;
-	bp->yb = (bp->height - bp->ys * bp->ncrows) / 2;
-
-	bp->edenwidth = bp->nhrows / 4;
-	bp->edenheight = bp->nhcols / 4;
+	bp->edenwidth = bp->nhcols / 4;
+	bp->edenheight = bp->nhrows / 4;
 	if (bp->edenheight)
-		bp->edenheight -= !(bp->edenheight && 1);	/* Make sure its odd */
-	bp->eden = (bp->edenwidth && bp->edenheight);
+		bp->edenheight -= !(bp->edenheight & 1);	/* Make sure its odd */
+	bp->eden = (bp->edenwidth > 1 && bp->edenheight > 1);
 	if (bp->eden) {
 		bp->edenstartx = NRAND(bp->nhcols - bp->edenwidth);
 		bp->edenstarty = NRAND(bp->nhrows - bp->edenheight);
