@@ -1,6 +1,6 @@
 
-#ifndef lint
-static char sccsid[] = "@(#)demon.c	3.11 96/09/20 xlockmore";
+#if !defined( lint ) && !defined( SABER )
+static const char sccsid[] = "@(#)demon.c	4.01 97/02/24 xlockmore";
 
 #endif
 
@@ -13,16 +13,17 @@ static char sccsid[] = "@(#)demon.c	3.11 96/09/20 xlockmore";
  * See xlock.c for copying information.
  *
  * Revision History:
- * 30-May-96: Ron Hitchens <ronh@utw.com>
+ * 30-May-96: Ron Hitchens <ron@idiom.com>
  *            Fixed memory management that caused leaks
  * 14-Apr-96: -neighbors 6 runtime-time option added
- *            -gridsize <int> runtime option added 50 good for hexagons
- *            while 170 for squares.
  * 21-Aug-95: Coded from A.K. Dewdney's "Computer Recreations", Scientific
  *            American Magazine" Aug 1989 pp 102-105.
  *            also used life.c as a guide.
  */
 
+/*-
+ * A cellular universe of 4 phases debris, droplets, defects, and demons.
+ */
 #include "xlock.h"
 
 /*-
@@ -37,9 +38,11 @@ ModeSpecOpt demon_opts =
 {0, NULL, 0, NULL, NULL};
 
 extern int  neighbors;
-extern int  gridsize;
 
 #define REDRAWSTEP 2000		/* How many cells to draw per cycle */
+#define MINSTATES 2
+#define MINGRIDSIZE 24
+#define MINSIZE 4
 
 /* Singly linked list */
 typedef struct _DemonList {
@@ -94,8 +97,12 @@ drawcell(ModeInfo * mi, int col, int row, unsigned char state)
 
 		dp->hexagonList[0].x = dp->xb + ccol * dp->xs;
 		dp->hexagonList[0].y = dp->yb + crow * dp->ys;
-		XFillPolygon(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
-			     dp->hexagonList, 6, Convex, CoordModePrevious);
+		if (dp->xs == 1 && dp->ys == 1)
+			XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi),
+				       MI_GC(mi), dp->hexagonList[0].x, dp->hexagonList[0].y, 1, 1);
+		else
+			XFillPolygon(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
+			      dp->hexagonList, 6, Convex, CoordModePrevious);
 	} else {
 		XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
 		dp->xb + dp->xs * col, dp->yb + dp->ys * row, dp->xs, dp->ys);
@@ -147,7 +154,8 @@ free_state(demonstruct * dp, int state)
 		(void) free((void *) current);
 	}
 	dp->demonlist[state] = NULL;
-	dp->ncells[state] = 0;
+	if (dp->ncells)
+		dp->ncells[state] = 0;
 }
 
 
@@ -160,6 +168,21 @@ free_list(demonstruct * dp)
 		free_state(dp, state);
 }
 
+static void
+free_struct(demonstruct * dp)
+{
+	if (dp->demonlist != NULL)
+		free_list(dp);
+	if (dp->demonlist != NULL)
+		(void) free((void *) dp->demonlist);
+	if (dp->ncells != NULL)
+		(void) free((void *) dp->ncells);
+	if (dp->oldcell != NULL)
+		(void) free((void *) dp->oldcell);
+	if (dp->newcell != NULL)
+		(void) free((void *) dp->newcell);
+}
+
 static void
 draw_state(ModeInfo * mi, int state)
 {
@@ -212,20 +235,7 @@ draw_state(ModeInfo * mi, int state)
 		(void) free((void *) rects);
 	}
 	free_state(dp, state);
-	if (dp->redrawing) {
-		int         i;
-
-		for (i = 0; i < REDRAWSTEP; i++) {
-			if (dp->oldcell[dp->redrawpos]) {
-				drawcell(mi, dp->redrawpos % dp->ncols, dp->redrawpos / dp->ncols,
-					 dp->oldcell[dp->redrawpos]);
-			}
-			if (++(dp->redrawpos) >= dp->ncols * dp->nrows) {
-				dp->redrawing = 0;
-				break;
-			}
-		}
-	}
+	XFlush(MI_DISPLAY(mi));
 }
 
 static void
@@ -247,6 +257,7 @@ RandomSoup(ModeInfo * mi)
 void
 init_demon(ModeInfo * mi)
 {
+	int         size = MI_SIZE(mi);
 	demonstruct *dp;
 
 	if (demons == NULL) {
@@ -256,18 +267,15 @@ init_demon(ModeInfo * mi)
 	}
 	dp = &demons[MI_SCREEN(mi)];
 	dp->generation = 0;
+	dp->redrawing = 0;
 	dp->states = MI_BATCHCOUNT(mi);
-	if (dp->states < 1)
-		dp->states = 1;
-	if (dp->demonlist == NULL)
-		dp->demonlist = (DemonList **) calloc(dp->states, sizeof (DemonList *));
-	else
-		free_list(dp);
-	if (dp->ncells == NULL)
-		dp->ncells = (int *) calloc(dp->states, sizeof (int));
-
-	else
-		(void) memset((char *) dp->ncells, 0, dp->states * sizeof (int));
+	if (dp->states < -MINSTATES)
+		dp->states = NRAND(-dp->states - MINSTATES + 1) + MINSTATES;
+	else if (dp->states < MINSTATES)
+		dp->states = MINSTATES;
+	free_struct(dp);
+	dp->demonlist = (DemonList **) calloc(dp->states, sizeof (DemonList *));
+	dp->ncells = (int *) calloc(dp->states, sizeof (int));
 
 	dp->state = 0;
 
@@ -280,8 +288,6 @@ init_demon(ModeInfo * mi)
 		dp->neighbors = 6;
 	else
 		dp->neighbors = (LRAND() & 1) ? 4 : 6;
-	if (gridsize < 4)
-		gridsize = 4;
 
 	if (dp->neighbors == 6) {
 		int         nccols, ncrows, i;
@@ -290,44 +296,54 @@ init_demon(ModeInfo * mi)
 			dp->width = 2;
 		if (dp->height < 4)
 			dp->height = 4;
-		nccols = MIN(dp->width, 2 * (gridsize + 1));
-		ncrows = MIN(dp->height, 2 * (gridsize + 2));
-		dp->ncols = nccols >> 1;
-		dp->nrows = (ncrows >> 2) << 1;
-		nccols = dp->ncols << 1;
-		ncrows = dp->nrows << 1;
-		dp->xs = dp->width / nccols;
-		if (dp->xs < 2)
-			dp->xs = 2;
-		dp->ys = dp->height / ncrows;
-		if (dp->ys < 2)
-			dp->ys = 2;
+		if (size < -MINSIZE)
+			dp->ys = NRAND(MIN(-size, MAX(MINSIZE, MIN(dp->width, dp->height) /
+				      MINGRIDSIZE)) - MINSIZE + 1) + MINSIZE;
+		else if (size < MINSIZE) {
+			if (!size)
+				dp->ys = MAX(MINSIZE, MIN(dp->width, dp->height) / MINGRIDSIZE);
+			else
+				dp->ys = MINSIZE;
+		} else
+			dp->ys = MIN(size, MAX(MINSIZE, MIN(dp->width, dp->height) /
+					       MINGRIDSIZE));
+		dp->xs = dp->ys;
+		nccols = MAX(dp->width / dp->xs - 2, 2);
+		ncrows = MAX(dp->height / dp->ys - 1, 2);
+		dp->ncols = nccols / 2;
+		dp->nrows = 2 * (ncrows / 4);
 		dp->xb = (dp->width - dp->xs * nccols) / 2 + dp->xs;
 		dp->yb = (dp->height - dp->ys * ncrows) / 2 + dp->ys;
-		dp->ncols -= 1;
-		dp->nrows -= 2;
 		for (i = 0; i < 6; i++) {
 			dp->hexagonList[i].x = (dp->xs - 1) * hexagonUnit[i].x;
 			dp->hexagonList[i].y = (dp->ys - 1) * hexagonUnit[i].y / 2;
 		}
 	} else {
-		dp->ncols = MIN(dp->width, gridsize);
-		dp->nrows = MIN(dp->height, gridsize);
-		dp->xs = dp->width / dp->ncols;
-		dp->ys = dp->height / dp->nrows;
+		if (size < -MINSIZE)
+			dp->ys = NRAND(MIN(-size, MAX(MINSIZE, MIN(dp->width, dp->height) /
+				      MINGRIDSIZE)) - MINSIZE + 1) + MINSIZE;
+		else if (size < MINSIZE) {
+			if (!size)
+				dp->ys = MAX(MINSIZE, MIN(dp->width, dp->height) / MINGRIDSIZE);
+			else
+				dp->ys = MINSIZE;
+		} else
+			dp->ys = MIN(size, MAX(MINSIZE, MIN(dp->width, dp->height) /
+					       MINGRIDSIZE));
+		dp->xs = dp->ys;
+		dp->ncols = MAX(dp->width / dp->xs, 2);
+		dp->nrows = MAX(dp->height / dp->ys, 2);
 		dp->xb = (dp->width - dp->xs * dp->ncols) / 2;
 		dp->yb = (dp->height - dp->ys * dp->nrows) / 2;
 	}
 
 	XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
 
-	if (dp->oldcell == NULL)
-		dp->oldcell = (unsigned char *)
-			malloc(dp->ncols * dp->nrows * sizeof (unsigned char));
+	dp->oldcell = (unsigned char *)
+		malloc(dp->ncols * dp->nrows * sizeof (unsigned char));
 
-	if (dp->newcell == NULL)
-		dp->newcell = (unsigned char *)
-			malloc(dp->ncols * dp->nrows * sizeof (unsigned char));
+	dp->newcell = (unsigned char *)
+		malloc(dp->ncols * dp->nrows * sizeof (unsigned char));
 
 	RandomSoup(mi);
 }
@@ -454,6 +470,19 @@ draw_demon(ModeInfo * mi)
 			draw_state(mi, dp->state);
 		dp->state++;
 	}
+
+	if (dp->redrawing) {
+		for (i = 0; i < REDRAWSTEP; i++) {
+			if (dp->oldcell[dp->redrawpos]) {
+				drawcell(mi, dp->redrawpos % dp->ncols, dp->redrawpos / dp->ncols,
+					 dp->oldcell[dp->redrawpos]);
+			}
+			if (++(dp->redrawpos) >= dp->ncols * dp->nrows) {
+				dp->redrawing = 0;
+				break;
+			}
+		}
+	}
 }
 
 void
@@ -462,22 +491,8 @@ release_demon(ModeInfo * mi)
 	if (demons != NULL) {
 		int         screen;
 
-		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++) {
-			free_list(&demons[screen]);
-			if (demons[screen].demonlist != NULL)
-				(void) free((void *) demons[screen].demonlist);
-			/* ncells is referenced by free_list() above */
-			if (demons[screen].ncells != NULL)
-				(void) free((void *) demons[screen].ncells);
-			if (demons[screen].oldcell != NULL)
-				(void) free((void *) demons[screen].oldcell);
-			if (demons[screen].newcell != NULL)
-				(void) free((void *) demons[screen].newcell);
-			/*
-			 * No need to nullify these pointers, they will
-			 * evaporate when the global array "demons" is freed
-			 */
-		}
+		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
+			free_struct(&demons[screen]);
 		(void) free((void *) demons);
 		demons = NULL;
 	}
