@@ -1,13 +1,13 @@
 
-#ifndef lint
-static char sccsid[] = "@(#)eyes.c	3.11 96/09/20 xlockmore";
+#if !defined( lint ) && !defined( SABER )
+static const char sccsid[] = "@(#)eyes.c	4.00 97/01/01 xlockmore";
 
 #endif
 
 /*-
  * eyes.c - Follow the bouncing Grelb
  *
- * Copyright 1996 by Ron Hitchens <ronh@utw.com>
+ * Copyright 1996 by Ron Hitchens <ron@idiom.com>
  *
  * Adapted from the ubiquitous xeyes demo supplied by MIT with the
  * X Window System.
@@ -25,7 +25,6 @@ static char sccsid[] = "@(#)eyes.c	3.11 96/09/20 xlockmore";
 
 /* ---------------------------------------------------------------------- */
 
-#include <math.h>
 #include "xlock.h"
 
 /* ---------------------------------------------------------------------- */
@@ -224,7 +223,7 @@ join_rects(XRectangle * r1, XRectangle * r2, XRectangle * ret)
  *      This code lifted intact from the xeyes widget.
  */
 
-#if (defined( SVR4 ) || defined( SYSV ) && defined( SYSV386 )) && defined( __STDC__ )
+#if defined( SVR4 ) || defined( SYSV ) && defined( SYSV386 )
 extern double hypot(double, double);
 
 #endif
