@@ -1,12 +1,12 @@
 
-#ifndef lint
-static char sccsid[] = "@(#)flag.c	3.11 96/09/20 xlockmore";
+#if !defined( lint ) && !defined( SABER )
+static const char sccsid[] = "@(#)flag.c	4.00 97/01/01 xlockmore";
 
 #endif
 
 /*-
  * flag.c - PEtite demo X11 de charles vidal 15 05 96
- *          <charles@cln46fw.der.edf.fr>
+ *          <vidalc@univ-mlv.fr>
  *          tourne sous Linux et SOLARIS
  *          thank's to Bas van Gaalen, Holland, PD, for his sources
  *          in pascal vous devez rajouter une ligne dans mode.c
@@ -17,11 +17,10 @@ static char sccsid[] = "@(#)flag.c	3.11 96/09/20 xlockmore";
  * 01-May-96: written.
  */
 
-#include <math.h>
 #include "xlock.h"
 #include "flag.h"
 
-#define MINPOINTSIZE 1
+#define MINSIZE 1
 #define MAXSCALE 8
 #define MINSCALE 2
 #define MAXINITSIZE 6
@@ -50,6 +49,7 @@ typedef struct {
 	int         pointsize;
 	double      size;
 	double      inctaille;
+	int         startcolor;
 } flagstruct;
 
 static flagstruct *flags = NULL;
@@ -70,64 +70,6 @@ initSintab(ModeInfo * mi)
 		fp->stab[i] = (int) (sin(i * 4 * M_PI / ANGLES) * fp->samp) + fp->sofs;
 }
 
-void
-init_flag(ModeInfo * mi)
-{
-	Display    *display = MI_DISPLAY(mi);
-	flagstruct *fp;
-
-	if (flags == NULL) {
-		if ((flags = (flagstruct *) calloc(MI_NUM_SCREENS(mi),
-					       sizeof (flagstruct))) == NULL)
-			return;
-	}
-	fp = &flags[MI_SCREEN(mi)];
-
-	fp->width = MI_WIN_WIDTH(mi);
-	fp->height = MI_WIN_HEIGHT(mi);
-
-	fp->samp = MAXAMP;	/* Amplitude */
-	fp->sofs = 20;		/* ???????? */
-	fp->pointsize = MI_BATCHCOUNT(mi);
-	if (fp->pointsize < -MINPOINTSIZE)
-		fp->pointsize = NRAND(-fp->pointsize - MINPOINTSIZE + 1) + MINPOINTSIZE;
-	if (fp->pointsize < MINPOINTSIZE ||
-	fp->width <= MAXW(fp->pointsize) || fp->height <= MAXH(fp->pointsize))
-		fp->pointsize = MINPOINTSIZE;
-	fp->size = MAXINITSIZE;	/* Initial distance between pts */
-	fp->inctaille = 0.05;
-	fp->timer = 0;
-	fp->sidx = fp->x_flag = fp->y_flag = 0;
-
-	if (!fp->initialized) {
-		fp->initialized = True;
-		if (!(fp->cache = XCreatePixmap(display, MI_WINDOW(mi),
-		MAXW(fp->pointsize), MAXH(fp->pointsize), MI_WIN_DEPTH(mi))))
-			error("%s: catastrophe memoire\n");
-	}
-	XSetForeground(display, MI_GC(mi), MI_WIN_BLACK_PIXEL(mi));
-	XFillRectangle(display, fp->cache, MI_GC(mi),
-		       0, 0, MAXW(fp->pointsize), MAXH(fp->pointsize));
-	/* don't want any exposure events from XCopyArea */
-	XSetGraphicsExposures(display, MI_GC(mi), False);
-
-	if (fp->width <= MAXW(fp->pointsize) || fp->height <= MAXH(fp->pointsize)) {
-		fp->samp = MINAMP;
-		fp->sofs = 0;
-		fp->x_flag = random_num(fp->width - MINW(fp->pointsize));
-		fp->y_flag = random_num(fp->height - MINH(fp->pointsize));
-	} else {
-		fp->samp = MAXAMP;
-		fp->sofs = 20;
-		fp->x_flag = random_num(fp->width - MAXW(fp->pointsize));
-		fp->y_flag = random_num(fp->height - MAXH(fp->pointsize));
-	}
-
-	initSintab(mi);
-
-	XClearWindow(display, MI_WINDOW(mi));
-}
-
 static void
 affiche(ModeInfo * mi)
 {
@@ -151,8 +93,8 @@ affiche(ModeInfo * mi)
 				XSetForeground(display, MI_GC(mi), MI_WIN_WHITE_PIXEL(mi));
 			else
 				XSetForeground(display, MI_GC(mi),
-				MI_PIXEL(mi, (y + x + fp->sidx) % NUMCOLORS));
-			if (fp->pointsize <= MINPOINTSIZE)
+					       MI_PIXEL(mi, (y + x + fp->sidx + fp->startcolor) % MI_NPIXELS(mi)));
+			if (fp->pointsize <= 1)
 				XDrawPoint(display, fp->cache, MI_GC(mi), xp, yp);
 			else
 				XFillRectangle(display, fp->cache, MI_GC(mi), xp, yp,
@@ -161,6 +103,66 @@ affiche(ModeInfo * mi)
 		}
 }
 
+void
+init_flag(ModeInfo * mi)
+{
+	Display    *display = MI_DISPLAY(mi);
+	int         size = MI_SIZE(mi);
+	flagstruct *fp;
+
+	if (flags == NULL) {
+		if ((flags = (flagstruct *) calloc(MI_NUM_SCREENS(mi),
+					       sizeof (flagstruct))) == NULL)
+			return;
+	}
+	fp = &flags[MI_SCREEN(mi)];
+
+	fp->width = MI_WIN_WIDTH(mi);
+	fp->height = MI_WIN_HEIGHT(mi);
+
+	fp->samp = MAXAMP;	/* Amplitude */
+	fp->sofs = 20;		/* ???????? */
+	fp->pointsize = size;
+	if (size < -MINSIZE)
+		fp->pointsize = NRAND(-size - MINSIZE + 1) + MINSIZE;
+	if (fp->pointsize < MINSIZE ||
+	fp->width <= MAXW(fp->pointsize) || fp->height <= MAXH(fp->pointsize))
+		fp->pointsize = MINSIZE;
+	fp->size = MAXINITSIZE;	/* Initial distance between pts */
+	fp->inctaille = 0.05;
+	fp->timer = 0;
+	fp->sidx = fp->x_flag = fp->y_flag = 0;
+
+	if (!fp->initialized) {
+		fp->initialized = True;
+		if (!(fp->cache = XCreatePixmap(display, MI_WINDOW(mi),
+		MAXW(fp->pointsize), MAXH(fp->pointsize), MI_WIN_DEPTH(mi))))
+			error("%s: catastrophe memoire\n");
+	}
+	XSetForeground(display, MI_GC(mi), MI_WIN_BLACK_PIXEL(mi));
+	XFillRectangle(display, fp->cache, MI_GC(mi),
+		       0, 0, MAXW(fp->pointsize), MAXH(fp->pointsize));
+	/* don't want any exposure events from XCopyArea */
+	XSetGraphicsExposures(display, MI_GC(mi), False);
+	if (MI_NPIXELS(mi) > 2)
+		fp->startcolor = NRAND(MI_NPIXELS(mi));
+	if (fp->width <= MAXW(fp->pointsize) || fp->height <= MAXH(fp->pointsize)) {
+		fp->samp = MINAMP;
+		fp->sofs = 0;
+		fp->x_flag = random_num(fp->width - MINW(fp->pointsize));
+		fp->y_flag = random_num(fp->height - MINH(fp->pointsize));
+	} else {
+		fp->samp = MAXAMP;
+		fp->sofs = 20;
+		fp->x_flag = random_num(fp->width - MAXW(fp->pointsize));
+		fp->y_flag = random_num(fp->height - MAXH(fp->pointsize));
+	}
+
+	initSintab(mi);
+
+	XClearWindow(display, MI_WINDOW(mi));
+}
+
 void
 draw_flag(ModeInfo * mi)
 {
