@@ -1,11 +1,10 @@
-
-#ifndef lint
-static char sccsid[] = "@(#)life1d.c	3.11 96/09/20 xlockmore";
+#if !defined( lint ) && !defined( SABER )
+static const char sccsid[] = "@(#)life1d.c	4.00 97/01/01 xlockmore";
 
 #endif
 
 /*-
- * life1d.c - Stephen Wolfram's game of Life for xlock, the X Window
+ * life1d.c - Stephen Wolfram's 1d game of Life for xlock, the X Window
  *            System lockscreen.
  *
  * Copyright (c) 1995 by David Bagley.
@@ -13,7 +12,7 @@ static char sccsid[] = "@(#)life1d.c	3.11 96/09/20 xlockmore";
  * See xlock.c for copying information.
  *
  * Revision History:
- * Changes of David Bagley <bagleyd@hertz.njit.edu>
+ * Changes of David Bagley <bagleyd@bigfoot.com>
  * 27-Jul-95: written, used life.c as a basis, using totalistic rules
  *            (default).  Special thanks to Harold V. McIntosh
  *            <mcintosh@servidor.unam.mx> for providing me with the
@@ -34,7 +33,9 @@ static char sccsid[] = "@(#)life1d.c	3.11 96/09/20 xlockmore";
  *
  */
 
-#include "xlock.h"
+#define LIFE1DBITS(n,w,h)\
+  lp->pixmaps[lp->init_bits++]=\
+  XCreatePixmapFromBitmapData(display,window,(char *)n,w,h,1,0,1)
 
 /* aliases for vars defined in the bitmap file */
 #define CELL_WIDTH   image_width
@@ -42,34 +43,47 @@ static char sccsid[] = "@(#)life1d.c	3.11 96/09/20 xlockmore";
 #define CELL_BITS    image_bits
 
 #include "life1d.xbm"
+#include "xlock.h"
 
-#if defined( TOTALISTIC ) && defined( LCAU )
-#undef TOTALISTIC
-#undef LCAU
-#endif
-#if !defined( TOTALISTIC ) && !defined( LCAU )
-/* Switch the following from 0 to 1 for alternate life or compile with -DLCAU */
-#if 0
-#define LCAU
-#else
-#define TOTALISTIC
-#endif
-#endif
+#define DEF_TOTALISTIC  "True"	/* FALSE is LCAU */
 
-#ifdef TOTALISTIC
-#define STATES 5
-#define RADIUS 4
-#define MAXSUM_SIZE ((STATES-1)*(RADIUS*2+1)+1)
-#else
-#define STATES 4
-#define RADIUS 1
-#define MAXSUM_SIZE 64		/* power(STATES,RADIUS*2+1) */
-#endif
+static Bool totalistic;
+static int  maxstates;
+static int  maxradius;
+static int  maxsum_size;
+
+static XrmOptionDescRec opts[] =
+{
+	{"-totalistic", ".life1d.totalistic", XrmoptionNoArg, (caddr_t) "on"},
+	{"+totalistic", ".life1d.totalistic", XrmoptionNoArg, (caddr_t) "off"}
+};
+static argtype vars[] =
+{
+{(caddr_t *) & totalistic, "totalistic", "Totalistic", DEF_TOTALISTIC, t_Bool}
+};
+static OptionStruct desc[] =
+{
+	{"-/+totalistic", "turn on/off totalistic rules (else LCAU rules)"}
+};
 
 ModeSpecOpt life1d_opts =
-{0, NULL, 0, NULL, NULL};
+{2, opts, 1, vars, desc};
 
-extern int  gridsize;
+#define MINGRIDSIZE 10
+#define MINSIZE 2		/* 3 may be better here */
+#define REPEAT 24
+#define PATTERNSIZE 8
+#define MAXSTATES 5
+
+static unsigned char patterns[MAXSTATES][PATTERNSIZE] =
+{
+	{0xee, 0xdd, 0xbb, 0x77, 0xee, 0xdd, 0xbb, 0x76},	/* dark \ stripe */
+	{0xff, 0x99, 0x99, 0xff, 0xff, 0x99, 0x99, 0xff},	/* spots */
+	{0xaa, 0xff, 0xff, 0x55, 0xaa, 0xff, 0xff, 0x55},	/* black+grey - stripe */
+	{0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff},	/* black */
+
+	{0x55, 0xaa, 0x55, 0xaa, 0x55, 0xaa, 0x55, 0xaa}	/* 50% grey */
+};
 
 static XImage logo =
 {
@@ -80,26 +94,26 @@ static XImage logo =
 };
 
 typedef struct {
+	int         init_bits;
 	int         pixelmode;
-	int         xs, ys;
-	int         xb, yb;
-	int         generation;
+	int         xs, ys, xb, yb;	/* cell size, grid border */
+	int         screen_generation, row;
 	int         nrows, ncols, border;
 	int         width, height;
 	int         k, r;
 	long        code;
-	unsigned char nextstate[MAXSUM_SIZE];
-	int         colors[STATES - 1];
-
+	int         repeating;
+	unsigned char *nextstate;
+	int         colors[MAXSTATES];
+	Pixmap      pixmaps[MAXSTATES];
+	GC          stippledGC;
 	unsigned char *newcells;
 	unsigned char *oldcells;
 	unsigned char *buffer;
 } life1dstruct;
 
 static life1dstruct *life1ds = NULL;
-static int  icon_width, icon_height;
 
-#ifdef TOTALISTIC
 static int  totalistic_rules[][3] =
 {
 
@@ -263,11 +277,10 @@ static int  totalistic_rules[][3] =
 #endif
 };
 
-#define NRULES (sizeof totalistic_rules / sizeof totalistic_rules[0])
+#define TOTALISTICRULES (sizeof totalistic_rules / sizeof totalistic_rules[0])
 
-#else /* !TOTALISTIC */
 #if 0
-unsigned char lcau21_rules[][8] =
+static unsigned char lcau21_rules[][8] =
 {
 	"00010010",		/* 18 Hollow enlarging triangles */
 	"00110110",		/* 54 Hollow triangles */
@@ -278,10 +291,10 @@ unsigned char lcau21_rules[][8] =
 	"11000001",		/* 193 Solid left triangles */
 };
 
-#define N2RULES (sizeof lcau21_rules / sizeof lcau21_rules[0])
+#define LCAU2RULES (sizeof lcau21_rules / sizeof lcau21_rules[0])
 #endif
 
-unsigned char lcau31_rules[][27] =
+static unsigned char lcau31_rules[][27] =
 {
 	"000222111000111222222111000",	/* equal thirds */
 	"002220210110110211110002200",	/* threads on triangles */
@@ -309,9 +322,9 @@ unsigned char lcau31_rules[][27] =
 	"222022001222211202022120012",	/* macrocell w/ 0122 membrane */
 };
 
-#define N3RULES (sizeof lcau31_rules / sizeof lcau31_rules[0])
+#define LCAU3RULES (sizeof lcau31_rules / sizeof lcau31_rules[0])
 
-unsigned char lcau41_rules[][64] =
+static unsigned char lcau41_rules[][64] =
 {
 	"0000000313131323232312121213232323231313131101010202020202010101",	/* skewed triangle */
 	"0000020202000000020232220203300000000101020030000100322121003020",	/* slow glider - copies bar */
@@ -344,14 +357,13 @@ unsigned char lcau41_rules[][64] =
 	"3333111122000022333311112200002233111133002222003311113300222200",	/* reversible Rule 22 */
 };
 
-#define N4RULES (sizeof lcau41_rules / sizeof lcau41_rules[0])
+#define LCAU4RULES (sizeof lcau41_rules / sizeof lcau41_rules[0])
 
-#ifdef N2RULES
-#define NRULES (N2RULES + N3RULES + N4RULES)
+#ifdef LCAU2RULES
+#define LCAURULES (LCAU2RULES + LCAU3RULES + LCAU4RULES)
 #else
-#define NRULES (N3RULES + N4RULES)
+#define LCAURULES (LCAU3RULES + LCAU4RULES)
 #endif
-#endif /* !TOTALISTIC */
 
 static void
 drawcell(ModeInfo * mi, int col, int row, unsigned int state)
@@ -369,15 +381,26 @@ drawcell(ModeInfo * mi, int col, int row, unsigned int state)
 	}
 	if (MI_NPIXELS(mi) > 2)
 		XSetForeground(display, gc, MI_PIXEL(mi, lp->colors[state - 1]));
-	else
-		XSetForeground(display, gc, MI_WIN_WHITE_PIXEL(mi));
-	if (lp->pixelmode)
-		XFillRectangle(display, window, gc,
-		lp->xb + lp->xs * col, lp->yb + lp->ys * row, lp->xs, lp->ys);
-	else
-		XPutImage(display, window, gc, &logo,
-			  0, 0, lp->xb + lp->xs * col, lp->yb + lp->ys * row,
-			  icon_width, icon_height);
+	if (lp->pixelmode | (MI_NPIXELS(mi) <= 2)) {
+		if (MI_NPIXELS(mi) <= 2) {
+			XGCValues   gcv;
+
+			gcv.stipple = lp->pixmaps[state - 1];
+			gcv.foreground = MI_WIN_WHITE_PIXEL(mi);
+			gcv.background = MI_WIN_BLACK_PIXEL(mi);
+			gcv.fill_style = FillOpaqueStippled;
+			XChangeGC(display, lp->stippledGC,
+				  GCStipple | GCFillStyle | GCForeground | GCBackground, &gcv);
+			XFillRectangle(display, window, lp->stippledGC,
+				       lp->xb + lp->xs * col, lp->yb + lp->ys * row, lp->xs, lp->ys);
+		} else
+			XFillRectangle(display, window, gc,
+				       lp->xb + lp->xs * col, lp->yb + lp->ys * row, lp->xs, lp->ys);
+	} else {
+		XPutImage(display, window, gc,
+		   &logo, 0, 0, lp->xb + lp->xs * col, lp->yb + lp->ys * row,
+			  logo.width, logo.height);
+	}
 }
 
 static void
@@ -407,53 +430,54 @@ power(int x, int n)
 static void
 GetRule(life1dstruct * lp, int i)
 {
-	long        maxsum_size, j;
+	long        sum_size, j;
 
-#ifdef TOTALISTIC
-	long        code, pow;
+	if (totalistic) {
+		long        code, pow;
 
-	lp->k = totalistic_rules[i][0];
-	lp->r = totalistic_rules[i][1];
-	maxsum_size = (lp->k - 1) * (lp->r * 2 + 1) + 1;
-	code = lp->code = totalistic_rules[i][2];
+		lp->k = totalistic_rules[i][0];
+		lp->r = totalistic_rules[i][1];
+		sum_size = (lp->k - 1) * (lp->r * 2 + 1) + 1;
+		code = lp->code = totalistic_rules[i][2];
 
-	pow = power(lp->k, maxsum_size - 1);	/* Should be less than max long */
-	for (j = 0; j < maxsum_size; j++) {
-		lp->nextstate[maxsum_size - 1 - j] = (unsigned char) (code / pow);
-		code -= (long) lp->nextstate[maxsum_size - 1 - j] * pow;
-		pow /= (long) lp->k;
-	}
-#else
-	lp->r = 1;
-#ifdef N2RULES
-	if (i < N2RULES) {
-		lp->k = 2;
-		maxsum_size = power(lp->k, 2 * lp->r + 1);
-		for (j = 0; j < maxsum_size; j++)
-			lp->nextstate[maxsum_size - 1 - j] = lcau21_rules[i][j] - '0';
-	} else if (i < N2RULES + N3RULES) {
-#else
-	if (i < N3RULES) {
-#endif
-		lp->k = 3;
-		maxsum_size = power(lp->k, 2 * lp->r + 1);
-#ifdef N2RULES
-		i -= N2RULES;
-#endif
-		for (j = 0; j < maxsum_size; j++)
-			lp->nextstate[maxsum_size - 1 - j] = lcau31_rules[i][j] - '0';
+		pow = power(lp->k, sum_size - 1);	/* Should be less than max long */
+		for (j = 0; j < sum_size; j++) {
+			lp->nextstate[sum_size - 1 - j] = (unsigned char) (code / pow);
+			code -= (long) lp->nextstate[sum_size - 1 - j] * pow;
+			pow /= (long) lp->k;
+		}
 	} else {
-		lp->k = 4;
-		maxsum_size = power(lp->k, 2 * lp->r + 1);
-#ifdef N2RULES
-		i -= (N2RULES + N3RULES);
+		lp->r = 1;
+#ifdef LCAU2RULES
+		if (i < LCAU2RULES) {
+			lp->k = 2;
+			sum_size = power(lp->k, 2 * lp->r + 1);
+			for (j = 0; j < sum_size; j++)
+				lp->nextstate[sum_size - 1 - j] = lcau21_rules[i][j] - '0';
+		} else if (i < LCAU2RULES + LCAU3RULES)
 #else
-		i -= N3RULES;
+		if (i < LCAU3RULES)
 #endif
-		for (j = 0; j < maxsum_size; j++)
-			lp->nextstate[maxsum_size - 1 - j] = lcau41_rules[i][j] - '0';
+		{
+			lp->k = 3;
+			sum_size = power(lp->k, 2 * lp->r + 1);
+#ifdef LCAU2RULES
+			i -= LCAU2RULES;
+#endif
+			for (j = 0; j < sum_size; j++)
+				lp->nextstate[sum_size - 1 - j] = lcau31_rules[i][j] - '0';
+		} else {
+			lp->k = 4;
+			sum_size = power(lp->k, 2 * lp->r + 1);
+#ifdef LCAU2RULES
+			i -= (LCAU2RULES + LCAU3RULES);
+#else
+			i -= LCAU3RULES;
+#endif
+			for (j = 0; j < sum_size; j++)
+				lp->nextstate[sum_size - 1 - j] = lcau41_rules[i][j] - '0';
+		}
 	}
-#endif
 }
 
 /* The following sometimes does not detect when there are multiple periodic
@@ -465,21 +489,14 @@ compare(ModeInfo * mi)
 	int         row, col, tryagain = False;
 	unsigned char *initl, *cmpr;
 
-	initl = lp->buffer + (lp->nrows - 1) * lp->ncols;
-	for (row = lp->nrows - 2; row >= 0; row--) {
+	initl = lp->buffer + lp->row * lp->ncols;
+	for (row = lp->row - 1; row >= 0; row--) {
 		cmpr = lp->buffer + row * lp->ncols;
 		for (col = 0; col < lp->ncols; col++) {
 			tryagain = False;
-			if (MI_NPIXELS(mi) > 2) {
-				if (*(initl + col) != *cmpr) {
-					tryagain = True;
-					break;
-				}
-			} else {
-				if ((*(initl + col) == 0) != (*cmpr == 0)) {
-					tryagain = True;
-					break;
-				}
+			if (*(initl + col) != *cmpr) {
+				tryagain = True;
+				break;
 			}
 			cmpr++;
 		}
@@ -492,6 +509,9 @@ compare(ModeInfo * mi)
 void
 init_life1d(ModeInfo * mi)
 {
+	Display    *display = MI_DISPLAY(mi);
+	Window      window = MI_WINDOW(mi);
+	int         size = MI_SIZE(mi);
 	life1dstruct *lp;
 	int         i;
 
@@ -502,16 +522,36 @@ init_life1d(ModeInfo * mi)
 	}
 	lp = &life1ds[MI_SCREEN(mi)];
 
-	lp->generation = 0;
+	lp->screen_generation = 0;
+	lp->row = 0;
+
+	if (totalistic) {
+		maxstates = MAXSTATES;
+		maxradius = 4;
+		maxsum_size = (maxstates - 1) * (maxradius * 2 + 1) + 1;
+	} else {
+		maxstates = MAXSTATES - 1;
+		maxradius = 1;
+		maxsum_size = power(maxstates, (2 * maxradius + 1));
+	}
+	if (lp->nextstate == NULL)
+		lp->nextstate = (unsigned char *) malloc(maxsum_size *
+						     sizeof (unsigned char));
 
 	if (!logo.data) {
-		icon_width = CELL_WIDTH;
-		icon_height = CELL_HEIGHT;
+		logo.width = CELL_WIDTH;
+		logo.height = CELL_HEIGHT;
 		logo.data = (char *) CELL_BITS;
-		logo.width = icon_width;
-		logo.height = icon_height;
 		logo.bytes_per_line = (logo.width + 7) / 8;
 	}
+	if (lp->init_bits == 0) {
+		XGCValues   gcv;
+
+		gcv.fill_style = FillOpaqueStippled;
+		lp->stippledGC = XCreateGC(display, window, GCFillStyle, &gcv);
+		for (i = 0; i < MAXSTATES; i++)
+			LIFE1DBITS(patterns[i], PATTERNSIZE, PATTERNSIZE);
+	}
 	if (lp->newcells != NULL)
 		(void) free((void *) lp->newcells);
 	if (lp->oldcells != NULL)
@@ -524,35 +564,54 @@ init_life1d(ModeInfo * mi)
 		lp->width = 2;
 	if (lp->height < 2)
 		lp->height = 2;
-	lp->pixelmode = (lp->width + lp->height < 8 * (icon_width + icon_height));
-	if (lp->pixelmode) {
-		lp->ncols = MIN(lp->width / 2, gridsize);
-		lp->nrows = MIN(lp->height / 2, gridsize);
+	if (size == 0 ||
+	 MINGRIDSIZE * size > lp->width || MINGRIDSIZE * size > lp->height) {
+		if (lp->width > MINGRIDSIZE * logo.width &&
+		    lp->height > MINGRIDSIZE * logo.height) {
+			lp->pixelmode = False;
+			lp->xs = logo.width;
+			lp->ys = logo.height;
+		} else {
+			lp->pixelmode = True;
+			lp->xs = lp->ys = MAX(MINSIZE, MIN(lp->width, lp->height) / MINGRIDSIZE);
+		}
 	} else {
-		lp->ncols = MIN(lp->width / icon_width, gridsize);
-		lp->nrows = MIN(lp->height / icon_height, gridsize);
+		lp->pixelmode = True;
+		if (size < -MINSIZE)
+			lp->ys = NRAND(MIN(-size, MAX(MINSIZE, MIN(lp->width, lp->height) /
+				      MINGRIDSIZE)) - MINSIZE + 1) + MINSIZE;
+		else if (size < MINSIZE)
+			lp->ys = MINSIZE;
+		else
+			lp->ys = MIN(size, MAX(MINSIZE, MIN(lp->width, lp->height) /
+					       MINGRIDSIZE));
+		lp->xs = lp->ys;
 	}
+	lp->ncols = MAX(lp->width / lp->xs, 2);
+	lp->nrows = MAX(lp->height / lp->ys, 2);
 	lp->border = (lp->nrows / 2 + 1) * MI_CYCLES(mi);
 	lp->newcells = (unsigned char *) calloc(lp->ncols + 2 * lp->border,
 						sizeof (unsigned char));
 
-	lp->oldcells = (unsigned char *) calloc(lp->ncols + 2 * (RADIUS * lp->border),
-						sizeof (unsigned char));
+	lp->oldcells = (unsigned char *) calloc(lp->ncols +
+		       2 * (maxradius + lp->border), sizeof (unsigned char));
 
 	lp->buffer = (unsigned char *) calloc(lp->ncols * lp->nrows,
 					      sizeof (unsigned char));
 
-	lp->xs = lp->width / lp->ncols;
-	lp->ys = lp->height / lp->nrows;
 	lp->xb = (lp->width - lp->xs * lp->ncols) / 2;
 	lp->yb = (lp->height - lp->ys * lp->nrows) / 2;
 
-	GetRule(lp, (int) NRAND(NRULES));
+	GetRule(lp, (int) NRAND((totalistic) ? TOTALISTICRULES : LCAURULES));
 	if (MI_WIN_IS_VERBOSE(mi)) {
 		(void) printf("colors %d, radius %d, code %ld, rule ",
 			      lp->k, lp->r, lp->code);
-		for (i = (lp->k - 1) * (lp->r * 2 + 1); i >= 0; i--)
-			(void) printf("%d", (int) lp->nextstate[i]);
+		if (totalistic)
+			for (i = (lp->k - 1) * (lp->r * 2 + 1); i >= 0; i--)
+				(void) printf("%d", (int) lp->nextstate[i]);
+		else
+			for (i = power(lp->k, (lp->r * 2 + 1)); i >= 0; i--)
+				(void) printf("%d", (int) lp->nextstate[i]);
 		(void) printf("\n");
 	}
 	if (MI_NPIXELS(mi) > 2)
@@ -560,7 +619,7 @@ init_life1d(ModeInfo * mi)
 			lp->colors[i] = (NRAND(MI_NPIXELS(mi)) + i * MI_NPIXELS(mi)) /
 				(lp->k - 1);
 	RandomSoup(lp, 40, 25);
-	(void) memcpy((char *) (lp->oldcells + RADIUS + lp->border),
+	(void) memcpy((char *) (lp->oldcells + maxradius + lp->border),
 		      (char *) (lp->newcells + lp->border), lp->ncols);
 	XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
 }
@@ -569,44 +628,84 @@ void
 draw_life1d(ModeInfo * mi)
 {
 	life1dstruct *lp = &life1ds[MI_SCREEN(mi)];
-	int         row, col;
+	Display    *display = MI_DISPLAY(mi);
+	int         col;
 
-	if (lp->generation > MI_CYCLES(mi) || (lp->generation && compare(mi)))
-		init_life1d(mi);
-
-	for (col = 0; col < lp->ncols; col++)
-		if (lp->buffer[col] != lp->newcells[col + lp->border])
-			drawcell(mi, col, 0, lp->newcells[col + lp->border]);
-	(void) memcpy((char *) lp->buffer, (char *) (lp->newcells + lp->border),
-		      lp->ncols);
-
-	for (row = 1; row < lp->nrows; row++) {
-		for (col = 0; col < lp->ncols + 2 * lp->border; col++) {
-#ifdef TOTALISTIC
-			int         sum = 0, m;
-
-			for (m = col - lp->r; m <= col + lp->r; m++)
-				sum += lp->oldcells[m + RADIUS];
-#else
-			int         sum = 0, pow = 1, m;
-
-			for (m = col + lp->r; m >= col - lp->r; m--) {
-				sum += lp->oldcells[m + RADIUS] * pow;
-				pow *= lp->k;
-			}
-#endif
-			lp->newcells[col] = lp->nextstate[sum];
-		}
-		(void) memcpy((char *) (lp->oldcells + RADIUS),
-			  (char *) lp->newcells, lp->ncols + 2 * lp->border);
+	if (lp->row == 0) {
+		lp->repeating = 0;
+		if (lp->screen_generation > MI_CYCLES(mi))
+			init_life1d(mi);
 
 		for (col = 0; col < lp->ncols; col++)
-			if (lp->buffer[col + row * lp->ncols] != lp->newcells[col + lp->border])
-				drawcell(mi, col, row, lp->newcells[col + lp->border]);
-		(void) memcpy((char *) (lp->buffer + row * lp->ncols),
+			if (lp->buffer[col] != lp->newcells[col + lp->border])
+				drawcell(mi, col, 0, lp->newcells[col + lp->border]);
+		(void) memcpy((char *) lp->buffer, (char *) (lp->newcells + lp->border),
+			      lp->ncols);
+	} else {
+		for (col = 0; col < lp->ncols + 2 * lp->border; col++) {
+			int         sum = 0, m;
+
+			if (totalistic) {
+				for (m = col - lp->r; m <= col + lp->r; m++)
+					sum += lp->oldcells[m + maxradius];
+			} else {
+				int         pow = 1;
+
+				for (m = col + lp->r; m >= col - lp->r; m--) {
+					sum += lp->oldcells[m + maxradius] * pow;
+					pow *= lp->k;
+				}
+			}
+			lp->newcells[col] = lp->nextstate[sum];
+		}
+		(void) memcpy((char *) (lp->oldcells + maxradius),
+			  (char *) lp->newcells, lp->ncols + 2 * lp->border);
+
+		for (col = 0; col < lp->ncols; col++) {
+			if (lp->buffer[col + lp->row * lp->ncols] !=
+			    lp->newcells[col + lp->border])
+				drawcell(mi, col, lp->row, lp->newcells[col + lp->border]);
+		}
+		(void) memcpy((char *) (lp->buffer + lp->row * lp->ncols),
 			    (char *) (lp->newcells + lp->border), lp->ncols);
+		{
+			int         temp = compare(mi);
+
+			if (temp)
+				lp->repeating += temp;
+			else
+				lp->repeating = 0;
+		}
+		lp->repeating += (lp->row == lp->nrows - 1) ? REPEAT * compare(mi) : 0;
+	}
+	if (lp->repeating >= 1) {
+		XGCValues   gcv;
+
+		gcv.stipple = lp->pixmaps[MAXSTATES - 1];
+		gcv.fill_style = FillStippled;
+		gcv.foreground = MI_WIN_BLACK_PIXEL(mi);
+		XChangeGC(MI_DISPLAY(mi), lp->stippledGC,
+			  GCStipple | GCFillStyle | GCForeground, &gcv);
+		XFillRectangle(display, MI_WINDOW(mi), lp->stippledGC,
+			       0, lp->yb + lp->ys * lp->row,
+			       lp->width, lp->ys);
+	}
+	lp->row++;
+	if (lp->repeating >= REPEAT) {
+		if (lp->row < lp->nrows) {
+			XSetForeground(display, MI_GC(mi), MI_WIN_BLACK_PIXEL(mi));
+			XFillRectangle(display, MI_WINDOW(mi), MI_GC(mi),
+				       0, lp->yb + lp->ys * lp->row,
+			  lp->width, lp->height - lp->ys * lp->row - lp->yb);
+		}
+		lp->screen_generation = MI_CYCLES(mi);
+		lp->row = lp->nrows;
+	}
+	if (lp->row >= lp->nrows) {
+		lp->screen_generation++;
+		MI_PAUSE(mi) = 2000000;
+		lp->row = 0;
 	}
-	lp->generation++;
 }
 
 void
@@ -617,13 +716,23 @@ release_life1d(ModeInfo * mi)
 
 		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++) {
 			life1dstruct *lp = &life1ds[screen];
+			int         shade;
 
+			if (lp->stippledGC != NULL) {
+				XFreeGC(MI_DISPLAY(mi), lp->stippledGC);
+			}
+			if (lp->init_bits != 0) {
+				for (shade = 0; shade < MAXSTATES; shade++)
+					XFreePixmap(MI_DISPLAY(mi), lp->pixmaps[shade]);
+			}
 			if (lp->newcells != NULL)
 				(void) free((void *) lp->newcells);
 			if (lp->oldcells != NULL)
 				(void) free((void *) lp->oldcells);
 			if (lp->buffer != NULL)
 				(void) free((void *) lp->buffer);
+			if (lp->nextstate != NULL)
+				(void) free((void *) lp->nextstate);
 		}
 		(void) free((void *) life1ds);
 		life1ds = NULL;
@@ -635,10 +744,10 @@ refresh_life1d(ModeInfo * mi)
 {
 	life1dstruct *lp = &life1ds[MI_SCREEN(mi)];
 	int         row, col, nrow;
-  
+
 	for (row = 0; row < lp->nrows; row++) {
-    nrow = row * lp->ncols;
+		nrow = row * lp->ncols;
 		for (col = 0; col < lp->ncols; col++)
-				drawcell(mi, col, row, lp->buffer[col + nrow]);
-  }
+			drawcell(mi, col, row, lp->buffer[col + nrow]);
+	}
 }
