@@ -1,6 +1,5 @@
-
-#ifndef lint
-static char sccsid[] = "@(#)life3d.c	3.11 96/09/20 xlockmore";
+#if !defined( lint ) && !defined( SABER )
+static char sccsid[] = "@(#)life3d.c	4.00 97/01/01 xlockmore";
 
 #endif
 
@@ -30,7 +29,6 @@ static char sccsid[] = "@(#)life3d.c	3.11 96/09/20 xlockmore";
  *   with (latest?) update from 87/2/1
  */
 
-#include <math.h>
 #include "xlock.h"
 
 #define ON 0x40
@@ -72,7 +70,10 @@ static char sccsid[] = "@(#)life3d.c	3.11 96/09/20 xlockmore";
 #define WHITE 4
 #define NUMPTS 42
 
-#define DEF_RULE3D  "4555"
+/* #define DEF_RULE3D  "0" */
+#define DEF_RULE3D  "1"
+/* #define DEF_RULE3D  "4555" */
+/* #define DEF_RULE3D  "5766" */
 
 static int  rule3d;
 
@@ -86,7 +87,7 @@ static argtype vars[] =
 };
 static OptionStruct desc[] =
 {
-     {"-rule3d", "<living_min><living_max><birth_min><birth_max> parameters"}
+ {"-rule3d num", "<living_min><living_max><birth_min><birth_max> parameters"}
 };
 
 ModeSpecOpt life3d_opts =
@@ -102,8 +103,8 @@ typedef struct _CellList {
 } CellList;
 
 typedef struct {
- int  living_min, living_max, birth_min, birth_max;
-  int         pattern, patterned_rule;
+	int         living_min, living_max, birth_min, birth_max;
+	int         pattern, patterned_rule;
 	int         rule3d;
 	int         initialized;
 	int         wireframe;	/* FALSE */
@@ -1132,28 +1133,29 @@ static int  patterns_6767[][3 * NUMPTS + 1] =
 	}
 };
 
-static int npatterns[] = {
-      (sizeof patterns_4555 / sizeof patterns_4555[0]),
-      (sizeof patterns_5766 / sizeof patterns_5766[0]),
-      (sizeof patterns_5655 / sizeof patterns_5655[0]),
-      (sizeof patterns_6767 / sizeof patterns_6767[0]),
+static int  npatterns[] =
+{
+	(sizeof patterns_4555 / sizeof patterns_4555[0]),
+	(sizeof patterns_5766 / sizeof patterns_5766[0]),
+	(sizeof patterns_5655 / sizeof patterns_5655[0]),
+	(sizeof patterns_6767 / sizeof patterns_6767[0]),
 };
 
 static int
 Rule2Array(int rule)
 {
-		switch (rule) {
-			case 4555:
-				return LIFE_4555;
-			case 5766:
-				return LIFE_5766;
-			case 5655:
-				return LIFE_5655;
-			case 6767:
-				return LIFE_6767;
-   default:
-     return LIFE_RULES;
-}
+	switch (rule) {
+		case 4555:
+			return LIFE_4555;
+		case 5766:
+			return LIFE_5766;
+		case 5655:
+			return LIFE_5655;
+		case 6767:
+			return LIFE_6767;
+		default:
+			return LIFE_RULES;
+	}
 }
 
 static int
@@ -1161,16 +1163,16 @@ Array2Rule(int array)
 {
 	switch (array) {
 		case LIFE_4555:
-		return 4555;
-	case LIFE_5766:
-		return 5766;
-	case LIFE_5655:
-				return 5655;
-	case LIFE_6767:
-		return 6767;
-   default:
-     return 0;
-   }
+			return 4555;
+		case LIFE_5766:
+			return 5766;
+		case LIFE_5655:
+			return 5655;
+		case LIFE_6767:
+			return 6767;
+		default:
+			return 0;
+	}
 }
 
 /*--- list ---*/
@@ -1271,9 +1273,9 @@ BaseOffset(life3dstruct * lp,
 	*b = ((x & 0x7c) << 7) + ((y & 0x7c) << 2) + ((z & 0x7c) >> 2);
 	*o = (x & 3) + ((y & 3) << 2) + ((z & 3) << 4);
 
-	/* Purify thinks there is an memory leak here, looks fine to me, IMHO
-	   bagleyd */
 	if (!lp->base[*b])
+/* PURIFY 3.2 on Solaris2 and PURIFY 4.0 on SunOS4 reports a memory leak on
+   the next line. I think its faked out */
 		lp->base[*b] = (unsigned char *) calloc(64, sizeof (unsigned char));
 }
 
@@ -1479,18 +1481,18 @@ GetPattern(ModeInfo * mi, int pattern_rule, int pattern)
 
 	switch (pattern_rule) {
 		case LIFE_4555:
-      patptr = &patterns_4555[pattern][0];
-		  break;
-	case LIFE_5766:
-      patptr = &patterns_5766[pattern][0];
-		  break;
-	case LIFE_5655:
-      patptr = &patterns_5655[pattern][0];
-		  break;
-	case LIFE_6767:
-      patptr = &patterns_6767[pattern][0];
-		  break;
-  }
+			patptr = &patterns_4555[pattern][0];
+			break;
+		case LIFE_5766:
+			patptr = &patterns_5766[pattern][0];
+			break;
+		case LIFE_5655:
+			patptr = &patterns_5655[pattern][0];
+			break;
+		case LIFE_6767:
+			patptr = &patterns_6767[pattern][0];
+			break;
+	}
 	while ((x = *patptr++) != 127) {
 		y = *patptr++;
 		z = *patptr++;
@@ -1891,17 +1893,17 @@ shooter(life3dstruct * lp)
 	}
 }
 
-void
+static void
 initrule(ModeInfo * mi)
 {
 	life3dstruct *lp = &life3ds[MI_SCREEN(mi)];
 
-		   if (MI_WIN_IS_VERBOSE(mi))
-			  (void) printf("rule3d %d\n", lp->rule3d);
-		   lp->living_min = lp->rule3d / 1000;
-		   lp->living_max = (lp->rule3d / 100) % 10;
-		   lp->birth_min = (lp->rule3d / 10) % 10;
-		   lp->birth_max = lp->rule3d % 10;
+	if (MI_WIN_IS_VERBOSE(mi))
+		(void) printf("rule3d %d\n", lp->rule3d);
+	lp->living_min = lp->rule3d / 1000;
+	lp->living_max = (lp->rule3d / 100) % 10;
+	lp->birth_min = (lp->rule3d / 10) % 10;
+	lp->birth_max = lp->rule3d % 10;
 }
 
 void
@@ -1933,7 +1935,7 @@ init_life3d(ModeInfo * mi)
 		 * (not 6) cells around them.   These rules are not general enough
 		 * to allow for that.
 		 */
-    initrule(mi);
+		initrule(mi);
 	}
 	if (!lp->initialized) {
 		lp->dist = 50.0 /*30.0 */ ;
@@ -1981,11 +1983,11 @@ init_life3d(ModeInfo * mi)
 	lp->vz = (sin(lp->alt * IP) * lp->dist);
 	NewViewpoint(lp, lp->vx, lp->vy, lp->vz);
 
-  pattern_rule = Rule2Array(lp->rule3d);
-  if (pattern_rule < LIFE_RULES)
-    npats = npatterns[pattern_rule];
-  else
-    npats = 0;
+	pattern_rule = Rule2Array(lp->rule3d);
+	if (pattern_rule < LIFE_RULES)
+		npats = npatterns[pattern_rule];
+	else
+		npats = 0;
 	lp->pattern = NRAND(npats + 2);
 	if (lp->pattern >= npats)
 		RandomSoup(mi, 30, 10);
@@ -2006,8 +2008,7 @@ draw_life3d(ModeInfo * mi)
 	if (++lp->generation > MI_CYCLES(mi) || !lp->visible) {
 		/*CountCells3D(lp) == 0) */
 		init_life3d(mi);
-  }
-
+	}
 	/*
 	 * generate a randomized shooter aimed roughly toward the center of the
 	 * screen after batchcount.
@@ -2043,38 +2044,38 @@ refresh_life3d(ModeInfo * mi)
 void
 change_life3d(ModeInfo * mi)
 {
-  life3dstruct *lp = &life3ds[MI_SCREEN(mi)];
-  int         npats, pattern_rule;
+	life3dstruct *lp = &life3ds[MI_SCREEN(mi)];
+	int         npats, pattern_rule;
 
-  lp->generation = 0;
+	lp->generation = 0;
 
-		End3D(lp);
+	End3D(lp);
 	lp->memstart = 1;
 	/*lp->tablesMade = 0; */
 
 	XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
 
-  lp->pattern = lp->pattern + 1;
-  pattern_rule = Rule2Array(lp->rule3d);
-  if (pattern_rule < LIFE_RULES)
-    npats = npatterns[pattern_rule];
-  else
-    npats = 0;
-  if (lp->pattern >= npats + 2) {
-     lp->pattern = 0;
-     if (rule3d == 0) {
-       pattern_rule++;
-       if (pattern_rule >= LIFE_RULES)
-         pattern_rule = 0;
-       lp->rule3d = Array2Rule(pattern_rule);
-       initrule(mi);
-     } else if (rule3d == 1) {
-       pattern_rule++;
-       if (pattern_rule >= LIFE_GLIDERS)
-         pattern_rule = 0;
-       lp->rule3d = Array2Rule(pattern_rule);
-       initrule(mi);
-    }
+	lp->pattern = lp->pattern + 1;
+	pattern_rule = Rule2Array(lp->rule3d);
+	if (pattern_rule < LIFE_RULES)
+		npats = npatterns[pattern_rule];
+	else
+		npats = 0;
+	if (lp->pattern >= npats + 2) {
+		lp->pattern = 0;
+		if (rule3d == 0) {
+			pattern_rule++;
+			if (pattern_rule >= LIFE_RULES)
+				pattern_rule = 0;
+			lp->rule3d = Array2Rule(pattern_rule);
+			initrule(mi);
+		} else if (rule3d == 1) {
+			pattern_rule++;
+			if (pattern_rule >= LIFE_GLIDERS)
+				pattern_rule = 0;
+			lp->rule3d = Array2Rule(pattern_rule);
+			initrule(mi);
+		}
 	}
 	if (lp->pattern >= npats)
 		RandomSoup(mi, 30, 10);
