@@ -1,6 +1,5 @@
-
-#ifndef lint
-static char sccsid[] = "@(#)logout.c	3.11 96/09/20 xlockmore";
+#if !defined( lint ) && !defined( SABER )
+static const char sccsid[] = "@(#)logout.c	4.00 97/01/01 xlockmore";
 
 #endif
 
@@ -33,15 +32,16 @@ static char sccsid[] = "@(#)logout.c	3.11 96/09/20 xlockmore";
  *            Liverpool Computer Science
  */
 
-#if defined( AUTO_LOGOUT ) || defined( LOGOUT_BUTTON )
-
 #include "xlock.h"
-#ifdef SYSLOG
+
+#if defined( USE_AUTO_LOGOUT ) || defined( USE_LOGOUT_BUTTON ) || defined( USE_BOMB )
+
+#if ( HAVE_SYSLOG_H && defined( USE_SYSLOG ))
 #include <syslog.h>
 #endif
 #include <sys/signal.h>
 
-extern Bool inroot, inwindow, nolock;
+extern Bool inroot, inwindow, nolock, debug;
 extern char *ProgramName;
 
 /* 
@@ -64,182 +64,17 @@ extern char *ProgramName;
  *
  * If you can't log the user out (and you don't want to use the brute
  * force method) simply return from logoutUser(), and xlock will continue
- * on it's merry way (only applies if AUTO_LOGOUT or LOGOUT_BUTTON is
- * defined.)
+ * on it's merry way (only applies if USE_AUTO_LOGOUT or USE_LOGOUT_BUTTON
+ * is defined.)
  */
 
 #define NAP_TIME        5	/* Sleep between shutdown attempts */
 
-void
-ugly_logout(void)
-{
-
-#ifndef KILL_ALL_OTHERS
-#define KILL_ALL_OTHERS -1
-	(void) signal(SIGHUP, SIG_IGN);
-	(void) signal(SIGTERM, SIG_IGN);
-#endif
-
-	(void) kill(KILL_ALL_OTHERS, SIGHUP);
-	(void) sleep(NAP_TIME);
-	(void) kill(KILL_ALL_OTHERS, SIGTERM);
-	(void) sleep(NAP_TIME);
-
-#ifdef SYSLOG
-	syslog(LOG_NOTICE, "xlock: failed to exit - sending kill (uid %d)\n",
-	       getuid());
-#endif
-
-	(void) kill(KILL_ALL_OTHERS, SIGKILL);
-	(void) sleep(NAP_TIME);
-
-#ifdef SYSLOG
-	syslog(LOG_WARNING, "xlock: still won't exit - suicide (uid %d)\n",
-	       getuid());
-#endif
-
-	(void) kill(getpid(), SIGKILL);
-	exit(-1);
-
-}
 
 #ifdef CLOSEDOWN_LOGOUT
 
 /* Logout the user by contacting the display and closeing all windows */
 
-static void closedown();	/* the closedown function from xclosedown.c */
-
-void
-logoutUser(void)
-{
-#ifdef SYSLOG
-	syslog(LOG_INFO, "xlock: expired. closing down %s\n", getenv("DISPLAY"));
-#endif
-
-	closedown();
-
-/* ugly_logout(); */
-	exit(-1);
-}
-
-#else /* !CLOSEDOWN_LOGOUT */
-
-#ifdef SESSION_LOGOUT
-
-/* sample logoutUser (described above) */
-
-void
-logoutUser(void)
-{
-	char       *pidstr;
-	int         pid;
-
-#ifdef SYSLOG
-	syslog(LOG_INFO, "xlock: expired. closing down (uid %d) on %s\n",
-	       getuid(), getenv("DISPLAY"));
-#endif
-
-	pidstr = getenv("XSESSION");
-	if (pidstr) {
-		pid = atoi(pidstr);
-		kill(pid, SIGTERM);
-		(void) sleep(5);
-	}
-/* ugly_logout(); */
-	(void) sleep(1);
-}
-
-#else
-
-void
-logoutUser(void)
-{
-
-#ifdef SYSLOG
-	syslog(LOG_INFO, "xlock: expired. closing down (uid %d) on %s\n",
-	       getuid(), getenv("DISPLAY"));
-#endif
-
-	ugly_logout();
-}
-
-#endif /* SESSION_LOGOUT */
-#endif /* CLOSEDOWN_LOGOUT */
-
-
-	/*
-	 *  Determine whether to "fully" lock the terminal, or
-	 *    whether the time-limited version should be imposed.
-	 *
-	 *  Policy:
-	 *      Members of staff can fully lock
-	 *        (hard-wired user/group names + file read at run time)
-	 *      Students (i.e. everyone else)
-	 *          are forced to use the time-limit version.
-	 *
-	 *  An alternative policy could be based on display location
-	 */
-#define FULL_LOCK       1
-#define TEMP_LOCK       0
-
-/* assuming only staff file is needed */
-#ifndef STAFF_FILE
-#define STAFF_FILE      "/usr/remote/etc/xlock.staff"
-#endif
-
-#undef passwd
-#undef pw_name
-#undef getpwnam
-#undef getpwuid
-#include <pwd.h>
-#include <grp.h>
-#include <sys/param.h>
-#include <errno.h>
-#ifndef NGROUPS
-#include <limits.h>		/* or <sys/param.h> */
-#define NGROUPS NGROUPS_MAX
-#endif
-
-int
-fullLock(void)
-{
-	uid_t       uid;
-	int         ngroups = NGROUPS;
-	gid_t       mygidset[NGROUPS];
-	int         ngrps, i;
-	struct passwd *pwp;
-	struct group *gp;
-	FILE       *fp;
-	char        buf[BUFSIZ];
-
-	if ((uid = getuid()) == 0)
-		return (FULL_LOCK);	/* root */
-	if (inwindow || inroot || nolock)
-		return (FULL_LOCK);	/* (mostly) harmless user */
-
-	pwp = getpwuid(uid);
-	if ((ngrps = getgroups(ngroups, mygidset)) == -1)
-		perror(ProgramName);
-
-	if ((fp = fopen(STAFF_FILE, "r")) == NULL)
-		return (TEMP_LOCK);
-
-	while ((fgets(buf, BUFSIZ, fp)) != NULL) {
-		char       *cp;
-
-		if ((cp = (char *) strchr(buf, '\n')) != NULL)
-			*cp = '\0';
-		if (!strcmp(buf, pwp->pw_name))
-			return (FULL_LOCK);
-		if ((gp = getgrnam(buf)) != NULL)	/* check all of user's groups */
-			for (i = 0; i < ngrps; ++i)
-				if (gp->gr_gid == mygidset[i])
-					return (FULL_LOCK);
-	}
-
-	return (TEMP_LOCK);
-}
-
 
 /* 
  *  Window shutdown program by Janet L. Carson, Baylor College of Medicine.
@@ -444,7 +279,7 @@ kill_tree(Window w)
  */
 
 static void
-closedown(void)
+closedownLogout(void)
 {
 	int         j;
 	extern Display *dsp;
@@ -471,13 +306,179 @@ closedown(void)
 
 	/* wait for things to clean themselves up */
 
-	(void) sleep(5);
+	(void) sleep(NAP_TIME);
 
 	/* this will forcibly kill anything that's still around --
 	   this second pass may or may not be needed... */
 
 	for (j = 0; j < ScreenCount(d); j++)
 		kill_tree(RootWindow(d, j));
+	(void) sleep(NAP_TIME);
 }
 
+#endif /* CLOSEDOWN_LOGOUT */
+
+#ifdef SESSION_LOGOUT
+static void
+sessionLogout(void)
+{
+	char       *pidstr;
+
+	pidstr = getenv("XSESSION");
+	if (pidstr) {
+		kill(atoi(pidstr), SIGTERM);
+		(void) sleep(NAP_TIME);
+	}
+}
+
+#endif /* SESSION_LOGOUT */
+
+static void
+uglyLogout(void)
+{
+#ifdef VMS
+	(void) finish();
+	(void) system("mcr decw$endsession -noprompt");
+#else
+#ifndef KILL_ALL_OTHERS
+#define KILL_ALL_OTHERS -1
+	(void) signal(SIGHUP, SIG_IGN);
+	(void) signal(SIGTERM, SIG_IGN);
 #endif
+
+	(void) kill(KILL_ALL_OTHERS, SIGHUP);
+	(void) sleep(NAP_TIME);
+	(void) kill(KILL_ALL_OTHERS, SIGTERM);
+	(void) sleep(NAP_TIME);
+
+#if ( HAVE_SYSLOG_H && defined( USE_SYSLOG ))
+	syslog(LOG_NOTICE, "xlock: failed to exit - sending kill (uid %d)\n",
+	       getuid());
+#endif
+
+	(void) kill(KILL_ALL_OTHERS, SIGKILL);
+	(void) sleep(NAP_TIME);
+
+#if ( HAVE_SYSLOG_H && defined( USE_SYSLOG ))
+	syslog(LOG_WARNING, "xlock: still won't exit - suicide (uid %d)\n",
+	       getuid());
+#endif
+
+	(void) kill(getpid(), SIGKILL);
+#endif
+	exit(-1);
+}
+
+void
+logoutUser(void)
+{
+#ifdef SYSLOG
+	syslog(LOG_INFO, "xlock: expired. closing down (uid %d) on %s\n",
+	       getuid(), getenv("DISPLAY"));
+#endif
+
+#ifdef _SGI_SOURCE /* What is a good switch for SGI/IRIX */
+	(void) finish();
+	(void) system("/usr/bin/X11/tellwm end_session >/dev/null 2>&1");
+	(void) sleep(10); /* Give the above a chance to run */
+#endif
+
+#ifdef CLOSEDOWN_LOGOUT
+	closedownLogout();
+#endif
+#ifdef SESSION_LOGOUT
+	sessionLogout();
+#endif
+	uglyLogout();
+	exit(-1);
+}
+
+
+#if defined( USE_AUTO_LOGOUT ) || defined( USE_LOGOUT_BUTTON )
+	/*
+	 *  Determine whether to "fully" lock the terminal, or
+	 *    whether the time-limited version should be imposed.
+	 *
+	 *  Policy:
+	 *      Members of staff can fully lock
+	 *        (hard-wired user/group names + file read at run time)
+	 *      Students (i.e. everyone else)
+	 *          are forced to use the time-limit version.
+	 *
+	 *  An alternative policy could be based on display location
+	 */
+#define FULL_LOCK       1
+#define TEMP_LOCK       0
+
+/* assuming only staff file is needed */
+#ifndef STAFF_FILE
+#define STAFF_FILE      "/usr/remote/etc/xlock.staff"
+#endif
+
+#undef passwd
+#undef pw_name
+#undef getpwnam
+#undef getpwuid
+#include <pwd.h>
+#include <grp.h>
+#include <sys/param.h>
+#include <errno.h>
+#ifndef NGROUPS
+#if HAVE_LIMITS_H
+#include <limits.h>
+#endif
+#define NGROUPS NGROUPS_MAX
+#endif
+
+int
+fullLock(void)
+{
+	uid_t       uid;
+	int         ngroups = NGROUPS;
+	gid_t       mygidset[NGROUPS];
+	int         ngrps, i;
+	struct passwd *pwp;
+	struct group *gp;
+	FILE       *fp;
+	char        buf[BUFSIZ];
+
+	uid = getuid();
+	/* Do not try to logout root! */
+	if (!uid)
+		return (FULL_LOCK);	/* root */
+	if (inwindow || inroot || nolock || debug)
+		return (FULL_LOCK);	/* (mostly) harmless user */
+
+/* PURIFY 3.2 on Solaris2 and PURIFY 4.0 on SunOS4 reports an uninitialized
+   memory read on the next line */
+	pwp = getpwuid(uid);
+	if ((ngrps = getgroups(ngroups, mygidset)) == -1)
+		perror(ProgramName);
+
+#ifdef STAFF_NETGROUP
+	if (innetgr(STAFF_NETGROUP, NULL, pwp->pw_name, NULL))
+		return (FULL_LOCK);
+#endif
+
+	if ((fp = fopen(STAFF_FILE, "r")) == NULL)
+		return (TEMP_LOCK);
+
+	while ((fgets(buf, BUFSIZ, fp)) != NULL) {
+		char       *cp;
+
+		if ((cp = (char *) strchr(buf, '\n')) != NULL)
+			*cp = '\0';
+		if (!strcmp(buf, pwp->pw_name))
+			return (FULL_LOCK);
+		if ((gp = getgrnam(buf)) != NULL)	/* check all of user's groups */
+			for (i = 0; i < ngrps; ++i)
+				if (gp->gr_gid == mygidset[i])
+					return (FULL_LOCK);
+	}
+	(void) fclose(fp);
+
+	return (TEMP_LOCK);
+}
+#endif /* USE_AUTO_LOGOUT || USE_LOGOUT_BUTTON */
+
+#endif /* USE_AUTO_LOGOUT || USE_LOGOUT_BUTTON || USE_BOMB */
