@@ -3,7 +3,7 @@
 #define __XLOCK_MODE_H__
 
 /*-
- * @(#)mode.h	3.11 96/09/20 xlockmore
+ * @(#)mode.h	4.00 97/01/01 xlockmore
  *
  * mode.h - mode management for xlock, the X Window System lockscreen.
  *
@@ -13,8 +13,8 @@
  *
  * Revision History:
  *
- * Changes of David Bagley (bagleyd@hertz.njit.edu)
- * 18-Mar-96: Ron Hitchens <ronh@utw.com>
+ * Changes of David Bagley (bagleyd@bigfoot.com)
+ * 18-Mar-96: Ron Hitchens <ron@idiom.com>
  *		Extensive revision to define new data types for
  *		the new mode calling scheme.
  * 02-Jun-95: Extracted out of resource.c.
@@ -27,29 +27,12 @@
 
 /* -------------------------------------------------------------------- */
 
-/* 
- * The xlock code attempts to be K&R compatible to support older
- * systems.  But if the compiler advertises ANSI capability by
- * defining __STDC__, we'll declare prototypes here.  This gives us
- * type checking for free of the hook procs in each mode.  Most
- * compilers will type-check properly if the prototype exists,
- * even though the function itself may use the old K&R argument
- * style.
- */
-
-#ifdef __STDC__
 struct LockStruct_s;
 struct ModeInfo_s;
 
 typedef void (ModeHook) (struct ModeInfo_s *);
 typedef void (HookProc) (struct LockStruct_s *, struct ModeInfo_s *);
 
-#else
-typedef void (ModeHook) ();
-typedef void (HookProc) ();
-
-#endif
-
 typedef struct LockStruct_s {
 	char       *cmdline_arg;	/* mode name */
 	ModeHook   *init_hook;	/* func to init a mode */
@@ -62,6 +45,7 @@ typedef struct LockStruct_s {
 	int         def_delay;	/* default delay for mode */
 	int         def_batchcount;
 	int         def_cycles;
+	int         def_size;
 	float       def_saturation;
 	char       *desc;	/* text description of mode */
 	unsigned int flags;	/* state flags for this mode */
@@ -82,9 +66,11 @@ typedef struct {
 	int         win_height;	/* height of current window */
 	int         win_depth;	/* depth of current window */
 	Visual     *visual;	/* visual of current window */
+	Colormap    colormap;	/* default colormap of current window */
 	unsigned long black_pixel;	/* pixel value for black */
 	unsigned long white_pixel;	/* pixel value for white */
 	unsigned int flags;	/* xlock window flags */
+	float       delta3d;
 } WindowInfo;
 
 #define WI_FLAG_INFO_INITTED	0x001	/* private state flag */
@@ -100,11 +86,11 @@ typedef struct {
 #define WI_FLAG_JUST_INITTED	0x400	/* private state flag */
 
 typedef struct {
-	float       delta3d;
 	long        pause;	/* output, set by mode */
 	long        delay;	/* inputs, current settings */
 	long        batchcount;
 	long        cycles;
+	long        size;
 	float       saturation;
 } RunInfo;
 
@@ -143,8 +129,10 @@ typedef struct ModeInfo_s {
 #define MI_WIN_HEIGHT(mi)	((mi)->windowinfo.win_height)
 #define MI_WIN_DEPTH(mi)	((mi)->windowinfo.win_depth)
 #define MI_VISUAL(mi)	((mi)->windowinfo.visual)
+#define MI_COLORMAP(mi)	((mi)->windowinfo.colormap)
 #define MI_WIN_BLACK_PIXEL(mi)	((mi)->windowinfo.black_pixel)
 #define MI_WIN_WHITE_PIXEL(mi)	((mi)->windowinfo.white_pixel)
+#define MI_DELTA3D(mi)	((mi)->windowinfo.delta3d)
 #define MI_WIN_FLAGS(mi)	((mi)->windowinfo.flags)
 #define MI_WIN_SET_FLAG_STATE(mi,f,bool) ((mi)->windowinfo.flags = \
 					(bool) ? (mi)->windowinfo.flags | f \
@@ -175,17 +163,18 @@ typedef struct ModeInfo_s {
 #define MI_RIGHT_COLOR(mi)	((mi)->screeninfo->rightcol)
 #define MI_LEFT_COLOR(mi)	((mi)->screeninfo->leftcol)
 
-#define MI_DELTA3D(mi)	((mi)->runinfo.delta3d)
 #define MI_PAUSE(mi)		((mi)->runinfo.pause)
 #define MI_DELAY(mi)		((mi)->runinfo.delay)
 #define MI_BATCHCOUNT(mi)	((mi)->runinfo.batchcount)
 #define MI_CYCLES(mi)		((mi)->runinfo.cycles)
+#define MI_SIZE(mi)		((mi)->runinfo.size)
 #define MI_SATURATION(mi)	((mi)->runinfo.saturation)
 
 #define MI_LOCKSTRUCT(mi)	((mi)->lockstruct)
 #define MI_DEFDELAY(mi)		((mi)->lockstruct->def_delay)
 #define MI_DEFBATCHCOUNT(mi)	((mi)->lockstruct->def_batchcount)
 #define MI_DEFCYCLES(mi)	((mi)->lockstruct->def_cycles)
+#define MI_DEFSIZE(mi)	((mi)->lockstruct->def_size)
 #define MI_DEFSATURATION(mi)	((mi)->lockstruct->def_saturation)
 
 #define MI_NAME(mi)		((mi)->lockstruct->cmdline_arg)
@@ -199,16 +188,9 @@ extern HookProc call_callback_hook;
 extern HookProc call_refresh_hook;
 extern HookProc call_change_hook;
 
-#ifdef __STDC__
 extern void set_default_mode(LockStruct *);
 extern void release_last_mode(ModeInfo *);
 
-#else
-extern void set_default_mode();
-extern void release_last_mode();
-
-#endif
-
 /* -------------------------------------------------------------------- */
 
 extern ModeHook init_ant;
@@ -217,6 +199,12 @@ extern ModeHook release_ant;
 extern ModeHook refresh_ant;
 extern ModeSpecOpt ant_opts;
 
+extern ModeHook init_ball;
+extern ModeHook draw_ball;
+extern ModeHook release_ball;
+extern ModeHook refresh_ball;
+extern ModeSpecOpt ball_opts;
+
 extern ModeHook init_bat;
 extern ModeHook draw_bat;
 extern ModeHook release_bat;
@@ -253,6 +241,14 @@ extern ModeHook release_bug;
 extern ModeHook refresh_bug;
 extern ModeSpecOpt bug_opts;
 
+#if defined( USE_XPM ) || defined( USE_XPMINC )
+extern ModeHook init_cartoon;
+extern ModeHook draw_cartoon;
+extern ModeHook release_cartoon;
+extern ModeSpecOpt cartoon_opts;
+
+#endif
+
 extern ModeHook init_clock;
 extern ModeHook draw_clock;
 extern ModeHook release_clock;
@@ -265,12 +261,24 @@ extern ModeHook release_daisy;
 extern ModeHook refresh_daisy;
 extern ModeSpecOpt daisy_opts;
 
+extern ModeHook init_dclock;
+extern ModeHook draw_dclock;
+extern ModeHook release_dclock;
+extern ModeHook refresh_dclock;
+extern ModeSpecOpt dclock_opts;
+
 extern ModeHook init_demon;
 extern ModeHook draw_demon;
 extern ModeHook release_demon;
 extern ModeHook refresh_demon;
 extern ModeSpecOpt demon_opts;
 
+extern ModeHook init_drift;
+extern ModeHook draw_drift;
+extern ModeHook release_drift;
+extern ModeHook refresh_drift;
+extern ModeSpecOpt drift_opts;
+
 extern ModeHook init_eyes;
 extern ModeHook draw_eyes;
 extern ModeHook release_eyes;
@@ -302,6 +310,14 @@ extern ModeHook release_galaxy;
 extern ModeHook refresh_galaxy;
 extern ModeSpecOpt galaxy_opts;
 
+#ifdef USE_GL
+extern ModeHook init_gear;
+extern ModeHook draw_gear;
+extern ModeHook release_gear;
+extern ModeSpecOpt gear_opts;
+
+#endif
+
 extern ModeHook init_geometry;
 extern ModeHook draw_geometry;
 extern ModeHook release_geometry;
@@ -338,6 +354,12 @@ extern ModeHook release_image;
 extern ModeHook refresh_image;
 extern ModeSpecOpt image_opts;
 
+extern ModeHook init_julia;
+extern ModeHook draw_julia;
+extern ModeHook release_julia;
+extern ModeHook refresh_julia;
+extern ModeSpecOpt julia_opts;
+
 extern ModeHook init_kaleid;
 extern ModeHook draw_kaleid;
 extern ModeHook release_kaleid;
@@ -382,6 +404,12 @@ extern ModeHook release_lissie;
 extern ModeHook refresh_lissie;
 extern ModeSpecOpt lissie_opts;
 
+extern ModeHook init_loop;
+extern ModeHook draw_loop;
+extern ModeHook release_loop;
+extern ModeHook refresh_loop;
+extern ModeSpecOpt loop_opts;
+
 extern ModeHook init_marquee;
 extern ModeHook draw_marquee;
 extern ModeHook release_marquee;
@@ -406,11 +434,20 @@ extern ModeHook release_nose;
 extern ModeHook refresh_nose;
 extern ModeSpecOpt nose_opts;
 
+extern ModeHook init_pacman;
+extern ModeHook draw_pacman;
+extern ModeHook release_pacman;
+extern ModeHook refresh_pacman;
+extern ModeSpecOpt pacman_opts;
+
 extern ModeHook init_penrose;
 extern ModeHook draw_penrose;
 extern ModeHook release_penrose;
 
-				     /* extern ModeHook refresh_penrose; *//* Needed */
+#if 0
+extern ModeHook refresh_penrose;	/* Needed */
+
+#endif
 extern ModeSpecOpt penrose_opts;
 
 extern ModeHook init_petal;
@@ -423,7 +460,10 @@ extern ModeHook init_puzzle;
 extern ModeHook draw_puzzle;
 extern ModeHook release_puzzle;
 
-				    /* extern ModeHook refresh_puzzle; *//* Needed, lots of special cases */
+#if 0
+extern ModeHook refresh_puzzle;	/* Needed */
+
+#endif
 extern ModeSpecOpt puzzle_opts;
 
 extern ModeHook init_pyro;
@@ -438,11 +478,11 @@ extern ModeHook release_qix;
 extern ModeHook refresh_qix;
 extern ModeSpecOpt qix_opts;
 
-extern ModeHook init_rock;
-extern ModeHook draw_rock;
-extern ModeHook release_rock;
-extern ModeHook refresh_rock;
-extern ModeSpecOpt rock_opts;
+extern ModeHook init_roll;
+extern ModeHook draw_roll;
+extern ModeHook release_roll;
+extern ModeHook refresh_roll;
+extern ModeSpecOpt roll_opts;
 
 extern ModeHook init_rotor;
 extern ModeHook draw_rotor;
@@ -460,7 +500,10 @@ extern ModeHook init_slip;
 extern ModeHook draw_slip;
 extern ModeHook release_slip;
 
-				  /* extern ModeHook refresh_slip; *//* Probably not practical */
+#if 0
+extern ModeHook refresh_slip;	/* Probably not practical */
+
+#endif
 extern ModeSpecOpt slip_opts;
 
 extern ModeHook init_sphere;
@@ -481,6 +524,12 @@ extern ModeHook release_spline;
 extern ModeHook refresh_spline;
 extern ModeSpecOpt spline_opts;
 
+extern ModeHook init_star;
+extern ModeHook draw_star;
+extern ModeHook release_star;
+extern ModeHook refresh_star;
+extern ModeSpecOpt star_opts;
+
 extern ModeHook init_swarm;
 extern ModeHook draw_swarm;
 extern ModeHook release_swarm;
@@ -505,6 +554,12 @@ extern ModeHook release_triangle;
 extern ModeHook refresh_triangle;
 extern ModeSpecOpt triangle_opts;
 
+extern ModeHook init_turtle;
+extern ModeHook draw_turtle;
+extern ModeHook release_turtle;
+extern ModeHook refresh_turtle;
+extern ModeSpecOpt turtle_opts;
+
 extern ModeHook init_wator;
 extern ModeHook draw_wator;
 extern ModeHook release_wator;
@@ -529,12 +584,50 @@ extern ModeHook release_worm;
 extern ModeHook refresh_worm;
 extern ModeSpecOpt worm_opts;
 
+#ifdef USE_HACKERS
+extern ModeHook init_fadeplot;
+extern ModeHook draw_fadeplot;
+extern ModeHook release_fadeplot;
+extern ModeHook refresh_fadeplot;
+extern ModeSpecOpt fadeplot_opts;
+
+#ifdef USE_GL
+extern ModeHook init_morph3d;
+extern ModeHook draw_morph3d;
+extern ModeHook release_morph3d;
+extern ModeHook change_morph3d;
+extern ModeSpecOpt morph3d_opts;
+#endif
+
+extern ModeHook init_trig;
+extern ModeHook draw_trig;
+extern ModeHook release_trig;
+extern ModeHook refresh_trig;
+extern ModeSpecOpt trig_opts;
+
+extern ModeHook init_tube;
+extern ModeHook draw_tube;
+extern ModeHook release_tube;
+extern ModeHook refresh_tube;
+extern ModeSpecOpt tube_opts;
+#endif
+
 extern ModeHook init_blank;
 extern ModeHook draw_blank;
 extern ModeHook release_blank;
 extern ModeHook refresh_blank;
 extern ModeSpecOpt blank_opts;
 
+#ifdef USE_BOMB
+extern ModeHook init_bomb;
+extern ModeHook draw_bomb;
+extern ModeHook release_bomb;
+extern ModeHook refresh_bomb;
+extern ModeHook change_bomb;
+extern ModeSpecOpt bomb_opts;
+
+#endif
+
 extern ModeHook init_random;
 extern ModeHook draw_random;
 extern ModeHook refresh_random;
