@@ -1,6 +1,6 @@
 
-#ifndef lint
-static char sccsid[] = "@(#)passwd.c	3.11 96/09/20 xlockmore";
+#if !defined( lint ) && !defined( SABER )
+static const char sccsid[] = "@(#)passwd.c	4.00 97/01/01 xlockmore";
 
 #endif
 
@@ -11,7 +11,7 @@ static char sccsid[] = "@(#)passwd.c	3.11 96/09/20 xlockmore";
  *
  * Revision History:
  *
- * Changes of David Bagley <bagleyd@hertz.njit.edu>
+ * Changes of David Bagley <bagleyd@bigfoot.com>
  * 25-May-96: When xlock is compiled with shadow passwords it will still
  *            work on non shadowed systems.  Marek Michalkiewicz
  *            <marekm@i17linuxb.ists.pwr.wroc.pl>
@@ -19,7 +19,7 @@ static char sccsid[] = "@(#)passwd.c	3.11 96/09/20 xlockmore";
  *            Added in ability for any of the root accounts to unlock
  *            screen.  Message now gets sent to syslog if root does the
  *            unlocking.
- * 23-Dec-95: Ron Hitchens <ronh@utw.com> reorganized.
+ * 23-Dec-95: Ron Hitchens <ron@idiom.com> reorganized.
  * 10-Dec-95: More context handling stuff for DCE thanks to
  *            Terje Marthinussen <terjem@cc.uit.no>.
  * 01-Sep-95: DCE code added thanks to Heath A. Kehoe
@@ -30,6 +30,14 @@ static char sccsid[] = "@(#)passwd.c	3.11 96/09/20 xlockmore";
 
 #include "xlock.h"
 
+#ifdef VMS
+#include <str$routines.h>
+#include <starlet.h>
+#define ROOT "SYSTEM"
+#else
+#define ROOT "root"
+#endif
+
 extern char *ProgramName;
 extern Bool allowroot;
 extern Bool inroot;
@@ -37,19 +45,10 @@ extern Bool inwindow;
 extern Bool grabmouse;
 extern Bool nolock;
 
-#ifdef VMS
-#define ROOT "SYSTEM"
-#else
-#define ROOT "root"
-#endif
 #if !defined( VMS ) && !defined( SUNOS_ADJUNCT_PASSWD )
 #include <pwd.h>
 #endif
 
-#if defined( __FreeBSD__ ) || defined( __NetBSD__ ) || defined( __hpux ) || defined( SUNOS_ADJUNCT_PASSWD )
-#include <sys/types.h>
-#endif
-
 #ifdef __bsdi__
 #include <sys/param.h>
 #if _BSDI_VERSION >= 199608
@@ -64,10 +63,10 @@ static login_cap_t *rlc = NULL;
 
 #endif
 
-#ifdef MULTIPLE_ROOT
+#if ( HAVE_FCNTL_H && defined( USE_MULTIPLE_ROOT ))
 #include <fcntl.h>
 #include <errno.h>
-#ifdef SYSLOG
+#if ( HAVE_SYSLOG_H && defined( USE_SYSLOG ))
 #include <syslog.h>
 #endif
 extern int  errno;
@@ -121,28 +120,27 @@ new_pwlnode(void)
 #endif
 
 #ifdef OSF1_ENH_SEC
-#include <sys/types.h>
 #include <sys/security.h>
 #include <prot.h>
 #endif
 
-#if defined( __linux__ ) && defined( __ELF__ ) && !defined( HAS_SHADOW )
+#if defined( __linux__ ) && defined( __ELF__ ) && !defined( HAVE_SHADOW )
 /* Linux may or may not have shadow passwords, so it is best to make the same
    binary work with both shadow and non-shadow passwords. It's easy with the
    ELF libc since it has getspnam() and there is no need for any additional
    libraries like libshadow.a.  This is a quick hack; it probably should be
    done properly in the Imakefile.  */
-#define HAS_SHADOW
+#define HAVE_SHADOW
 #endif
 
-#if defined( __linux__ ) && defined( HAS_SHADOW ) && defined( HAS_PW_ENCRYPT )
+#if defined( __linux__ ) && defined( HAVE_SHADOW ) && defined( HAVE_PW_ENCRYPT )
 /* Deprecated - long passwords have known weaknesses.  Also, pw_encrypt is
    non-standard (requires libshadow.a) while everything else you need to
    support shadow passwords is in the standard (ELF) libc.  */
 #define crypt pw_encrypt
 #endif
 
-#ifdef HAS_SHADOW
+#ifdef HAVE_SHADOW
 #ifndef __hpux
 #include <shadow.h>
 #endif
@@ -156,7 +154,7 @@ new_pwlnode(void)
 #define pw_passwd pwa_passwd
 #define getpwnam(_s) getpwanam(_s)
 #define pw_name pwa_name
-#define getpwuid(_s) getpwanam(cuserid(NULL))
+#define getpwuid(_s) (((_s)==0)?getpwanam(ROOT):getpwanam(cuserid(NULL)))
 #endif /* SUNOS_ADJUNCT_PASSWD */
 
 /* System V Release 4 redefinitions of BSD functions and structures */
@@ -210,7 +208,7 @@ struct itmlst {
 
 #endif /* VMS */
 
-#ifdef HP_PASSWDETC
+#ifdef HP_PASSWDETC		/* HAVE_SYS_WAIT_H */
 #include <sys/wait.h>
 #endif /* HP_PASSWDETC */
 
@@ -237,13 +235,12 @@ static int  check_dce_net_passwd(char *, char *);
 
 #endif
 
-#if defined( HAS_KRB4 ) || defined( HAS_KRB5 )
-#ifdef HAS_KRB4
+#if defined( HAVE_KRB4 ) || defined( HAVE_KRB5 )
+#ifdef HAVE_KRB4
 #include <krb.h>
-#else /* HAS_KRB5 */
+#else /* HAVE_KRB5 */
 #include <krb5.h>
 #endif
-#include <sys/types.h>
 #include <sys/param.h>
 static int  krb_check_password(struct passwd *, char *);
 
@@ -252,6 +249,7 @@ static int  krb_check_password(struct passwd *, char *);
 #if 0
 extern char *crypt(char *, char *);
 extern char *crypt(const char *, const char *);
+
 #endif
 
 #if (defined( USE_XLOCKRC ) || (!defined( OSF1_ENH_SEC ) &&  !defined( HP_PASSWDETC ) && !defined( VMS )))
@@ -259,21 +257,45 @@ static struct passwd *
 my_passwd_entry(void)
 {
 	int         uid;
-	char       *user;
 	struct passwd *pw;
 
+#ifdef HAVE_SHADOW
+	struct spwd *spw;
+
+#endif
+
 	uid = getuid();
-	pw = 0;
-	user = getenv("LOGNAME");
-	if (!user)
-		user = getenv("USER");
-	if (user) {
-		pw = getpwnam(user);
-		if (pw && (pw->pw_uid != uid))
-			pw = 0;
+#ifndef SUNOS_ADJUNCT_PASSWD
+	{
+		char       *user = NULL;
+
+		pw = 0;
+		user = getenv("LOGNAME");
+		if (!user)
+			user = getenv("USER");
+		if (user) {
+/* PURIFY 3.2 on Solaris2 reports an uninitialized memory read and PURIFY 4.0
+   on SunOS4 reports a file descriptor allocated on the next line */
+			pw = getpwnam(user);
+/* PURIFY 3.2 on Solaris2 reports an uninitialized memory read on the next
+   line PURIFY 4.0 on SunOS4 does not report this error */
+			if (pw && (pw->pw_uid != uid))
+				pw = 0;
+		}
 	}
 	if (!pw)
+#endif
 		pw = getpwuid(uid);
+#ifdef HAVE_SHADOW
+	if ((spw = getspnam(pw->pw_name)) != NULL) {
+		char       *tmp;	/* swap */
+
+		tmp = pw->pw_passwd;
+		pw->pw_passwd = spw->sp_pwdp;
+		spw->sp_pwdp = tmp;
+	}
+	endspent();
+#endif
 	return (pw);
 }
 #endif
@@ -342,9 +364,11 @@ getUserName(void)
 	if (!(pw = my_passwd_entry()))
 		/*if ((pw = (struct passwd *) getpwuid(getuid())) == NULL) */
 		error("%s: could not get user name.\n");
+/* PURIFY 3.2 on Solaris2 reports an uninitialized memory read on the next
+   line. PURIFY 4.0 on SunOS4 does not report this error */
 	(void) strcpy(user, pw->pw_name);
 
-#if MULTIPLE_ROOT
+#if ( HAVE_FCNTL_H && defined( USE_MULTIPLE_ROOT ))
 	get_multiple(pw);
 #endif
 
@@ -353,7 +377,7 @@ getUserName(void)
 #endif /* !VMS */
 }
 
-#ifdef HAS_SHADOW
+#if defined( HAVE_SHADOW ) && !defined( USE_XLOCKRC )
 static int
 passwd_invalid(char *passwd)
 {
@@ -364,7 +388,8 @@ passwd_invalid(char *passwd)
 #endif
 
 #if !defined( ultrix ) && !defined( DCE_PASSWD ) && !defined( BSD_AUTH )
-/* This routine is not needed if MULTIPE_ROOT */
+#ifndef USE_XLOCKRC
+/* This routine is not needed if HAVE_FCNTL_H and USE_MULTIPLE_ROOT */
 static void
 getCryptedUserPasswd(void)
 {
@@ -374,15 +399,15 @@ getCryptedUserPasswd(void)
 
 	il[0].buflen = 2;
 	il[0].code = UAI$_SALT;
-	il[0].addr = &uai_salt;
+	il[0].addr = (long) &uai_salt;
 	il[0].retadr = 0;
 	il[1].buflen = 8;
 	il[1].code = UAI$_PWD;
-	il[1].addr = &hash_password;
+	il[1].addr = (long) &hash_password;
 	il[1].retadr = 0;
 	il[2].buflen = 1;
 	il[2].code = UAI$_ENCRYPT;
-	il[2].addr = &uai_encrypt;
+	il[2].addr = (long) &uai_encrypt;
 	il[2].retadr = 0;
 	il[3].buflen = 0;
 	il[3].code = 0;
@@ -443,20 +468,13 @@ getCryptedUserPasswd(void)
 
 	struct passwd *pw;
 
-#ifdef HAS_SHADOW
-	struct spwd *spw;
-
-#endif
 	if (!(pw = my_passwd_entry()))
-		/*if ((pw = (struct passwd *) getpwuid(getuid())) == NULL) */
 		error("%s: could not get encrypted user password.\n");
-#ifdef HAS_SHADOW
-	if ((spw = getspnam(pw->pw_name)) != NULL)
-		pw->pw_passwd = spw->sp_pwdp;
-	endspent();
+	/*if ((pw = (struct passwd *) getpwuid(getuid())) == NULL) */
 	/* Check if there is any chance of unlocking the display later...  */
 	/* Program probably needs to be setuid to root.  */
 	/* If this is not possible, compile with -DUSE_XLOCKRC */
+#ifdef HAVE_SHADOW
 	if (passwd_invalid(pw->pw_passwd))
 		error("%s: it looks like you have shadow passwording.\nContact your administrator.\n");
 #endif
@@ -467,6 +485,8 @@ getCryptedUserPasswd(void)
 #endif /* !VMS */
 }
 
+#endif /* !USE_XLOCKRC */
+
 static void
 getCryptedRootPasswd(void)
 {
@@ -476,15 +496,15 @@ getCryptedRootPasswd(void)
 
 	il[0].buflen = 2;
 	il[0].code = UAI$_SALT;
-	il[0].addr = &root_salt;
+	il[0].addr = (long) &root_salt;
 	il[0].retadr = 0;
 	il[1].buflen = 8;
 	il[1].code = UAI$_PWD;
-	il[1].addr = &root_password;
+	il[1].addr = (long) &root_password;
 	il[1].retadr = 0;
 	il[2].buflen = 1;
 	il[2].code = UAI$_ENCRYPT;
-	il[2].addr = &root_encrypt;
+	il[2].addr = (long) &root_encrypt;
 	il[2].retadr = 0;
 	il[3].buflen = 0;
 	il[3].code = 0;
@@ -543,26 +563,34 @@ getCryptedRootPasswd(void)
 #else /* !OSF1_ENH_SEC */
 	struct passwd *pw;
 
-#ifdef HAS_SHADOW
+#ifdef HAVE_SHADOW
 	struct spwd *spw;
 
 #endif
 
+/* PURIFY 3.2 on Solaris2 reports an uninitialized memory read on the next
+   line. PURIFY 4.0 on SunOS4 does not report this error */
 	if (!(pw = getpwnam(ROOT)))
 		if (!(pw = getpwuid(0)))
 			/*if ((pw = (struct passwd *) getpwuid(0)) == NULL) */
 			error("%s: could not get encrypted root password.\n");
-#ifdef HAS_SHADOW
-	spw = getspnam(pw->pw_name);
-	if (spw)
+#ifdef HAVE_SHADOW
+	if ((spw = getspnam(pw->pw_name)) != NULL) {
+		char       *tmp;	/* swap */
+
+		tmp = pw->pw_passwd;
 		pw->pw_passwd = spw->sp_pwdp;
+		spw->sp_pwdp = tmp;
+	}
 	endspent();
 #endif
+/* PURIFY 3.2 on Solaris2 reports an uninitialized memory read on the next
+   line. PURIFY 4.0 on SunOS4 does not report this error */
 	(void) strcpy(rootpass, pw->pw_passwd);
 
-#ifdef MULTIPLE_ROOT
+#if ( HAVE_FCNTL_H && defined( USE_MULTIPLE_ROOT ))
 	set_multiple();
-#endif /* MULTIPLE_ROOT */
+#endif /* HAVE_FCNTL_H && MULTIPLE_ROOT */
 
 #endif /* !OSF1_ENH_SEC */
 #endif /* !HP_PASSWDETC */
@@ -638,7 +666,7 @@ checkPasswd(char *buffer)
 	char       *name;
 
 	done = 0;
-#ifdef        MULTIPLE_ROOT
+#if ( HAVE_FCNTL_H && defined( USE_MULTIPLE_ROOT ))
 	/* Scan through the linked list until you match a password.  Print
 	 * message to log if password match doesn't equal the user.
 	 *
@@ -677,7 +705,7 @@ checkPasswd(char *buffer)
 				done = 1;
 		}
 	}
-#ifdef        MULTIPLE_ROOT
+#if ( HAVE_FCNTL_H && defined( USE_MULTIPLE_ROOT ))
 }
 #endif
 
@@ -690,13 +718,13 @@ checkPasswd(char *buffer)
 	done = !ka_UserAuthenticate(user, "", 0, buffer, 0, &reason);
 	if (!done)
 #endif /* !AFS */
-#if defined(HAS_KRB4) || defined(HAS_KRB5)
+#if defined(HAVE_KRB4) || defined(HAVE_KRB5)
 		if (!strcmp(userpass, "*"))
 			done = (krb_check_password((struct passwd *) getpwuid(getuid()), buffer) ||
 				(allowroot && !strcmp((char *) crypt(buffer, rootpass), rootpass)));
 		else
-#endif /* !HAS_KRB4 && !HAS_KRB5 */
-#ifdef MULTIPLE_ROOT
+#endif /* !HAVE_KRB4 && !HAVE_KRB5 */
+#if ( HAVE_FCNTL_H && defined( USE_MULTIPLE_ROOT ))
 			/* Scan through the linked list until you match a password.  Print
 			 * message to log if password match doesn't equal the user.
 			 *
@@ -706,7 +734,7 @@ checkPasswd(char *buffer)
 			done = 0;
 	for (pwll = pwllh; pwll->next; pwll = pwll->next)
 		if (!strcmp((char *) crypt(buffer, pwll->pw_passwd), pwll->pw_passwd)) {
-#ifdef SYSLOG
+#if ( HAVE_SYSLOG_H && defined( USE_SYSLOG ))
 			if (strcmp(user, pwll->pw_name) != 0)
 				syslog(LOG_NOTICE, "xlock: %s unlocked screen", pwll->pw_name);
 #endif
@@ -983,7 +1011,7 @@ check_dce_net_passwd(char *usr, char *pass)
 
 #endif /* DCE_PASSWD */
 
-#ifdef HAS_KRB4
+#ifdef HAVE_KRB4
 int
 krb_check_password(struct passwd *pwd, char *pass)
 {
@@ -1011,9 +1039,9 @@ krb_check_password(struct passwd *pwd, char *pass)
 	}
 	return 0;
 }
-#endif /* HAS_KERB4 */
+#endif /* HAVE_KERB4 */
 
-#ifdef HAS_KRB5
+#ifdef HAVE_KRB5
 /*-
  * Pretty much all of this was snatched out of the kinit from the Kerberos5b6
  * distribution.  The reason why I felt it was necessary to use kinit was
@@ -1068,7 +1096,7 @@ krb_check_password(struct passwd *pwd, char *pass)
 		com_err(ProgramName, code, "when unparsing name");
 		return 0;
 	}
-	memset((char *) &my_creds, 0, sizeof (my_creds));
+	(void) memset((char *) &my_creds, 0, sizeof (my_creds));
 
 	my_creds.client = me;
 
@@ -1099,7 +1127,7 @@ krb_check_password(struct passwd *pwd, char *pass)
 					     NULL, preauth, pass, 0,
 					     &my_creds, 0);
 
-	memset(pass, 0, sizeof (pass));
+	(void) memset(pass, 0, sizeof (pass));
 
 	if (code) {
 		if (code == KRB5KRB_AP_ERR_BAD_INTEGRITY)
@@ -1125,7 +1153,7 @@ krb_check_password(struct passwd *pwd, char *pass)
 
 	return 1;		/* success */
 }
-#endif /* HAS_KRB5 */
+#endif /* HAVE_KRB5 */
 
 #ifndef VMS
 #undef passwd
@@ -1136,7 +1164,7 @@ krb_check_password(struct passwd *pwd, char *pass)
 #endif
 #endif
 
-#ifdef MULTIPLE_ROOT
+#if ( HAVE_FCNTL_H && defined( USE_MULTIPLE_ROOT ))
 
 void
 get_multiple(struct passwd *pw)
@@ -1223,7 +1251,7 @@ set_multiple(void)
 	char        buf[BUFMAX], xlen;
 	pid_t       cid;
 
-#ifdef HAS_SHADOW
+#ifdef HAVE_SHADOW
 	struct spwd *spw;
 
 #endif
@@ -1246,10 +1274,14 @@ set_multiple(void)
 		while ((pw = getpwent()) != (struct passwd *) NULL) {
 			if (pw->pw_uid != 0)
 				continue;
-#ifdef HAS_SHADOW
-			spw = getspnam(pw->pw_name);
-			if (spw)
+#ifdef HAVE_SHADOW
+			if ((spw = getspnam(pw->pw_name)) != NULL) {
+				char       *tmp;	/* swap */
+
+				tmp = pw->pw_passwd;
 				pw->pw_passwd = spw->sp_pwdp;
+				spw->sp_pwdp = tmp;
+			}
 #endif
 			if (pw->pw_passwd[0] != '*') {
 				xlen = strlen(pw->pw_name);
@@ -1277,6 +1309,9 @@ set_multiple(void)
 				sbuf += xlen + 1;
 			}
 		}
+#ifdef HAVE_SHADOW
+		endspent();
+#endif
 		cbuf = &buf[sbuf];
 		*cbuf = -1;
 		sbuf++;
@@ -1378,8 +1413,8 @@ set_multiple(void)
 static void
 gpass(void)
 {
-#include <sys/types.h>
 #include <sys/param.h>
+#include <sys/stat.h>
 #define CPASSLENGTH 14
 #define CPASSCHARS 64
 	extern char *cpasswd;
