@@ -1,6 +1,5 @@
-
-#ifndef lint
-static char sccsid[] = "@(#)penrose.c  3.11 96/09/20 xlockmore";
+#if !defined( lint ) && !defined( SABER )
+static const char sccsid[] = "@(#)penrose.c	4.00 97/01/01 xlockmore";
 
 #endif
 
@@ -14,12 +13,11 @@ static char sccsid[] = "@(#)penrose.c  3.11 96/09/20 xlockmore";
  * Revision History:
  * 09-Sep-96: Written.
  */
+
 /*-
 Be careful, this probably still has a few bugs (many of which may only
-appear with a very low probability).  It is also filled with
-assert-based sanity checks, which can be disabled by defining NDEBUG.
-If one of these assertions fails (has not happened to me) while your
-screen is actually locked by the program, well...
+appear with a very low probability).  These are seen with -verbose .
+If one of these are hit penrose will reinitialize.
 */
 
 /*-
@@ -51,24 +49,22 @@ screen is actually locked by the program, well...
 
 #include "xlock.h"
 
-#include <assert.h>
-#include <math.h>
-
-/* Uncomment this to remove assert-based sanity checks. */
-/* #define NDEBUG */
-
 /*-
  * Annoyingly the ANSI C library people have reserved all identifiers
  * ending with _t for future use.  Hence we use _c as a suffix for
  * typedefs (c for class, although this is not C++).
  */
 
-#define MIN_SIZE 5
+#define MINSIZE 5
 
 /*-
  * In theory one could fit 10 tiles to a single vertex.  However, the
  * vertex rules only allow at most seven tiles to meet at a vertex.
  */
+
+#define CELEBRATE 31415927	/* This causes a pause, an error occurred. */
+#define COMPLETION 3141593	/* This causes a pause, an error occurred. */
+
 #define MAX_TILES_PER_VERTEX 7
 #define N_VERTEX_RULES 8
 #define ALLOC_NODE( type) ((type *)malloc( sizeof( type)))
@@ -267,8 +263,9 @@ static angle_c vtype_angles[] =
 
 /* Direction angle of an edge. */
 static      angle_c
-vertex_dir(fringe_node_c * vertex, unsigned side)
+vertex_dir(ModeInfo * mi, fringe_node_c * vertex, unsigned side)
 {
+	tiling_c   *tp = &tilings[MI_SCREEN(mi)];
 	fringe_node_c *v2 =
 	(side == S_LEFT ? vertex->next : vertex->prev);
 	register int i;
@@ -280,7 +277,14 @@ vertex_dir(fringe_node_c * vertex, unsigned side)
 			case -1:
 				return (2 * i + 5) % 10;
 		}
-	assert(0);
+	tp->done = True;
+	if (MI_WIN_IS_VERBOSE(mi)) {
+		(void) printf("Weirdness in vertex_dir (this has been reported)\n");
+		for (i = 0; i < 5; i++)
+			(void) printf("v2->fived[%d]=%d, vertex->fived[%d]=%d\n",
+				      i, v2->fived[i], i, vertex->fived[i]);
+	}
+	MI_PAUSE(mi) = CELEBRATE;
 	return 0;
 }
 
@@ -366,7 +370,7 @@ init_penrose(ModeInfo * mi)
 {
 	tiling_c   *tp;
 	fringe_node_c *fp;
-	int         i;
+	int         i, size;
 
 	if (tilings == NULL) {
 		if ((tilings = (tiling_c *) calloc(MI_NUM_SCREENS(mi),
@@ -384,26 +388,53 @@ init_penrose(ModeInfo * mi)
 		tp->thin_color = (NRAND(2 * MI_NPIXELS(mi) / 3) + tp->thick_color +
 				  MI_NPIXELS(mi) / 6) % MI_NPIXELS(mi);
 	}
-	tp->edge_length = MI_BATCHCOUNT(mi);
-	if (tp->edge_length < -MIN_SIZE) {
-		tp->edge_length = NRAND(-tp->edge_length - MIN_SIZE + 1) + MIN_SIZE;
-	} else if (tp->edge_length < MIN_SIZE)
-		tp->edge_length = MIN_SIZE;
+	size = MI_SIZE(mi);
+	if (size < -MINSIZE)
+		tp->edge_length = NRAND(MIN(-size, MAX(MINSIZE,
+		   MIN(tp->width, tp->height) / 2)) - MINSIZE + 1) + MINSIZE;
+	else if (size < MINSIZE) {
+		if (!size)
+			tp->edge_length = MAX(MINSIZE, MIN(tp->width, tp->height) / 2);
+		else
+			tp->edge_length = MINSIZE;
+	} else
+		tp->edge_length = MIN(size, MAX(MINSIZE,
+					    MIN(tp->width, tp->height) / 2));
 	tp->origin.x = (tp->width / 2 + NRAND(tp->width)) / 2;
 	tp->origin.y = (tp->height / 2 + NRAND(tp->height)) / 2;
 	tp->fringe.n_nodes = 2;
 	if (tp->fringe.nodes != 0)
 		release_screen(tp);
-	assert(tp->fringe.nodes == 0 && tp->forced.first == 0);
+	if (tp->fringe.nodes != 0 || tp->forced.first != 0) {
+		if (MI_WIN_IS_VERBOSE(mi)) {
+			(void) printf("Weirdness in init_penrose()\n");
+			(void) printf("tp->fringe.nodes = 0 && tp->forced.first = 0\n");
+		}
+		release_screen(tp);	/* Try again */
+		tp->done = True;
+	}
 	tp->forced.n_nodes = tp->forced.n_visible = 0;
 	fp = tp->fringe.nodes = ALLOC_NODE(fringe_node_c);
-	assert(fp != 0);
-
+	if (fp == 0) {
+		if (MI_WIN_IS_VERBOSE(mi)) {
+			(void) printf("Weirdness in init_penrose()\n");
+			(void) printf("fp = 0\n");
+		}
+		fp = tp->fringe.nodes = ALLOC_NODE(fringe_node_c);
+		tp->done = True;
+	}
 	/* First vertex. */
 	fp->rule_mask = (1 << N_VERTEX_RULES) - 1;
 	fp->list_ptr = 0;
 	fp->prev = fp->next = ALLOC_NODE(fringe_node_c);
-	assert(fp->next != 0);
+	if (fp->next == 0) {
+		if (MI_WIN_IS_VERBOSE(mi)) {
+			(void) printf("Weirdness in init_penrose()\n");
+			(void) printf("fp->next = 0\n");
+		}
+		fp->prev = fp->next = ALLOC_NODE(fringe_node_c);
+		tp->done = True;
+	}
 	fp->n_tiles = 0;
 	fp->loc = tp->origin;
 	fp->off_screen = False;
@@ -507,9 +538,9 @@ match_rules(fringe_node_c * vertex, rule_match_c * matches, int first_only)
 
 static int
 find_completions(fringe_node_c * vertex, rule_match_c * matches, int n_matches,
-		 unsigned side, vertex_type_c * results /*, int first_only*/)
+	       unsigned side, vertex_type_c * results /*, int first_only */ )
 {
-	int         n_res = 0;
+	int         n_res = 0, cont;
 	register int i, j;
 	vertex_type_c buf[MAX_COMPL];
 
@@ -524,11 +555,14 @@ find_completions(fringe_node_c * vertex, rule_match_c * matches, int n_matches,
 		% rule->n_tiles;
 		vertex_type_c vtype = rule->tiles[pos];
 
+		cont = 1;
 		for (j = 0; j < n_res; j++)
-			if (vtype == results[j])
-				goto cont;
-		results[n_res++] = vtype;
-	      cont:;
+			if (vtype == results[j]) {
+				cont = 0;
+				break;
+			}
+		if (cont)
+			results[n_res++] = vtype;
 	}
 	return n_res;
 }
@@ -617,19 +651,33 @@ draw_tile(fringe_node_c * v1, fringe_node_c * v2,
  * the vertex has become untileable set tp->done.  This is supposed
  * to detect dislocations -- never call this routine with a completely
  * tiled vertex.
+ *
+ * Check for untileable vertices in check_vertex and stop tiling as
+ * soon as one finds one.  I don't know if it is possible to run out
+ * of forced vertices while untileable vertices exist (or will
+ * cavities inevitably appear).  If this can happen, add_random_tile
+ * might get called with an untileable vertex, causing ( n <= 1).
+ * (This is what the tp->done checks for).
+ *
+ * A MI_PAUSE celebrates the dislocation.
  */
 static void
-check_vertex(fringe_node_c * vertex, tiling_c * tp)
+check_vertex(ModeInfo * mi, fringe_node_c * vertex, tiling_c * tp)
 {
 	rule_match_c hits[MAX_TILES_PER_VERTEX * N_VERTEX_RULES];
 	int         n_hits = match_rules(vertex, hits, False);
 	unsigned    forced_sides = 0;
 
-  if (vertex->rule_mask == 0)
-    tp->done = True;
-	if (1 == find_completions(vertex, hits, n_hits, S_LEFT, 0 /*, False*/))
+	if (vertex->rule_mask == 0) {
+		tp->done = True;
+		if (MI_WIN_IS_VERBOSE(mi)) {
+			(void) printf("Dislocation occured!\n");
+		}
+		MI_PAUSE(mi) = CELEBRATE;	/* Should be able to recover */
+	}
+	if (1 == find_completions(vertex, hits, n_hits, S_LEFT, 0 /*, False */ ))
 		forced_sides |= S_LEFT;
-	if (1 == find_completions(vertex, hits, n_hits, S_RIGHT, 0 /*, False*/))
+	if (1 == find_completions(vertex, hits, n_hits, S_RIGHT, 0 /*, False */ ))
 		forced_sides |= S_RIGHT;
 	if (forced_sides == 0) {
 		if (vertex->list_ptr != 0) {
@@ -672,9 +720,16 @@ check_vertex(fringe_node_c * vertex, tiling_c * tp)
  * the vertex being deleted.
  */
 static void
-delete_vertex(fringe_node_c * vertex, tiling_c * tp)
+delete_vertex(ModeInfo * mi, fringe_node_c * vertex, tiling_c * tp)
 {
-	assert(tp->fringe.nodes != vertex);
+	if (tp->fringe.nodes == vertex) {
+		tp->done = True;
+		if (MI_WIN_IS_VERBOSE(mi)) {
+			(void) printf("Weirdness in delete_penrose()\n");
+			(void) printf("tp->fringe.nodes == vertex\n");
+		}
+		MI_PAUSE(mi) = CELEBRATE;
+	}
 	if (vertex->list_ptr != 0) {
 		forced_node_c *node = *vertex->list_ptr;
 
@@ -695,10 +750,10 @@ delete_vertex(fringe_node_c * vertex, tiling_c * tp)
 /* Check whether the addition of a tile of type vtype would completely fill *
    the space available at vertex. */
 static int
-fills_vertex(vertex_type_c vtype, fringe_node_c * vertex)
+fills_vertex(ModeInfo * mi, vertex_type_c vtype, fringe_node_c * vertex)
 {
 	return
-		(vertex_dir(vertex, S_LEFT) - vertex_dir(vertex, S_RIGHT)
+		(vertex_dir(mi, vertex, S_LEFT) - vertex_dir(mi, vertex, S_RIGHT)
 		 - vtype_angle(vtype)) % 10 == 0;
 }
 
@@ -726,7 +781,8 @@ fills_vertex(vertex_type_c vtype, fringe_node_c * vertex)
 #define FC_TOTAL_MASK 0xff
 
 static unsigned
-fringe_changes(fringe_node_c * vertex, unsigned side, vertex_type_c vtype,
+fringe_changes(ModeInfo * mi, fringe_node_c * vertex,
+	       unsigned side, vertex_type_c vtype,
 	       fringe_node_c ** right, fringe_node_c ** far,
 	       fringe_node_c ** left)
 {
@@ -735,7 +791,7 @@ fringe_changes(fringe_node_c * vertex, unsigned side, vertex_type_c vtype,
 
 	if (far)
 		*far = 0;
-	if (fills_vertex(vtype, vertex)) {
+	if (fills_vertex(mi, vtype, vertex)) {
 		result |= FC_CUT_THIS;
 	} else if (side == S_LEFT) {
 		result |= FC_NEW_RIGHT;
@@ -751,7 +807,7 @@ fringe_changes(fringe_node_c * vertex, unsigned side, vertex_type_c vtype,
 		v = vertex->next;
 		if (left)
 			*left = v;
-		if (fills_vertex(VT_LEFT(vtype), v)) {
+		if (fills_vertex(mi, VT_LEFT(vtype), v)) {
 			result = (result & ~FC_NEW_FAR) | FC_CUT_LEFT;
 			f = v->next;
 			if (far)
@@ -762,7 +818,7 @@ fringe_changes(fringe_node_c * vertex, unsigned side, vertex_type_c vtype,
 		v = vertex->prev;
 		if (right)
 			*right = v;
-		if (fills_vertex(VT_RIGHT(vtype), v)) {
+		if (fills_vertex(mi, VT_RIGHT(vtype), v)) {
 			result = (result & ~FC_NEW_FAR) | FC_CUT_RIGHT;
 			f = v->prev;
 			if (far)
@@ -770,7 +826,7 @@ fringe_changes(fringe_node_c * vertex, unsigned side, vertex_type_c vtype,
 		}
 	}
 	if (!(result & FC_NEW_FAR)
-	    && fills_vertex(VT_FAR(vtype), f)) {
+	    && fills_vertex(mi, VT_FAR(vtype), f)) {
 		result |= FC_CUT_FAR;
 		result &= (~FC_NEW_LEFT & ~FC_NEW_RIGHT);
 		if (right && (result & FC_CUT_LEFT))
@@ -802,11 +858,18 @@ add_vtype(fringe_node_c * vertex, unsigned side, vertex_type_c vtype)
 }
 
 static fringe_node_c *
-alloc_vertex(angle_c dir, fringe_node_c * from, tiling_c * tp)
+alloc_vertex(ModeInfo * mi, angle_c dir, fringe_node_c * from, tiling_c * tp)
 {
 	fringe_node_c *v = ALLOC_NODE(fringe_node_c);
 
-	assert(v != 0);
+	if (v == 0) {
+		tp->done = True;
+		if (MI_WIN_IS_VERBOSE(mi)) {
+			(void) printf("Weirdness in alloc_vertex()\n");
+			(void) printf("v = 0\n");
+		}
+		MI_PAUSE(mi) = CELEBRATE;
+	}
 	*v = *from;
 	add_unit_vec(dir, v->fived);
 	v->loc = fived_to_loc(v->fived, tp);
@@ -837,17 +900,17 @@ alloc_vertex(angle_c dir, fringe_node_c * from, tiling_c * tp)
  * is not added.  The return value is true iff a tile was added.
  */
 static int
-add_tile(fringe_node_c * vertex, unsigned side, vertex_type_c vtype,
-	 ModeInfo * mi)
+add_tile(ModeInfo * mi,
+	 fringe_node_c * vertex, unsigned side, vertex_type_c vtype)
 {
-	tiling_c   *tp = tilings + MI_SCREEN(mi);
+	tiling_c   *tp = &tilings[MI_SCREEN(mi)];
 
 	fringe_node_c
 		* left = 0,
 		*right = 0,
 		*far = 0,
 		*node;
-	unsigned    fc = fringe_changes(vertex, side, vtype, &right, &far, &left);
+	unsigned    fc = fringe_changes(mi, vertex, side, vtype, &right, &far, &left);
 
 	vertex_type_c
 		ltype = VT_LEFT(vtype),
@@ -858,22 +921,27 @@ add_tile(fringe_node_c * vertex, unsigned side, vertex_type_c vtype,
 	   vertex->prev to the right. */
 
 	/* This should never occur. */
-	assert(!(fc & FC_BAG));
-
+	if (fc & FC_BAG) {
+		tp->done = True;
+		if (MI_WIN_IS_VERBOSE(mi)) {
+			(void) printf("Weirdness in add_tile()\n");
+			(void) printf("fc = %d, FC_BAG = %d\n", fc, FC_BAG);
+		}
+	}
 	if (side == S_LEFT) {
 		if (right == 0)
-			right = alloc_vertex(vertex_dir(vertex, S_LEFT) - vtype_angle(vtype),
-					     vertex, tp);
+			right = alloc_vertex(mi,
+					     vertex_dir(mi, vertex, S_LEFT) - vtype_angle(vtype), vertex, tp);
 		if (far == 0)
-			far = alloc_vertex(vertex_dir(left, S_RIGHT) + vtype_angle(ltype),
-					   left, tp);
+			far = alloc_vertex(mi,
+					   vertex_dir(mi, left, S_RIGHT) + vtype_angle(ltype), left, tp);
 	} else {
 		if (left == 0)
-			left = alloc_vertex(vertex_dir(vertex, S_RIGHT) + vtype_angle(vtype),
-					    vertex, tp);
+			left = alloc_vertex(mi,
+					    vertex_dir(mi, vertex, S_RIGHT) + vtype_angle(vtype), vertex, tp);
 		if (far == 0)
-			far = alloc_vertex(vertex_dir(right, S_LEFT) - vtype_angle(rtype),
-					   right, tp);
+			far = alloc_vertex(mi,
+					   vertex_dir(mi, right, S_LEFT) - vtype_angle(rtype), right, tp);
 	}
 
 	/* Having allocated the new vertices, but before joining them with
@@ -886,11 +954,11 @@ add_tile(fringe_node_c * vertex, unsigned side, vertex_type_c vtype,
 		    || ((fc & FC_NEW_FAR) && fived_equal(node->fived, far->fived))) {
 			/* Better luck next time. */
 			if (fc & FC_NEW_LEFT)
-				delete_vertex(left, tp);
+				delete_vertex(mi, left, tp);
 			if (fc & FC_NEW_RIGHT)
-				delete_vertex(right, tp);
+				delete_vertex(mi, right, tp);
 			if (fc & FC_NEW_FAR)
-				delete_vertex(far, tp);
+				delete_vertex(mi, far, tp);
 			return False;
 		}
 		node = node->next;
@@ -920,29 +988,29 @@ add_tile(fringe_node_c * vertex, unsigned side, vertex_type_c vtype,
 	/* Delete vertices that are no longer on the fringe.  Check the others. */
 	if (fc & FC_CUT_THIS) {
 		tp->fringe.nodes = far;
-		delete_vertex(vertex, tp);
+		delete_vertex(mi, vertex, tp);
 	} else {
 		add_vtype(vertex, side, vtype);
-		check_vertex(vertex, tp);
+		check_vertex(mi, vertex, tp);
 		tp->fringe.nodes = vertex;
 	}
 	if (fc & FC_CUT_FAR)
-		delete_vertex(far, tp);
+		delete_vertex(mi, far, tp);
 	else {
 		add_vtype(far, fc & FC_CUT_RIGHT ? S_LEFT : S_RIGHT, ftype);
-		check_vertex(far, tp);
+		check_vertex(mi, far, tp);
 	}
 	if (fc & FC_CUT_LEFT)
-		delete_vertex(left, tp);
+		delete_vertex(mi, left, tp);
 	else {
 		add_vtype(left, fc & FC_CUT_FAR ? S_LEFT : S_RIGHT, ltype);
-		check_vertex(left, tp);
+		check_vertex(mi, left, tp);
 	}
 	if (fc & FC_CUT_RIGHT)
-		delete_vertex(right, tp);
+		delete_vertex(mi, right, tp);
 	else {
 		add_vtype(right, fc & FC_CUT_FAR ? S_RIGHT : S_LEFT, rtype);
-		check_vertex(right, tp);
+		check_vertex(mi, right, tp);
 	}
 	return True;
 }
@@ -959,8 +1027,9 @@ add_tile(fringe_node_c * vertex, unsigned side, vertex_type_c vtype,
  * simpler version I used.
  */
 static int
-add_forced_tile(forced_node_c * node, ModeInfo * mi)
+add_forced_tile(ModeInfo * mi, forced_node_c * node)
 {
+	tiling_c   *tp = &tilings[MI_SCREEN(mi)];
 	unsigned    side;
 	vertex_type_c vtype;
 	rule_match_c hits[MAX_TILES_PER_VERTEX * N_VERTEX_RULES];
@@ -971,9 +1040,15 @@ add_forced_tile(forced_node_c * node, ModeInfo * mi)
 	else
 		side = node->forced_sides;
 	n = match_rules(node->vertex, hits, True);
-	n = find_completions(node->vertex, hits, n, side, &vtype /*, True*/);
-	assert(n > 0);
-	return add_tile(node->vertex, side, vtype, mi);
+	n = find_completions(node->vertex, hits, n, side, &vtype /*, True */ );
+	if (n <= 0) {
+		tp->done = True;
+		if (MI_WIN_IS_VERBOSE(mi)) {
+			(void) printf("Weirdness in add_forced_tile()\n");
+			(void) printf("n = %d\n", n);
+		}
+	}
+	return add_tile(mi, node->vertex, side, vtype);
 }
 
 
@@ -993,7 +1068,7 @@ legal_move(fringe_node_c * vertex, unsigned side, vertex_type_c vtype)
 	int         n_hits, n_legal, i;
 
 	n_hits = match_rules(vertex, hits, False);
-	n_legal = find_completions(vertex, hits, n_hits, side, legal_vt /*, False*/);
+	n_legal = find_completions(vertex, hits, n_hits, side, legal_vt /*, False */ );
 	for (i = 0; i < n_legal; i++)
 		if (legal_vt[i] == vtype)
 			return True;
@@ -1024,13 +1099,26 @@ add_random_tile(fringe_node_c * vertex, ModeInfo * mi)
 		tp->thick_color = tp->thin_color = MI_WIN_WHITE_PIXEL(mi);
 	n_hits = match_rules(vertex, hits, False);
 	side = NRAND(2) ? S_LEFT : S_RIGHT;
-	n = find_completions(vertex, hits, n_hits, side, vtypes /*, False*/);
-	assert(n > 1);		/* One answer would mean a forced tile. */
+	n = find_completions(vertex, hits, n_hits, side, vtypes /*, False */ );
+	/* One answer would mean a forced tile. */
+	if (n <= 0) {
+		tp->done = True;
+		if (MI_WIN_IS_VERBOSE(mi)) {
+			(void) printf("Weirdness in add_random_tile()\n");
+			(void) printf("n = %d\n", n);
+		}
+	}
 	no_good = 0;
 	n_good = n;
 	for (i = 0; i < n; i++) {
-		fc = fringe_changes(vertex, side, vtypes[i], &right, &far, &left);
-		assert(!(fc & FC_BAG));
+		fc = fringe_changes(mi, vertex, side, vtypes[i], &right, &far, &left);
+		if (fc & FC_BAG) {
+			tp->done = True;
+			if (MI_WIN_IS_VERBOSE(mi)) {
+				(void) printf("Weirdness in add_random_tile()\n");
+				(void) printf("fc = %d, FC_BAG = %d\n", fc, FC_BAG);
+			}
+		}
 		if (right) {
 			s = (((fc & FC_CUT_FAR) && (fc & FC_CUT_LEFT)) ? S_RIGHT : S_LEFT);
 			if (!legal_move(right, s, VT_RIGHT(vtypes[i]))) {
@@ -1055,14 +1143,26 @@ add_random_tile(fringe_node_c * vertex, ModeInfo * mi)
 			}
 		}
 	}
-	assert(n_good > 0);
+	if (n_good <= 0) {
+		tp->done = True;
+		if (MI_WIN_IS_VERBOSE(mi)) {
+			(void) printf("Weirdness in add_random_tile()\n");
+			(void) printf("n_good = %d\n", n_good);
+		}
+	}
 	n = NRAND(n_good);
 	for (i = j = 0; i <= n; i++, j++)
 		while (no_good & (1 << j))
 			j++;
 
-	i = add_tile(vertex, side, vtypes[j - 1], mi);
-	assert(i);
+	i = add_tile(mi, vertex, side, vtypes[j - 1]);
+	if (!i) {
+		tp->done = True;
+		if (MI_WIN_IS_VERBOSE(mi)) {
+			(void) printf("Weirdness in add_random_tile()\n");
+			(void) printf("i = %d\n", i);
+		}
+	}
 }
 
 /* One step of the growth algorithm. */
@@ -1082,13 +1182,13 @@ draw_penrose(ModeInfo * mi)
 		vertex_type_c vtype = VT_TOTAL_MASK & LRAND();
 
 		XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
-		(void) add_tile(tp->fringe.nodes, S_LEFT, vtype, mi);
+		(void) add_tile(mi, tp->fringe.nodes, S_LEFT, vtype);
 		return;
 	}
 	/* No visible nodes left. */
 	if (tp->fringe.n_nodes == 0) {
 		tp->done = True;
-		MI_PAUSE(mi) = 3141593;
+		MI_PAUSE(mi) = COMPLETION;	/* Just finished drawing */
 		return;
 	}
 	if (tp->forced.n_visible > 0 && tp->failures < 10) {
@@ -1118,7 +1218,7 @@ draw_penrose(ModeInfo * mi)
 		tp->failures = 0;
 		return;
 	}
-	if (add_forced_tile(p, mi))
+	if (add_forced_tile(mi, p))
 		tp->failures = 0;
 	else
 		tp->failures++;
