@@ -1,6 +1,6 @@
 
-#ifndef lint
-static char sccsid[] = "@(#)random.c	3.11 96/09/20 xlockmore";
+#if !defined( lint ) && !defined( SABER )
+static const char sccsid[] = "@(#)random.c	4.00 97/01/01 xlockmore";
 
 #endif
 
@@ -10,10 +10,10 @@ static char sccsid[] = "@(#)random.c	3.11 96/09/20 xlockmore";
  * See xlock.c for copying information.
  *
  * Revision History:
- * 18-Mar-96: Ron Hitchens <ronh@utw.com>
+ * 18-Mar-96: Ron Hitchens <ron@idiom.com>
  *		Re-coded for the ModeInfo calling scheme.  Added the
  *		change hook.  Get ready for 3.8 release.
- * 23-Dec-95: Ron Hitchens <ronh@utw.com>
+ * 23-Dec-95: Ron Hitchens <ron@idiom.com>
  *		Re-coded pickMode() to keep track of the modes, so
  *		that all modes are tried before there are any repeats.
  *		Also prevent a mode from being picked twice in a row
@@ -24,8 +24,11 @@ static char sccsid[] = "@(#)random.c	3.11 96/09/20 xlockmore";
 
 #include "xlock.h"
 
-
-#define NUMSPECIAL	2	/* blank,random */
+#ifdef USE_BOMB
+#define NUMSPECIAL	3	/* bomb, blank, random */
+#else
+#define NUMSPECIAL	2	/* blank, random */
+#endif
 
 #define GC_SAVE_VALUES (GCFunction|GCLineWidth|GCLineStyle|GCCapStyle|GCJoinStyle|GCGraphicsExposures|GCFont|GCSubwindowMode)
 
@@ -36,6 +39,7 @@ extern float saturation;
 extern int  delay;
 extern int  batchcount;
 extern int  cycles;
+extern int  size;
 
 static int  duration;
 static char *modelist;
@@ -52,16 +56,16 @@ static argtype vars[] =
 };
 static OptionStruct desc[] =
 {
-	{"-duration", "how long a mode runs before changing to another"},
-	{"-modelist", "list of modes to randomly choose from"}
+	{"-duration num", "how long a mode runs before changing to another"},
+	{"-modelist string", "list of modes to randomly choose from"}
 };
 
 ModeSpecOpt random_opts =
 {2, opts, 2, vars, desc};
 
 typedef struct {
-	int         width, height;
 	XGCValues   gcvs;
+	int         fix;
 } randomstruct;
 
 static int  currentmode = -1;
@@ -72,34 +76,6 @@ static Bool change_now = False;
 
 static randomstruct randoms[MAXSCREENS];
 
-static
-void
-setMode(int newmode, ModeInfo * mi)
-{
-	randomstruct *rp = &randoms[MI_SCREEN(mi)];
-	int         num_screens = ScreenCount(MI_DISPLAY(mi));
-	int         i;
-
-	currentmode = newmode;
-
-/* FIX THIS GLOBAL ACCESS */
-	delay = MI_DELAY(mi) = LockProcs[currentmode].def_delay;
-	batchcount = MI_BATCHCOUNT(mi) = LockProcs[currentmode].def_batchcount;
-	cycles = MI_CYCLES(mi) = LockProcs[currentmode].def_cycles;
-	saturation = MI_SATURATION(mi) = LockProcs[currentmode].def_saturation;
-
-	for (i = 0; i < num_screens; i++) {
-		XChangeGC(MI_DISPLAY(mi), MI_GC(mi),
-			  GC_SAVE_VALUES, &(rp->gcvs));
-		if (MI_NPIXELS(mi) > 2)
-			fixColormap(MI_DISPLAY(mi), MI_WINDOW(mi), i, MI_SATURATION(mi),
-				 MI_WIN_IS_INSTALL(mi), MI_WIN_IS_INROOT(mi),
-			      MI_WIN_IS_INWINDOW(mi), MI_WIN_IS_VERBOSE(mi));
-	}
-	if (MI_WIN_IS_VERBOSE(mi))
-		(void) printf("mode: %s\n", LockProcs[currentmode].cmdline_arg);
-}
-
 static
 int
 pickMode(void)
@@ -243,34 +219,63 @@ parsemodelist(void)
 #endif
 }
 
+static
 void
-init_random(ModeInfo * mi)
+setMode(ModeInfo * mi, int newmode)
 {
-	Display    *display = MI_DISPLAY(mi);
 	randomstruct *rp = &randoms[MI_SCREEN(mi)];
 	int         i;
 
-	rp->width = MI_WIN_WIDTH(mi);
-	rp->height = MI_WIN_HEIGHT(mi);
+	currentmode = newmode;
+
+/* FIX THIS GLOBAL ACCESS */
+	delay = MI_DELAY(mi) = LockProcs[currentmode].def_delay;
+	batchcount = MI_BATCHCOUNT(mi) = LockProcs[currentmode].def_batchcount;
+	cycles = MI_CYCLES(mi) = LockProcs[currentmode].def_cycles;
+	size = MI_SIZE(mi) = LockProcs[currentmode].def_size;
+	saturation = MI_SATURATION(mi) = LockProcs[currentmode].def_saturation;
+
+	for (i = 0; i < MI_NUM_SCREENS(mi); i++) {
+
+		XChangeGC(MI_DISPLAY(mi), MI_GC(mi), GC_SAVE_VALUES,
+			  &(rp->gcvs));		/* Not sure if this is right for multiscreens */
+		randoms[i].fix = True;
+	}
+	if (MI_WIN_IS_VERBOSE(mi))
+		(void) printf("mode: %s\n", LockProcs[currentmode].cmdline_arg);
+}
+
+void
+init_random(ModeInfo * mi)
+{
+	int         scrn = MI_SCREEN(mi);
+	randomstruct *rp = &randoms[scrn];
+	int         i;
 
 	if (currentmode < 0) {
 		parsemodelist();
 
-		for (i = 0; i < MI_NUM_SCREENS(mi); i++)
-			(void) XGetGCValues(display, MI_GC(mi), GC_SAVE_VALUES, &(rp->gcvs));
-		setMode(pickMode(), mi);
+		for (i = 0; i < MI_NUM_SCREENS(mi); i++) {
+			(void) XGetGCValues(MI_DISPLAY(mi), MI_GC(mi),
+					    GC_SAVE_VALUES, &(rp->gcvs));
+		}
+		setMode(mi, pickMode());
 		starttime = seconds();
 		if (duration < 0)
 			duration = 0;
 	}
+	if (rp->fix) {
+		fixColormap(MI_DISPLAY(mi), MI_WINDOW(mi), MI_SCREEN(mi),
+		MI_SATURATION(mi), MI_WIN_IS_MONO(mi), MI_WIN_IS_INSTALL(mi),
+			    MI_WIN_IS_INROOT(mi), MI_WIN_IS_INWINDOW(mi), MI_WIN_IS_VERBOSE(mi));
+		rp->fix = False;
+	}
 	call_init_hook(&LockProcs[currentmode], mi);
 }
 
 void
 draw_random(ModeInfo * mi)
 {
-	Display    *display = MI_DISPLAY(mi);
-	GC          gc = MI_GC(mi);
 	int         scrn = MI_SCREEN(mi);
 	randomstruct *rp = &randoms[scrn];
 	int         newmode, now = seconds();
@@ -280,18 +285,22 @@ draw_random(ModeInfo * mi)
 	if ((scrn == 0) && do_init) {
 		do_init = 0;
 	}
-	if (change_now || ((scrn == 0) && (has_run > duration))) {
+	if ((scrn == 0) && (change_now || (has_run > duration))) {
 		newmode = pickMode();
 
-		XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
-		XFillRectangle(display, MI_WINDOW(mi), gc,
-			       0, 0, rp->width, rp->height);
+		XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
 
-		setMode(newmode, mi);
+		setMode(mi, newmode);
 		starttime = now;
 		do_init = 1;
 		change_now = False;
 	}
+	if (rp->fix) {
+		fixColormap(MI_DISPLAY(mi), MI_WINDOW(mi), MI_SCREEN(mi),
+		MI_SATURATION(mi), MI_WIN_IS_MONO(mi), MI_WIN_IS_INSTALL(mi),
+			    MI_WIN_IS_INROOT(mi), MI_WIN_IS_INWINDOW(mi), MI_WIN_IS_VERBOSE(mi));
+		rp->fix = False;
+	}
 	if (do_init) {
 		call_init_hook(&LockProcs[currentmode], mi);
 	}
@@ -308,7 +317,8 @@ refresh_random(ModeInfo * mi)
 void
 change_random(ModeInfo * mi)
 {
-	change_now = True;	/* force a change on next draw callback */
+	if (MI_SCREEN(mi) == 0)
+		change_now = True;	/* force a change on next draw callback */
 
 	draw_random(mi);
 }
