@@ -1,5 +1,6 @@
-#ifndef lint
-static char sccsid[] = "@(#)swirl.c	3.9 96/05/25 xlockmore";
+
+#if !defined( lint ) && !defined( SABER )
+static const char sccsid[] = "@(#)swirl.c	4.00 97/01/01 xlockmore";
 
 #endif
 
@@ -18,10 +19,8 @@ static char sccsid[] = "@(#)swirl.c	3.9 96/05/25 xlockmore";
  *            based on original code by R.Taylor
  */
 
-#include <math.h>
-#include <time.h>
-
 #include "xlock.h"
+#include <time.h>
 
 /****************************************************************/
 
@@ -29,11 +28,11 @@ static char sccsid[] = "@(#)swirl.c	3.9 96/05/25 xlockmore";
 #define MIN_RES         5	/* minimim resolution (>= MIN_RES) */
 #define MAX_RES         1	/* maximum resolution (>0) */
 #define TWO_PLANE_PCNT  30	/* probability for two plane mode (0-100) */
-#define RESTART         2000	/* number of cycles before restart */
+#define RESTART         2500	/* number of cycles before restart */
 #define BATCH_DRAW      100	/* points to draw per iteration */
 
 ModeSpecOpt swirl_opts =
-{0, NULL, NULL, NULL};
+{0, NULL, 0, NULL, NULL};
 
 /* knot types */
 typedef enum {
@@ -144,8 +143,10 @@ random_no(unsigned int n)
    -      swirl is the swirl data */
 
 static void
-initialise_swirl(SWIRL_P swirl)
+initialise_swirl(ModeInfo * mi, SWIRL_P swirl)
 {
+	Display    *display = MI_DISPLAY(mi);
+
 	swirl->width = 0;	/* width and height of window */
 	swirl->height = 0;
 	swirl->depth = 0;
@@ -177,14 +178,14 @@ initialise_swirl(SWIRL_P swirl)
 	swirl->current_map = 0;	/* current colour map, 0..dcolours-1 */
 
 	/* set up fg fb colour specs */
-	swirl->white = WhitePixel(dsp, screen);
-	swirl->black = BlackPixel(dsp, screen);
-	swirl->fg = Scr[screen].fgcol;
-	swirl->bg = Scr[screen].bgcol;
+	swirl->white = MI_WIN_WHITE_PIXEL(mi);
+	swirl->black = MI_WIN_BLACK_PIXEL(mi);
+	swirl->fg = MI_FG_COLOR(mi);
+	swirl->bg = MI_BG_COLOR(mi);
 	swirl->fgcol.pixel = swirl->fg;
 	swirl->bgcol.pixel = swirl->bg;
-	XQueryColor(dsp, DefaultColormap(dsp, screen), &(swirl->fgcol));
-	XQueryColor(dsp, DefaultColormap(dsp, screen), &(swirl->bgcol));
+	XQueryColor(display, MI_COLORMAP(mi), &(swirl->fgcol));
+	XQueryColor(display, MI_COLORMAP(mi), &(swirl->bgcol));
 }
 
 /****************************************************************/
@@ -197,14 +198,13 @@ initialise_swirl(SWIRL_P swirl)
  * -      swirl is the swirl data
  */
 static void
-initialise_image(SWIRL_P swirl)
+initialise_image(Display * dpy, SWIRL_P swirl)
 {
 	unsigned int pad;
 	int         bytes_per_line;
 
 	/* get the bitmap pad */
-	pad = BitmapPad(dsp);
-
+	pad = BitmapPad(dpy);
 	/* destroy the old image (destroy XImage and data) */
 	if (swirl->ximage != NULL)
 		XDestroyImage(swirl->ximage);
@@ -216,7 +216,7 @@ initialise_image(SWIRL_P swirl)
 	swirl->image = (unsigned char *) calloc(bytes_per_line * swirl->height, 1);
 
 	/* create an ximage with this */
-	swirl->ximage = XCreateImage(dsp, swirl->visual, swirl->rdepth, ZPixmap,
+	swirl->ximage = XCreateImage(dpy, swirl->visual, swirl->rdepth, ZPixmap,
 				     0, (char *) swirl->image, swirl->width,
 				     swirl->height, pad, bytes_per_line);
 }
@@ -354,7 +354,7 @@ set_black_and_white(SWIRL_P swirl, XColor * values)
  * -      c points to the colour to add
  */
 static void
-set_colour(SWIRL_P swirl, XColor ** value, long unsigned int *pixel, COLOUR_P c)
+set_colour(SWIRL_P swirl, XColor ** value, unsigned long *pixel, COLOUR_P c)
 {
 	Bool        done;
 	unsigned long fg, bg, white, black;
@@ -454,7 +454,7 @@ get_colour(SWIRL_P swirl, XColor ** value, COLOUR_P c)
  * -      c1, c2 are the colours to interpolate between
  */
 static void
-interpolate(SWIRL_P swirl, XColor ** values, long unsigned int *pixel, int n, COLOUR_P c1, COLOUR_P c2)
+interpolate(SWIRL_P swirl, XColor ** values, unsigned long *pixel, int n, COLOUR_P c1, COLOUR_P c2)
 {
 	int         i, r, g, b;
 	COLOUR      c;
@@ -463,7 +463,7 @@ interpolate(SWIRL_P swirl, XColor ** values, long unsigned int *pixel, int n, CO
 	/* maximum value */
 	maxv = (255 << 8);
 
-	for (i = 0; i < n / 2 && *pixel < swirl->colours; i++) {
+	for (i = 0; i < n / 2 && (int) *pixel < swirl->colours; i++) {
 		/* work out the colour */
 		r = c1->r + 2 * i * ((int) c2->r) / n;
 		c.r = (r > (int) maxv) ? maxv : r;
@@ -475,7 +475,7 @@ interpolate(SWIRL_P swirl, XColor ** values, long unsigned int *pixel, int n, CO
 		/* set it up */
 		set_colour(swirl, values, pixel, &c);
 	}
-	for (i = n / 2; i >= 0 && *pixel < swirl->colours; i--) {
+	for (i = n / 2; i >= 0 && (int) *pixel < swirl->colours; i--) {
 		r = c2->r + 2 * i * ((int) c1->r) / n;
 		c.r = (r > (int) maxv) ? maxv : r;
 		g = c2->g + 2 * i * ((int) c1->g) / n;
@@ -571,7 +571,7 @@ basic_map(SWIRL_P swirl, XColor * values)
 		    (int) ((double) swirl->dcolours * ((double) L3 / (double) L)), c + 2, c);
 
 	/* fill up any remaining slots (due to rounding) */
-	while (pixel < swirl->colours) {
+	while ((int) pixel < swirl->colours) {
 		/* repeat the last colour */
 		set_colour(swirl, &value, &pixel, c);
 	}
@@ -645,7 +645,7 @@ pre_rotate(SWIRL_P swirl, XColor * values)
  * Gets a XVisualInfo structure that refers to a given visual or the default
  * visual.
  *
- * -      dpy is the display
+ * -      mi is the ModeInfo
  * -      visual is the visual to look up NULL => look up the default visual
  * -      default_info is set to point to the member of the returned list
  *          that corresponds to the default visual.
@@ -654,24 +654,26 @@ pre_rotate(SWIRL_P swirl, XColor * values)
  *   with XFree.
  */
 static XVisualInfo *
-default_visual_info(Display * dpy, Visual * visual, XVisualInfo ** default_info)
+default_visual_info(ModeInfo * mi, Visual * visual, XVisualInfo ** default_info)
 {
+	Display    *display = MI_DISPLAY(mi);
 	XVisualInfo *info_list;
 	int         i, n;
 
 	/* get a complete list of visuals */
-	info_list = XGetVisualInfo(dpy, VisualNoMask, NULL, &n);
+	info_list = XGetVisualInfo(display, VisualNoMask, NULL, &n);
 	if ((info_list == NULL) || (n == 0))
 		return (NULL);
 
 	/* do we need to get the default visual? */
 	if (visual == NULL)
-		visual = DefaultVisual(dpy, screen);
+		visual = DefaultVisual(display, MI_SCREEN(mi));
 
 	/* search through the list for the default visual */
 	*default_info = info_list;
 	for (i = 0; i < n; i++, (*default_info)++)
-		if ((*default_info)->visual == visual && (*default_info)->screen == screen) {
+		if ((*default_info)->visual == visual &&
+		    (*default_info)->screen == MI_SCREEN(mi)) {
 			break;
 		}
 	/* return the list (and the default info) */
@@ -691,24 +693,25 @@ default_visual_info(Display * dpy, Visual * visual, XVisualInfo ** default_info)
 static void
 create_colourmap(ModeInfo * mi, SWIRL_P swirl)
 {
+	Display    *display = MI_DISPLAY(mi);
 	XVisualInfo *list, *default_info;
 	int         preserve;
 	int         n_rotations;
 	int         i;
 
 	/* get information about the default visual */
-	list = default_visual_info(dsp, swirl->visual, &default_info);
+	list = default_visual_info(mi, swirl->visual, &default_info);
 
 	/* how many colours are there altogether? */
-	swirl->colours = (MI_WIN_IS_MONO(mi)) ? 2 : (1 << swirl->depth);
+	swirl->colours = MI_NPIXELS(mi);
+	if (swirl->colours < 2)
+		swirl->colours = 2;
 	if (swirl->colours > default_info->colormap_size) {
 		swirl->colours = default_info->colormap_size;
 	}
 	/* how many colours should we preserve (out of white, black, fg, bg)? */
-	if (((swirl->bg == swirl->black) &&
-	     ((swirl->fg == swirl->black) || (swirl->fg == swirl->white))) ||
-	    ((swirl->fg == swirl->black) &&
-	     ((swirl->bg == swirl->black) || (swirl->bg == swirl->white))))
+	if (((swirl->bg == swirl->black) || (swirl->bg == swirl->white)) &&
+	    ((swirl->fg == swirl->black) || (swirl->fg == swirl->white)))
 		preserve = 2;
 	else if ((swirl->bg == swirl->black) || (swirl->fg == swirl->black) ||
 		 (swirl->bg == swirl->white) || (swirl->fg == swirl->white) ||
@@ -718,17 +721,25 @@ create_colourmap(ModeInfo * mi, SWIRL_P swirl)
 		preserve = 4;
 
 	/* how many colours should we animate? */
-	swirl->dcolours = (swirl->colours > preserve + 1) ? swirl->colours - preserve :
-		swirl->colours;
+	swirl->dcolours = (swirl->colours > preserve + 1) ?
+		swirl->colours - preserve : swirl->colours;
 
 	/* are we in monochrome mode? */
-	swirl->monochrome = MI_WIN_IS_MONO(mi) || (swirl->colours == 2);
+	swirl->monochrome = (swirl->colours <= 2);
 
 	/* is the colourmap fixed (we'll leave it alone in mono too) */
+
+#if defined(__cplusplus) || defined(c_plusplus)
+	swirl->fixed_colourmap = ((default_info->c_class == StaticGray) ||
+				  (default_info->c_class == StaticColor) ||
+				  (default_info->c_class == TrueColor) ||
+	swirl->monochrome || MI_WIN_IS_INROOT(mi) || !MI_WIN_IS_INSTALL(mi));
+#else
 	swirl->fixed_colourmap = ((default_info->class == StaticGray) ||
 				  (default_info->class == StaticColor) ||
 				  (default_info->class == TrueColor) ||
 	swirl->monochrome || MI_WIN_IS_INROOT(mi) || !MI_WIN_IS_INSTALL(mi));
+#endif
 
 #ifdef FORCEFIXEDCOLORS
 	/* pretending a fixed colourmap */
@@ -755,7 +766,8 @@ create_colourmap(ModeInfo * mi, SWIRL_P swirl)
 
 		/* create a colour map */
 		if (!swirl->fixed_colourmap)
-			swirl->cmap = XCreateColormap(dsp, swirl->win, swirl->visual, AllocAll);
+			swirl->cmap =
+				XCreateColormap(display, swirl->win, swirl->visual, AllocAll);
 	}
 	/* select a set of colours for the colour map */
 	basic_map(swirl, swirl->rgb_values);
@@ -766,9 +778,14 @@ create_colourmap(ModeInfo * mi, SWIRL_P swirl)
 		pre_rotate(swirl, swirl->rgb_values);
 
 		/* store the colours in the colour map */
-		XStoreColors(dsp, swirl->cmap, swirl->rgb_values, swirl->colours);
+		XStoreColors(display, swirl->cmap, swirl->rgb_values, swirl->colours);
 	} else {
-		if (default_info->class == TrueColor) {
+#if defined(__cplusplus) || defined(c_plusplus)
+		if (default_info->c_class == TrueColor)
+#else
+		if (default_info->class == TrueColor)
+#endif
+		{
 			int         rsh, gsh, bsh;
 			unsigned long int t;
 
@@ -792,7 +809,7 @@ create_colourmap(ModeInfo * mi, SWIRL_P swirl)
 		} else {
 			/* lookup the colours in the fixed colour map */
 			for (i = 0; i < swirl->colours; i++)
-				(void) XAllocColor(dsp, DefaultColormap(dsp, screen),
+				(void) XAllocColor(display, MI_COLORMAP(mi),
 						   &(swirl->rgb_values[i]));
 		}
 	}
@@ -805,11 +822,12 @@ create_colourmap(ModeInfo * mi, SWIRL_P swirl)
  *
  * Install a new set of colours into the colour map
  *
+ * -      dpy is the display
  * -      swirl is the swirl data
  * -      shift is the amount to rotate the colour map by
  */
 static void
-install_map(SWIRL_P swirl, int shift)
+install_map(Display * dpy, SWIRL_P swirl, int shift)
 {
 	if (!swirl->fixed_colourmap) {
 		/* shift the colour map */
@@ -817,7 +835,7 @@ install_map(SWIRL_P swirl, int shift)
 			swirl->dcolours;
 
 		/* store it */
-		XStoreColors(dsp, swirl->cmap,
+		XStoreColors(dpy, swirl->cmap,
 			     swirl->rgb_values +
 			     swirl->current_map * swirl->colours,
 			     swirl->colours);
@@ -942,98 +960,6 @@ create_knots(SWIRL_P swirl)
 
 /****************************************************************/
 
-/* 
- * init_swirl
- *
- * Initialise things for swirling
- *
- * -      win is the window to draw in
- */
-void
-init_swirl(ModeInfo * mi)
-{
-	Window      win = MI_WINDOW(mi);
-	SWIRL_P     swirl;
-
-	/* does the swirls array exist? */
-	if (swirls == NULL) {
-		int         i;
-
-		/* allocate an array, one entry for each screen */
-		swirls = (SWIRL_P) calloc(ScreenCount(dsp), sizeof (SWIRL));
-
-		/* initialise them all */
-		for (i = 0; i < ScreenCount(dsp); i++)
-			initialise_swirl((&swirls[i]));
-	}
-	/* get a pointer to this swirl */
-	swirl = &(swirls[screen]);
-
-	/* get window parameters */
-	swirl->win = win;
-	swirl->width = MI_WIN_WIDTH(mi);
-	swirl->height = MI_WIN_HEIGHT(mi);
-	swirl->depth = MI_WIN_DEPTH(mi);
-	swirl->rdepth = swirl->depth;
-	swirl->visual = MI_VISUAL(mi);
-
-	if (swirl->depth > 16)
-		swirl->depth = 16;
-
-	/* initialise image for speeding up drawing */
-	initialise_image(swirl);
-
-	/* clear the window (before setting the colourmap) */
-	XSetForeground(dsp, Scr[screen].gc, swirl->black);
-	XFillRectangle(dsp, win, Scr[screen].gc,
-		       0, 0, swirl->width, swirl->height);
-
-	/* initialise the colours from which the colourmap is derived */
-	initialise_colours(basic_colours, MI_SATURATION(mi));
-
-	/* set up the colour map */
-	create_colourmap(mi, swirl);
-
-	/* attach the colour map to the window (if we have one) */
-	if (!swirl->fixed_colourmap) {
-#if 1
-		set_colormap(win, swirl->cmap, MI_WIN_IS_INWINDOW(mi));
-#else
-		XSetWindowColormap(dsp, win, swirl->cmap);
-		(void) XSetWMColormapWindows(dsp, win, &win, 1);
-		XInstallColormap(dsp, swirl->cmap);
-#endif
-	}
-	/* resolution starts off chunky */
-	swirl->resolution = MIN_RES + 1;
-
-	/* calculate the pixel step for this resulution */
-	swirl->r = (1 << (swirl->resolution - 1));
-
-	/* how many knots? */
-	swirl->n_knots = random_no((unsigned int) MI_BATCHCOUNT(mi) / 2) +
-		MI_BATCHCOUNT(mi) + 1;
-
-	/* what type of knots? */
-	swirl->knot_type = ALL;	/* for now */
-
-	/* use two_plane mode occaisionally */
-	if (random_no(100) <= TWO_PLANE_PCNT) {
-		swirl->two_plane = swirl->first_plane = True;
-		swirl->max_resolution = 2;
-	} else
-		swirl->two_plane = False;
-
-	/* fix the knot values */
-	create_knots(swirl);
-
-	/* we are off */
-	swirl->started = True;
-	swirl->drawing = False;
-}
-
-/****************************************************************/
-
 /* 
  * do_point
  *
@@ -1132,10 +1058,10 @@ do_point(SWIRL_P swirl, int i, int j)
 
 	/* if fg and bg are 1 and 0 we should be OK, but just in case */
 	while ((dcolours > 2) &&
-	       (((value % swirl->colours) == swirl->fg) ||
-		((value % swirl->colours) == swirl->bg) ||
-		((value % swirl->colours) == swirl->white) ||
-		((value % swirl->colours) == swirl->black))) {
+	       (((value % swirl->colours) == (int) swirl->fg) ||
+		((value % swirl->colours) == (int) swirl->bg) ||
+		((value % swirl->colours) == (int) swirl->white) ||
+		((value % swirl->colours) == (int) swirl->black))) {
 		value++;
 	}
 
@@ -1163,19 +1089,14 @@ do_point(SWIRL_P swirl, int i, int j)
  * -      v is the value
  */
 static void
-draw_block(XImage * ximage, int x, int y, int s, long unsigned int v)
+draw_block(XImage * ximage, int x, int y, int s, unsigned long v)
 {
-	/* is it just a point? */
-	if (s == 1)
-		XPutPixel(ximage, x, y, v);
-	else {
-		int         a, b;
+	int         a, b;
 
-		for (a = 0; a < s; a++)
-			for (b = 0; b < s; b++) {
-				XPutPixel(ximage, x + b, y + a, v);
-			}
-	}
+	for (a = 0; a < s; a++)
+		for (b = 0; b < s; b++) {
+			XPutPixel(ximage, x + b, y + a, v);
+		}
 }
 
 /****************************************************************/
@@ -1187,7 +1108,7 @@ draw_block(XImage * ximage, int x, int y, int s, long unsigned int v)
  * -    win is the window to update
  */
 static void
-draw_point(SWIRL_P swirl, Window win)
+draw_point(ModeInfo * mi, SWIRL_P swirl)
 {
 	int         r;
 	int         x, y;
@@ -1217,7 +1138,8 @@ draw_point(SWIRL_P swirl, Window win)
 		draw_block(swirl->ximage, x, y, r, do_point(swirl, x, y));
 
 	/* update the screen */
-	XPutImage(dsp, win, Scr[screen].gc, swirl->ximage, x, y, x, y, r, r);
+	XPutImage(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi), swirl->ximage,
+		  x, y, x, y, r, r);
 }
 
 /****************************************************************/
@@ -1322,6 +1244,96 @@ next_point(SWIRL_P swirl)
 
 /****************************************************************/
 
+/* 
+ * init_swirl
+ *
+ * Initialise things for swirling
+ *
+ * -      win is the window to draw in
+ */
+void
+init_swirl(ModeInfo * mi)
+{
+	Display    *display = MI_DISPLAY(mi);
+	Window      window = MI_WINDOW(mi);
+	SWIRL_P     swirl;
+
+	/* does the swirls array exist? */
+	if (swirls == NULL) {
+		int         i;
+
+		/* allocate an array, one entry for each screen */
+		swirls = (SWIRL_P) calloc(ScreenCount(display), sizeof (SWIRL));
+
+		/* initialise them all */
+		for (i = 0; i < ScreenCount(display); i++)
+			initialise_swirl(mi, &swirls[i]);
+	}
+	/* get a pointer to this swirl */
+	swirl = &(swirls[MI_SCREEN(mi)]);
+
+	/* get window parameters */
+	swirl->win = window;
+	swirl->width = MI_WIN_WIDTH(mi);
+	swirl->height = MI_WIN_HEIGHT(mi);
+	swirl->depth = MI_WIN_DEPTH(mi);
+	swirl->rdepth = swirl->depth;
+	swirl->visual = MI_VISUAL(mi);
+
+	if (swirl->depth > 16)
+		swirl->depth = 16;
+
+	/* initialise image for speeding up drawing */
+	initialise_image(display, swirl);
+
+	/* clear the window (before setting the colourmap) */
+	XClearWindow(display, MI_WINDOW(mi));
+	/* initialise the colours from which the colourmap is derived */
+	initialise_colours(basic_colours, MI_SATURATION(mi));
+
+	/* set up the colour map */
+	create_colourmap(mi, swirl);
+
+	/* attach the colour map to the window (if we have one) */
+	if (!swirl->fixed_colourmap) {
+#if 1
+		setColormap(display, window, swirl->cmap, MI_WIN_IS_INWINDOW(mi));
+#else
+		XSetWindowColormap(display, window, swirl->cmap);
+		(void) XSetWMColormapWindows(display, window, &window, 1);
+		XInstallColormap(display, swirl->cmap);
+#endif
+	}
+	/* resolution starts off chunky */
+	swirl->resolution = MIN_RES + 1;
+
+	/* calculate the pixel step for this resulution */
+	swirl->r = (1 << (swirl->resolution - 1));
+
+	/* how many knots? */
+	swirl->n_knots = random_no((unsigned int) MI_BATCHCOUNT(mi) / 2) +
+		MI_BATCHCOUNT(mi) + 1;
+
+	/* what type of knots? */
+	swirl->knot_type = ALL;	/* for now */
+
+	/* use two_plane mode occaisionally */
+	if (random_no(100) <= TWO_PLANE_PCNT) {
+		swirl->two_plane = swirl->first_plane = True;
+		swirl->max_resolution = 2;
+	} else
+		swirl->two_plane = False;
+
+	/* fix the knot values */
+	create_knots(swirl);
+
+	/* we are off */
+	swirl->started = True;
+	swirl->drawing = False;
+}
+
+/****************************************************************/
+
 /* 
  * draw_swirl
  *
@@ -1332,24 +1344,20 @@ next_point(SWIRL_P swirl)
 void
 draw_swirl(ModeInfo * mi)
 {
-	Window      win = MI_WINDOW(mi);
-	SWIRL_P     swirl;
-
-	/* get the current swirl */
-	swirl = &(swirls[screen]);
+	SWIRL_P     swirl = &(swirls[MI_SCREEN(mi)]);
 
 	/* are we going? */
 	if (swirl->started) {
 		/* in the middle of drawing? */
 		if (swirl->drawing) {
 			/* rotate the colours */
-			install_map(swirl, swirl->dshift);
+			install_map(MI_DISPLAY(mi), swirl, swirl->dshift);
 
 			/* draw a batch of points */
 			swirl->batch_todo = BATCH_DRAW;
 			while ((swirl->batch_todo > 0) && swirl->drawing) {
 				/* draw a point */
-				draw_point(swirl, win);
+				draw_point(mi, swirl);
 
 				/* move to the next point */
 				next_point(swirl);
@@ -1359,7 +1367,7 @@ draw_swirl(ModeInfo * mi)
 			}
 		} else {
 			/* rotate the colours */
-			install_map(swirl, swirl->shift);
+			install_map(MI_DISPLAY(mi), swirl, swirl->shift);
 
 			/* time for a higher resolution? */
 			if (swirl->resolution > swirl->max_resolution) {
@@ -1400,3 +1408,43 @@ draw_swirl(ModeInfo * mi)
 }
 
 /****************************************************************/
+
+void
+release_swirl(ModeInfo * mi)
+{
+	/* does the swirls array exist? */
+	if (swirls != NULL) {
+		int         i;
+
+		/* free them all */
+		for (i = 0; i < MI_NUM_SCREENS(mi); i++) {
+			SWIRL_P     swirl = &(swirls[i]);
+
+			if (swirl->cmap != (Colormap) NULL)
+				XFreeColormap(MI_DISPLAY(mi), swirl->cmap);
+			if (swirl->rgb_values != NULL)
+				XFree((void *) swirl->rgb_values);
+			if (swirl->ximage != NULL)
+				XDestroyImage(swirl->ximage);
+			if (swirl->knots)
+				(void) free((void *) swirl->knots);
+		}
+		/* deallocate an array, one entry for each screen */
+		(void) free((void *) swirls);
+		swirls = NULL;
+	}
+}
+
+/****************************************************************/
+
+void
+refresh_swirl(ModeInfo * mi)
+{
+	SWIRL_P     swirl = &(swirls[MI_SCREEN(mi)]);
+
+	if (swirl->started) {
+		if (swirl->drawing)
+			swirl->resolution = swirl->resolution + 1;
+		swirl->drawing = False;
+	}
+}
