@@ -1,6 +1,6 @@
 
-#ifndef lint
-static char sccsid[] = "@(#)triangle.c	3.11 96/09/20 xlockmore";
+#if !defined( lint ) && !defined( SABER )
+static const char sccsid[] = "@(#)triangle.c	4.00 97/01/01 xlockmore";
 
 #endif
 
@@ -14,16 +14,16 @@ static char sccsid[] = "@(#)triangle.c	3.11 96/09/20 xlockmore";
  * Revision History:
  * 10-Mar-96: re-arranged and re-formatted the code for appearance and
  *	to make common subroutines.  Simplified.
- *	Ron Hitchens <ronh@utw.com>
+ *	Ron Hitchens <ron@idiom.com>
  * 07-Mar-96: Removed internal delay code, set MI_PAUSE(mi) for inter-scene
  *	delays.  No other delays are needed here.
  *	Made pause time sensitive to value of cycles (in 10ths of a second)
  *	Removed (hopefully) all references to globals.
- *	Ron Hitchens <ronh@utw.com>
+ *	Ron Hitchens <ron@idiom.com>
  * 27-Feb-96 Undid the changes listed below.  Added ModeInfo argument.
  *	Implemented delay between scenes using the MI_PAUSE(mi) scheme.
- *	Ron Hitchens <ronh@utw.com>
- * 27-Dec-95 Ron Hitchens <ronh@utw.com>
+ *	Ron Hitchens <ron@idiom.com>
+ * 27-Dec-95 Ron Hitchens <ron@idiom.com>
  *      Modified logic of draw_triangle() to provide a delay (sensitive
  *      to the value of cycles) between each iteration.  Because this
  *      mode is so compute intensive, when the new event loop adjusted
@@ -35,7 +35,6 @@ static char sccsid[] = "@(#)triangle.c	3.11 96/09/20 xlockmore";
  * 01-Oct-95 Written by Tobias Gloth
  */
 
-#include <math.h>
 #include "xlock.h"
 
 #define MAX_STEPS 8
@@ -257,14 +256,14 @@ draw_triangle(ModeInfo * mi)
 		 TODO: This is wrong for multi-screens ***
 		 */
 		if (tp->init_now) {
-			MI_PAUSE(mi) = MI_CYCLES(mi) * 100000;
+			MI_PAUSE(mi) = 2000000;
 		}
 		return;
 	}
 	if (!(++tp->stage)) {
 		tp->h[0][0] = (short int) MAX(0, DISPLACE(0, tp->delta[0]));
 		tp->h[tp->size][0] = (short int) MAX(0, DISPLACE(0, tp->delta[0]));
-		tp->h[0][tp->size] = (short int)MAX(0, DISPLACE(0, tp->delta[0]));
+		tp->h[0][tp->size] = (short int) MAX(0, DISPLACE(0, tp->delta[0]));
 	} else {
 		d = 2 << (tp->steps - tp->stage);
 		d2 = d / 2;
@@ -273,11 +272,11 @@ draw_triangle(ModeInfo * mi)
 		for (i = 0; i < tp->size; i += d) {
 			for (j = 0; j < (tp->size - i); j += d) {
 				tp->h[i + d2][j] = (short int) DISPLACE(tp->h[i][j] +
-						tp->h[i + d][j], delta);
+						     tp->h[i + d][j], delta);
 				tp->h[i][j + d2] = (short int) DISPLACE(tp->h[i][j] +
-						 tp->h[i][j + d], delta);
+						     tp->h[i][j + d], delta);
 				tp->h[i + d2][j + d2] = (short int) DISPLACE(tp->h[i + d][j] +
-						 tp->h[i][j + d], delta);
+						     tp->h[i][j + d], delta);
 			}
 
 			tp->init_now = 0;
