@@ -1,6 +1,6 @@
 
-#ifndef lint
-static char sccsid[] = "@(#)utils.c	3.11 96/09/20 xlockmore";
+#if !defined( lint ) && !defined( SABER )
+static const char sccsid[] = "@(#)utils.c	4.00 97/01/01 xlockmore";
 
 #endif
 
@@ -11,9 +11,9 @@ static char sccsid[] = "@(#)utils.c	3.11 96/09/20 xlockmore";
  *
  * Revision History:
  *
- * Changes of David Bagley <bagleyd@hertz.njit.edu>
+ * Changes of David Bagley <bagleyd@bigfoot.com>
  *  4-Apr-96: Added procedures to handle wildcards on filenames
- *            J.Jansen <joukj@crys.chem.uva.nl>
+ *            J. Jansen <joukj@crys.chem.uva.nl>
  * 15-May-95: random number generator added, moved hsbramp.c to utils.c .
  *            Also renamed file from usleep.c to utils.c .
  * 14-Mar-95: patches for rand and seconds for VMS
@@ -35,24 +35,34 @@ static char sccsid[] = "@(#)utils.c	3.11 96/09/20 xlockmore";
 
 #include "xlock.h"
 
-#ifdef OLD_EVENT_LOOP
-#if defined( SYSV ) || defined( SVR4 )
+#ifdef USE_OLD_EVENT_LOOP
+#if !defined( VMS ) || defined( XVMSUTILS ) || ( __VMS_VER >= 70000000 )
+#ifdef USE_XVMSUTILS
+#include <X11/unix_time.h>
+#endif
+#if HAVE_SYS_TIME_H
 #include <sys/time.h>
+#else
+#if HAVE_SYS_SELECT_H
+#include <sys/select.h>
+#endif
+#endif
+#endif
+#if defined( SYSV ) || defined( SVR4 )
 #ifdef LESS_THAN_AIX3_2
 #include <sys/poll.h>
 #else /* !LESS_THAN_AIX3_2 */
 #include <poll.h>
 #endif /* !LESS_THAN_AIX3_2 */
-
 #endif /* defined( SYSV ) || defined( SVR4 ) */
 
-#ifndef HAS_USLEEP
+#ifndef HAVE_USLEEP
  /* usleep should be defined */
 int
 usleep(unsigned long usec)
 {
 #if (defined( SYSV ) || defined( SVR4 )) && !defined( __hpux )
-#ifdef HAS_NANOSLEEP
+#ifdef HAVE_NANOSLEEP
 	{
 		struct timespec rqt;
 
@@ -60,11 +70,17 @@ usleep(unsigned long usec)
 		rqt.tv_sec = usec / (unsigned long) 1000000;
 		return nanosleep(&rqt, NULL);
 	}
-#else
+#else /* !HAVE_NANOSLEEP */
 	(void) poll((void *) 0, (int) 0, usec / 1000);	/* ms resolution */
-#endif
+#endif /* !HAVE_NANOSLEEP */
+#else /* !SYSV */
+#if HAVE_GETTIMEOFDAY
+	struct timeval time_out;
+
+	time_out.tv_usec = usec % (unsigned long) 1000000;
+	time_out.tv_sec = usec / (unsigned long) 1000000;
+	(void) select(0, (void *) 0, (void *) 0, (void *) 0, &time_out);
 #else
-#if defined( VMS ) && ( __VMS_VER < 70000000 )
 	long        timadr[2];
 
 	if (usec != 0) {
@@ -74,19 +90,12 @@ usleep(unsigned long usec)
 		sys$setimr(4, &timadr, 0, 0, 0);
 		sys$waitfr(4);
 	}
-#else
-	struct timeval time_out;
-	extern int  select();
-
-	time_out.tv_usec = usec % (unsigned long) 1000000;
-	time_out.tv_sec = usec / (unsigned long) 1000000;
-	(void) select(0, (void *) 0, (void *) 0, (void *) 0, &time_out);
-#endif
 #endif
+#endif /* !SYSV */
 	return 0;
 }
-#endif
-#endif
+#endif /* !HAVE_USLEEP */
+#endif /* USE_OLD_EVENT_LOOP */
 
 /* 
  * returns the number of seconds since 01-Jan-70.
@@ -95,28 +104,28 @@ usleep(unsigned long usec)
 long
 seconds(void)
 {
-#if defined( VMS ) && ( __VMS_VER < 70000000 )
-	return (long) time(NULL);
-#else
+#if HAVE_GETTIMEOFDAY
 	struct timeval now;
 
 	(void) gettimeofday(&now, NULL);
 	return now.tv_sec;
+#else
+	return (long) time(NULL);
 #endif
 }
 
 static int
 readable(char *filename)
 {
-	FILE       *file;
+	FILE       *fp;
 
-	if (!(file = fopen(filename, "rb")))
+	if ((fp = fopen(filename, "rb")) == NULL)
 		return False;
-	(void) fclose(file);
+	(void) fclose(fp);
 	return True;
 }
 
-#if defined( ultrix ) || (defined( VMS ) && ( __VMS_VER < 70000000 ))
+#if (! HAVE_STRDUP )
 char       *
 strdup(char *str)
 {
@@ -183,15 +192,15 @@ isRibbon(void)
 }
 
 char       *
-getWords(void)
+getWords(int screen)
 {
 	FILE       *pp;
-	static char buf[BUFSIZ], progerr[BUFSIZ], progrun[BUFSIZ];
-	register char *p = buf;
+	static char buf[MAXSCREENS][BUFSIZ], progerr[BUFSIZ], progrun[BUFSIZ];
+	register char *p = buf[screen];
 	extern int  pclose(FILE *);
 	int         i;
 
-	buf[0] = '\0';
+	buf[screen][0] = '\0';
 
 	if (strlen(message))
 		getwordsfrom = FROM_RESRC;
@@ -212,68 +221,70 @@ getWords(void)
 	switch (getwordsfrom) {
 #ifndef VMS
 		case FROM_PROGRAM:
-			if ((pp = popen(progrun, "r"))) {
-				while (fgets(p, sizeof (buf) - strlen(buf), pp)) {
-					if (strlen(buf) + 1 < sizeof (buf))
-						p = buf + strlen(buf);
+			/* PURIFY 3.2 on Solaris2 reports duplication of file descriptor on next line.
+			   PURIFY 4.0 on SunOS4 does not report this error */
+			if ((pp = (FILE *) popen(progrun, "r")) != NULL) {
+				while (fgets(p, sizeof (buf[screen]) - strlen(buf[screen]), pp)) {
+					if (strlen(buf[screen]) + 1 < sizeof (buf[screen]))
+						p = buf[screen] + strlen(buf[screen]);
 					else
 						break;
 				}
 				(void) pclose(pp);
-				p = buf;
-				if (!buf[0])
-					(void) sprintf(buf, "\"%s\" produced no output!",
+				p = buf[screen];
+				if (!buf[screen][0])
+					(void) sprintf(buf[screen], "\"%s\" produced no output!",
 					 (!program) ? DEF_PROGRAM : program);
 				else {
 					(void) memset((char *) progerr, 0, sizeof (progerr));
 					(void) strcpy(progerr, "sh: ");
 					strcat_firstword(progerr, (!program) ? DEF_PROGRAM : program);
 					(void) strcat(progerr, ": not found\n");
-					if (!strcmp(buf, progerr))
+					if (!strcmp(buf[screen], progerr))
 						switch (NRAND(12)) {
 							case 0:
-								(void) strcat(buf,
+								(void) strcat(buf[screen],
 									      "( Get with the program, bub. )\n");
 								break;
 							case 1:
-								(void) strcat(buf,
+								(void) strcat(buf[screen],
 									      "( I blow my nose at you, you silly person! )\n");
 								break;
 							case 2:
-								(void) strcat(buf,
+								(void) strcat(buf[screen],
 									      "\nThe resource you want to\nset is `program'.\n");
 								break;
 							case 3:
-								(void) strcat(buf,
+								(void) strcat(buf[screen],
 									      "\nHelp!!  Help!!\nAAAAAAGGGGHHH!!  \n\n");
 								break;
 							case 4:
-								(void) strcat(buf,
+								(void) strcat(buf[screen],
 									      "( Hey, who called me `Big Nose'? )\n");
 								break;
 							case 5:
-								(void) strcat(buf,
+								(void) strcat(buf[screen],
 									      "( Hello?  Are you paying attention? )\n");
 								break;
 							case 6:
-								(void) strcat(buf,
+								(void) strcat(buf[screen],
 									      "sh: what kind of fool do you take me for? \n");
 								break;
 							case 7:
-								(void) strcat(buf,
+								(void) strcat(buf[screen],
 									      "\nRun me with -program \"fortune -o\".\n");
 								break;
 							case 8:
-								(void) strcat(buf,
+								(void) strcat(buf[screen],
 									      "( Where is your fortune? )\n");
 								break;
 							case 9:
-								(void) strcat(buf,
+								(void) strcat(buf[screen],
 									      "( Your fortune has not been written yet!! )");
 								break;
 						}
 				}
-				p = buf;
+				p = buf[screen];
 			} else {
 				perror(progrun);
 				p = def_words;
@@ -281,34 +292,35 @@ getWords(void)
 			break;
 #endif
 		case FROM_FORMATTEDFILE:
-			if ((pp = fopen(messagesfile, "r"))) {
+			if ((pp = fopen(messagesfile, "r")) != NULL) {
 				int         len_mess_file;
 
 				if (fscanf(pp, "%d", &len_mess_file)) {
 					int         no_quote;
 
 					if (len_mess_file <= 0)
-						buf[0] = '\0';
+						buf[screen][0] = '\0';
 					else {
-						(void) fgets(p, sizeof (buf) - strlen(buf), pp);
+						(void) fgets(p, sizeof (buf[screen]) - strlen(buf[screen]), pp);
 						/* get first '%%' (the one after the number of quotes) */
-						(void) fgets(p, sizeof (buf) - strlen(buf), pp);
+						(void) fgets(p, sizeof (buf[screen]) - strlen(buf[screen]), pp);
 						no_quote = NRAND(len_mess_file);
 						for (i = 0; i <= no_quote; ++i) {
 							unsigned int len_cur = 0;
 
-							buf[0] = '\0';
-							p = buf + strlen(buf);
-							while (fgets(p, sizeof (buf) - strlen(buf), pp)) {
-								if (strlen(buf) + 1 < sizeof (buf)) {
+							buf[screen][0] = '\0';
+							p = buf[screen] + strlen(buf[screen]);
+							while (fgets(p,
+								     sizeof (buf[screen]) - strlen(buf[screen]), pp)) {
+								if (strlen(buf[screen]) + 1 < sizeof (buf[screen])) {
 									/* a line with '%%' contains 3 characters */
-									if ((strlen(buf) == len_cur + 3) &&
+									if ((strlen(buf[screen]) == len_cur + 3) &&
 									    (p[0] == '%') && (p[1] == '%')) {
-										p[0] = '\0';	/* get rid of "%%" in buf */
+										p[0] = '\0';	/* get rid of "%%" in buf[screen] */
 										break;
 									} else {
-										p = buf + strlen(buf);
-										len_cur = strlen(buf);
+										p = buf[screen] + strlen(buf[screen]);
+										len_cur = strlen(buf[screen]);
 									}
 								} else
 									break;
@@ -316,34 +328,38 @@ getWords(void)
 						}
 					}
 					(void) fclose(pp);
-					if (!buf[0])
-						(void) sprintf(buf, "file \"%s\" is empty!", messagesfile);
-					p = buf;
+					if (!buf[screen][0])
+						(void) sprintf(buf[screen],
+							       "file \"%s\" is empty!", messagesfile);
+					p = buf[screen];
 				} else {
-					(void) sprintf(buf, "file \"%s\" not in correct format!",
-						       messagesfile);
-					p = buf;
+					(void) sprintf(buf[screen],
+						       "file \"%s\" not in correct format!", messagesfile);
+					p = buf[screen];
 				}
 			} else {
-				(void) sprintf(buf, "could not read file \"%s\"!", messagesfile);
-				p = buf;
+				(void) sprintf(buf[screen],
+				"could not read file \"%s\"!", messagesfile);
+				p = buf[screen];
 			}
 			break;
 		case FROM_FILE:
-			if ((pp = fopen(messagefile, "r"))) {
-				while (fgets(p, sizeof (buf) - strlen(buf), pp)) {
-					if (strlen(buf) + 1 < sizeof (buf))
-						p = buf + strlen(buf);
+			if ((pp = fopen(messagefile, "r")) != NULL) {
+				while (fgets(p, sizeof (buf[screen]) - strlen(buf[screen]), pp)) {
+					if (strlen(buf[screen]) + 1 < sizeof (buf[screen]))
+						p = buf[screen] + strlen(buf[screen]);
 					else
 						break;
 				}
 				(void) fclose(pp);
-				if (!buf[0])
-					(void) sprintf(buf, "file \"%s\" is empty!", messagefile);
-				p = buf;
+				if (!buf[screen][0])
+					(void) sprintf(buf[screen],
+					"file \"%s\" is empty!", messagefile);
+				p = buf[screen];
 			} else {
-				(void) sprintf(buf, "could not read file \"%s\"!", messagefile);
-				p = buf;
+				(void) sprintf(buf[screen],
+				 "could not read file \"%s\"!", messagefile);
+				p = buf[screen];
 			}
 			break;
 		case FROM_RESRC:
@@ -383,45 +399,50 @@ getFont(Display * display)
 static void
 randomImage(char *im_file)
 {
-#if !defined( VMS ) || defined( XVMSUTILS ) || ( __VMS_VER >= 70000000 )
-	extern char directory_[512];
+#if HAVE_DIRENT_H
+	extern char directory_r[DIRBUF];
 	extern struct dirent **image_list;
 	extern int  num_list;
 	int         num;
 
 	if (num_list > 0) {
 		num = NRAND(num_list);
-		(void) strcpy(im_file, directory_);
+		(void) strcpy(im_file, directory_r);
 		(void) strcat(im_file, image_list[num]->d_name);
 	}
 #endif
 }
 
-#if !defined( VMS ) || defined( XVMSUTILS ) ||  ( __VMS_VER >= 70000000 )
+#if HAVE_DIRENT_H
 
-/* index_ emulation of FORTRAN's index in C. * Author : J.Jansen */
+extern Bool debug;
+
+/* index_dir emulation of FORTRAN's index in C. Author: J. Jansen */
 static int
-index_(char *str1, char *substr)
+index_dir(char *str1, char *substr)
 {
-	int         i, num, l1 = strlen(str1), ls = strlen(substr);
-	char       *str1_, *substr_, *substr_last;
+	int         i, num, l1 = strlen(str1), ls = strlen(substr), found;
+	char       *str1_tmp, *substr_tmp, *substr_last;
 
 	num = l1 - ls + 1;
 	substr_last = substr + ls;
 	for (i = 0; i < num; ++i) {
-		str1_ = str1 + i;
-		substr_ = substr;
-		while (substr_ < substr_last)
-			if (*str1_++ != *substr_++)
-				goto not_found;
-		return (i + 1);
-	      not_found:;
+		str1_tmp = str1 + i;
+		substr_tmp = substr;
+		found = 1;
+		while (substr_tmp < substr_last)
+			if (*str1_tmp++ != *substr_tmp++) {
+				found = 0;
+				break;
+			}
+		if (found)
+			return (i + 1);
 	}
 	return (0);
 }
 
 #ifdef VMS
-/* Upcase string * Author J.Jansen */
+/* Upcase string  Author: J. Jansen */
 void
 upcase(char *s)
 {
@@ -438,75 +459,81 @@ upcase(char *s)
 
 #endif
 
-/* Split full path into directory and filename parts * author : J.Jansen */
+/* Split full path into directory and filename parts  Author: J. Jansen */
 void
-get_direc(char *fullpath, char *dir, char *filn_)
+get_dir(char *fullpath, char *dir, char *filename)
 {
 	char       *ln;
-	int         ip_ = 0, ip;
+	int         ip_temp = 0, ip;
 
 #ifdef VMS
-	ip = index_(fullpath, "]");
+	ip = index_dir(fullpath, "]");
 #else
 	ln = fullpath;
 	ip = 0;
-	while ((ip_ = index_(ln, "/"))) {
-		ip = ip + ip_;
+	while ((ip_temp = index_dir(ln, "/"))) {
+		ip = ip + ip_temp;
 		ln = fullpath + ip;
 	}
 #endif
 	if (ip == 0) {
-		(void) strcpy(filn_, fullpath);
 #ifdef VMS
 		(void) strcpy(dir, "[]");
 #else
 		(void) strcpy(dir, "./");
 #endif
 	} else {
-		(void) strcpy(dir, "\0");
-		(void) strncat(dir, fullpath, ip);
+		if (ip > DIRBUF - 1)
+			ip_temp = DIRBUF - 1;
+		else
+			ip_temp = ip;
+		(void) strncpy(dir, fullpath, ip_temp);
+		dir[ip_temp] = '\0';
 	}
 	ln = fullpath + ip;
-	(void) strcpy(filn_, ln);
+	(void) strncpy(filename, ln, MAXNAMLEN - 1);
+	filename[MAXNAMLEN - 1] = '\0';
 #ifdef VMS
-	upcase(filn_);		/* VMS knows uppercase filenames only */
+	upcase(filename);	/* VMS knows uppercase filenames only */
 #endif
+	if (debug)
+		(void) printf("get_dir %s %s %s\n", fullpath, dir, filename);
 }
 
-/* Procedure to select the matching filenames * author : J.Jansen */
+/* Procedure to select the matching filenames  Author: J. Jansen */
 int
 sel_image(struct dirent *name)
 {
-	extern char filenam_[256];
-	char       *nam_ = name->d_name;
-	char       *fil_ = filenam_;
+	extern char filename_r[MAXNAMLEN];
+	char       *name_tmp = name->d_name;
+	char       *filename_tmp = filename_r;
 	static int  numfrag = -1;
 	static char *frags[64];
 	int         ip, i;
 
 	if (numfrag == -1) {
 		++numfrag;
-		while ((ip = index_(fil_, "*"))) {
+		while ((ip = index_dir(filename_tmp, "*"))) {
 			frags[numfrag] = (char *) malloc(ip);
 			(void) strcpy(frags[numfrag], "\0");
-			(void) strncat(frags[numfrag], fil_, ip - 1);
+			(void) strncat(frags[numfrag], filename_tmp, ip - 1);
 			++numfrag;
-			fil_ = fil_ + ip;
+			filename_tmp = filename_tmp + ip;
 		}
-		frags[numfrag] = (char *) malloc(strlen(fil_));
-		(void) strcpy(frags[numfrag], fil_);
+		frags[numfrag] = (char *) malloc(strlen(filename_tmp));
+		(void) strcpy(frags[numfrag], filename_tmp);
 	}
 	for (i = 0; i <= numfrag; ++i) {
-		ip = index_(nam_, frags[i]);
+		ip = index_dir(name_tmp, frags[i]);
 		if (ip == 0)
 			return (0);
-		nam_ = nam_ + ip;
+		name_tmp = name_tmp + ip;
 	}
 
 	return (1);
 }
 
-/* scandir implementiation for VMS * Author : J.Jansen */
+/* scandir implementiation for VMS  Author: J. Jansen */
 /* name changed to scan_dir to solve portablity problems */
 #define _MEMBL_ 64
 int
@@ -514,46 +541,61 @@ scan_dir(const char *directoryname, struct dirent ***namelist,
 	 int         (*select) (struct dirent *),
 	 int         (*compare) (const void *, const void *))
 {
-	DIR        *dir_;
-	struct dirent *new_entry, **namelist_;
-	int         size_, num_list_;
+	DIR        *dirp;
+	struct dirent *new_entry, **namelist_tmp;
+	int         size_tmp, num_list_tmp;
 
-	if ((dir_ = opendir(directoryname)) == NULL)
+	if ((dirp = opendir(directoryname)) == NULL)
 		return (-1);
-	size_ = _MEMBL_;
-	namelist_ = (struct dirent **) malloc(size_ * sizeof (struct dirent *));
+	size_tmp = _MEMBL_;
+	namelist_tmp = (struct dirent **) malloc(size_tmp * sizeof (struct dirent *));
 
-	if (namelist_ == NULL)
+	if (namelist_tmp == NULL)
 		return (-1);
-	num_list_ = 0;
-	while ((new_entry = readdir(dir_)) != NULL) {
+	num_list_tmp = 0;
+	while ((new_entry = readdir(dirp)) != NULL) {
 #ifndef VMS
-		if (!strcmp(new_entry->d_name, ".."))
+		if (!strcmp(new_entry->d_name, ".") && !strcmp(new_entry->d_name, ".."))
 			continue;
 #endif
-
 		if (select != NULL && !(*select) (new_entry))
 			continue;
-		if (++num_list_ >= size_) {
-			size_ = size_ + _MEMBL_;
-			namelist_ = (struct dirent **) realloc((void *) namelist_, size_ *
-						   sizeof (struct dirent *));
+		if (++num_list_tmp >= size_tmp) {
+			size_tmp = size_tmp + _MEMBL_;
+			namelist_tmp = (struct dirent **) realloc(
+									 (void *) namelist_tmp, size_tmp * sizeof (struct dirent *));
 
-			if (namelist_ == NULL)
+			if (namelist_tmp == NULL)
 				return (-1);
 		}
-		namelist_[num_list_ - 1] = (struct dirent *) malloc(sizeof (struct
-								    dirent));
+		/* Core Dumps here for Solaris2 if files names in dir are > 14 characters */
+		namelist_tmp[num_list_tmp - 1] =
+			(struct dirent *) malloc(sizeof (struct dirent));
 
-		(void) strcpy(namelist_[num_list_ - 1]->d_name, new_entry->d_name);
+		(void) strcpy(namelist_tmp[num_list_tmp - 1]->d_name, new_entry->d_name);
+
+		*namelist = namelist_tmp;
+#ifdef SOLARIS2
+		if (debug)
+			(void) printf("file %d: %s\n",
+				      num_list_tmp - 1, namelist_tmp[num_list_tmp - 1]->d_name);
+#endif
 	}
-	(void) closedir(dir_);
-	if (num_list_ && compare != NULL)
-		(void) qsort((void *) namelist_, num_list_,
+#ifdef SOLARIS2			/* Data seems to get corrupted for Solaris2 */
+	if (debug) {
+		(void) printf("directory: %s\n", directoryname);
+		for (size_tmp = 0; size_tmp < num_list_tmp; size_tmp++)
+			(void) printf("file %d: %s\n", size_tmp, namelist_tmp[size_tmp]->d_name);
+	}
+#endif
+
+	(void) closedir(dirp);
+	if (num_list_tmp && compare != NULL)
+		(void) qsort((void *) namelist_tmp, num_list_tmp,
 			     sizeof (struct dirent *), compare);
 
-	*namelist = namelist_;
-	return (num_list_);
+	*namelist = namelist_tmp;
+	return (num_list_tmp);
 }
 
 #endif
@@ -561,11 +603,11 @@ scan_dir(const char *directoryname, struct dirent ***namelist,
 extern int  XbmReadFileToImage(char *filename,
 			       int *width, int *height, unsigned char **bits);
 
-#ifdef HAS_XPM
-#if 1
-#include <X11/xpm.h>
-#else
+#if defined( USE_XPM ) || defined( USE_XPMINC )
+#if USE_XPMINC
 #include <xpm.h>
+#else
+#include <X11/xpm.h>		/* Normal spot */
 #endif
 #endif
 #include "ras.h"
@@ -580,23 +622,37 @@ static XImage blogo =
 
 void
 getImage(ModeInfo * mi, XImage ** logo,
-	 int width, int height, unsigned char *bits, char **name,
-	 int *graphics_format, Bool * newcolormap)
+	 int width, int height, unsigned char *bits,
+#if defined( USE_XPM ) || defined( USE_XPMINC )
+	 char **name,
+#endif
+	 int *graphics_format, Colormap * ncm,
+	 unsigned long *blackpix, unsigned long *whitepix)
 {
 	extern char *imagefile;
 	Display    *display = MI_DISPLAY(mi);
+	Window      window = MI_WINDOW(mi);
 
-#ifdef HAS_XPM
+#if defined( USE_XPM ) || defined( USE_XPMINC )
 	XpmAttributes attrib;
-	int         screen = MI_SCREEN(mi);
 
-	attrib.visual = DefaultVisual(display, screen);
-	attrib.colormap = DefaultColormap(display, screen);
-	attrib.depth = DefaultDepth(display, screen);
+#endif
+
+	if (MI_WIN_IS_INSTALL(mi) && !MI_WIN_IS_INROOT(mi))
+		*ncm = XCreateColormap(display, window, MI_VISUAL(mi), AllocNone);
+	else
+		*ncm = None;
+
+#if defined( USE_XPM ) || defined( USE_XPMINC )
+	attrib.visual = MI_VISUAL(mi);
+	if (*ncm != None)
+		attrib.colormap = *ncm;
+	else
+		attrib.colormap = MI_COLORMAP(mi);
+	attrib.depth = MI_WIN_DEPTH(mi);
 	attrib.valuemask = XpmVisual | XpmColormap | XpmDepth;
 #endif
 
-	*newcolormap = False;
 	*graphics_format = 0;
 
 	randomImage(imagefile);
@@ -605,18 +661,28 @@ getImage(ModeInfo * mi, XImage ** logo,
 		if (readable(imagefile)) {
 			if (MI_NPIXELS(mi) > 2) {
 				if (RasterSuccess == RasterFileToImage(display, imagefile, logo
-				/* , MI_FG_COLOR(mi), MI_BG_COLOR(mi) */ ))
+				/* , MI_FG_COLOR(mi), MI_BG_COLOR(mi) */ )) {
 					*graphics_format = IS_RASTERFILE;
-				if (*graphics_format == IS_RASTERFILE &&
-				    MI_WIN_IS_INSTALL(mi) && !MI_WIN_IS_INROOT(mi))
-					*newcolormap = True;
-#ifdef HAS_XPM
-				if (*graphics_format <= 0)
+					if (MI_WIN_IS_INSTALL(mi) && !MI_WIN_IS_INROOT(mi))
+						SetImageColors(display, *ncm);
+					*blackpix = GetBlack();
+					*whitepix = GetWhite();
+				}
+			}
+#if defined( USE_XPM ) || defined( USE_XPMINC )
+#ifndef USE_MONOXPM
+			if (MI_NPIXELS(mi) > 2)
+#endif
+			{
+				if (*graphics_format <= 0) {
+					if (*ncm != None)
+						reserveColors(mi, *ncm, blackpix, whitepix);
 					if (XpmSuccess == XpmReadFileToImage(display, imagefile, logo,
 						  (XImage **) NULL, &attrib))
 						*graphics_format = IS_XPMFILE;
-#endif
+				}
 			}
+#endif
 			if (*graphics_format <= 0)
 				if (!blogo.data) {
 					if (BitmapSuccess == XbmReadFileToImage(imagefile,
@@ -635,11 +701,14 @@ getImage(ModeInfo * mi, XImage ** logo,
 		} else
 			(void) fprintf(stderr,
 				  "could not read file \"%s\"\n", imagefile);
-#ifdef HAS_XPM
-	if (MI_NPIXELS(mi) > 2 && *graphics_format <= 0)
-		if (XpmSuccess == XpmCreateImageFromData(display, name,
+#if defined( USE_XPM ) || defined( USE_XPMINC )
+	if (*graphics_format <= 0)
+#ifndef USE_MONOXPM
+		if (MI_NPIXELS(mi) > 2)
+#endif
+			if (XpmSuccess == XpmCreateImageFromData(display, name,
 					    logo, (XImage **) NULL, &attrib))
-			*graphics_format = IS_XPM;
+				*graphics_format = IS_XPM;
 #endif
 	if (*graphics_format <= 0) {
 		if (!blogo.data) {
@@ -652,6 +721,11 @@ getImage(ModeInfo * mi, XImage ** logo,
 			*graphics_format = IS_XBMDONE;
 		*logo = &blogo;
 	}
+	if (*ncm != None && *graphics_format != IS_RASTERFILE &&
+	    *graphics_format != IS_XPMFILE && *graphics_format != IS_XPM) {
+		XFreeColormap(display, *ncm);
+		*ncm = None;
+	}
 }
 
 void
