@@ -1,5 +1,5 @@
-#ifndef lint
-static char sccsid[] = "@(#)wator.c	3.11 96/09/20 xlockmore";
+#if !defined( lint ) && !defined( SABER )
+static const char sccsid[] = "@(#)wator.c	4.00 97/01/01 xlockmore";
 
 #endif
 
@@ -44,12 +44,12 @@ static char sccsid[] = "@(#)wator.c	3.11 96/09/20 xlockmore";
 #define REFLECTS 2
 #define BITMAPS (ORIENTS*REFLECTS*KINDS)
 #define KINDBITMAPS (ORIENTS*REFLECTS)
+#define MINGRIDSIZE 3		/* It is possible for the fish to take over */
+#define MINSIZE 4
 
 ModeSpecOpt wator_opts =
 {0, NULL, 0, NULL, NULL};
 
-extern int  gridsize;
-
 static XImage logo[BITMAPS] =
 {
 	{0, 0, 0, XYBitmap, (char *) fish0_bits, LSBFirst, 8, LSBFirst, 8, 1},
@@ -114,8 +114,12 @@ drawcell(ModeInfo * mi, int col, int row, unsigned long color, int bitmap)
 	else
 		XSetForeground(display, gc, MI_WIN_WHITE_PIXEL(mi));
 	if (wp->pixelmode)
-		XFillRectangle(display, MI_WINDOW(mi), gc,
-		wp->xb + wp->xs * col, wp->yb + wp->ys * row, wp->xs, wp->ys);
+		if (bitmap >= BITMAPS / 2 || (wp->xs == 1 && wp->ys == 1))
+			XFillRectangle(display, MI_WINDOW(mi), gc,
+				       wp->xb + wp->xs * col, wp->yb + wp->ys * row, wp->xs, wp->ys);
+		else
+			XFillArc(display, MI_WINDOW(mi), gc,
+				 wp->xb + wp->xs * col, wp->yb + wp->ys * row, wp->xs, wp->ys, 0, 23040);
 	else
 		XPutImage(display, MI_WINDOW(mi), gc, &logo[bitmap], 0, 0,
 			  wp->xb + wp->xs * col, wp->yb + wp->ys * row, icon_width, icon_height);
@@ -224,6 +228,7 @@ flush_kindlist(watorstruct * wp, int kind)
 void
 init_wator(ModeInfo * mi)
 {
+	int         size = MI_SIZE(mi);
 	watorstruct *wp;
 	int         i, col, row, colrow, kind;
 	cellstruct  info;
@@ -259,14 +264,32 @@ init_wator(ModeInfo * mi)
 		wp->width = 2;
 	if (wp->height < 2)
 		wp->height = 2;
-	wp->pixelmode = (wp->width + wp->height < 3 * (icon_width + icon_height));
-	if (wp->pixelmode) {
-		wp->ncols = MIN(wp->width / 2, gridsize);
-		wp->nrows = MIN(wp->height / 2, gridsize);
+	if (size == 0 ||
+	 MINGRIDSIZE * size > wp->width || MINGRIDSIZE * size > wp->height) {
+		if (wp->width > MINGRIDSIZE * icon_width &&
+		    wp->height > MINGRIDSIZE * icon_height) {
+			wp->pixelmode = False;
+			wp->xs = icon_width;
+			wp->ys = icon_height;
+		} else {
+			wp->pixelmode = True;
+			wp->xs = wp->ys = MAX(MINSIZE, MIN(wp->width, wp->height) / MINGRIDSIZE);
+		}
 	} else {
-		wp->ncols = MIN(wp->width / icon_width, gridsize);
-		wp->nrows = MIN(wp->height / icon_height, gridsize);
+		wp->pixelmode = True;
+		if (size < -MINSIZE)
+			wp->ys = NRAND(MIN(-size, MAX(MINSIZE, MIN(wp->width, wp->height) /
+				      MINGRIDSIZE)) - MINSIZE + 1) + MINSIZE;
+		else if (size < MINSIZE)
+			wp->ys = MINSIZE;
+		else
+			wp->ys = MIN(size, MAX(MINSIZE, MIN(wp->width, wp->height) /
+					       MINGRIDSIZE));
+		wp->xs = wp->ys;
 	}
+	wp->ncols = MAX(wp->width / wp->xs, 2);
+	wp->nrows = MAX(wp->height / wp->ys, 2);
+
 	if (wp->arr != NULL)
 		(void) free((void *) wp->arr);
 	wp->arr = (CellList **) calloc(wp->ncols * wp->nrows, sizeof (CellList *));
@@ -285,8 +308,6 @@ init_wator(ModeInfo * mi)
 		wp->breed[FISH] = 4;
 	wp->sstarve = 3;
 
-	wp->xs = wp->width / wp->ncols;
-	wp->ys = wp->height / wp->nrows;
 	wp->xb = (wp->width - wp->xs * wp->ncols) / 2;
 	wp->yb = (wp->height - wp->ys * wp->nrows) / 2;
 
