@@ -1,5 +1,5 @@
-#ifndef lint
-static char sccsid[] = "@(#)xlock.c	3.11 96/09/20 xlockmore";
+#if !defined( lint ) && !defined( SABER )
+static const char sccsid[] = "@(#)xlock.c	4.01 97/02/24 xlockmore";
 
 #endif
 
@@ -22,17 +22,17 @@ static char sccsid[] = "@(#)xlock.c	3.11 96/09/20 xlockmore";
  *
  * Revision History:
  *
- * Changes of David Bagley <bagleyd@hertz.njit.edu>
- * 01-Sep-96: Ron Hitchens <ronh@utw.com>
+ * Changes of David Bagley <bagleyd@bigfoot.com>
+ * 01-Sep-96: Ron Hitchens <ron@idiom.com>
  *            Updated xlock so it would refresh more reliably and
  *            handle window resizing.
- * 18-Mar-96: Ron Hitchens <ronh@utw.com>
+ * 18-Mar-96: Ron Hitchens <ron@idiom.com>
  *            Implemented new ModeInfo hook calling scheme.
  *            Created mode_info() to gather and pass info to hooks.
  *            Watch for and use MI_PAUSE value if mode hook sets it.
  *            Catch SIGSEGV, SIGBUS and SIGFPE signals.  Other should
  *            be caught.  Eliminate some globals.
- * 23-Dec-95: Ron Hitchens <ronh@utw.com>
+ * 23-Dec-95: Ron Hitchens <ron@idiom.com>
  *            Rewrote event loop so as not to use signals.
  * 01-Sep-95: initPasswd function, more stuff removed to passwd.c
  * 24-Jun-95: Cut out passwd stuff to passwd.c-> getPasswd & checkPasswd
@@ -157,38 +157,22 @@ static char sccsid[] = "@(#)xlock.c	3.11 96/09/20 xlockmore";
  *
  */
 
+#include "xlock.h"
 #include <signal.h>
-#ifndef NONSTDC
 #include <stdarg.h>
-#endif
 #include <errno.h>
-#include <sys/types.h>
-#ifdef AIXV3
-#include <sys/select.h>
-#else
+#if HAVE_SYS_TIME_H
 #include <sys/time.h>
 #endif
-#include "xlock.h"
+#if HAVE_SYS_SELECT_H && defined(AIXV3)
+#include <sys/select.h>
+#endif
 #include <X11/cursorfont.h>
 #include <X11/Xatom.h>
 #ifdef USE_VROOT
 #include "vroot.h"
 #endif
-#ifdef SYSLOG
-#include <syslog.h>
-#endif
-#ifdef HAS_RPLAY
-#include <rplay.h>
-#endif
-#ifdef VMS_PLAY
-#define rplay_display vms_play
-#define HAS_RPLAY
-#endif
-#ifdef DEF_PLAY
-#define rplay_display system_play
-#define HAS_RPLAY
-#endif
-#if defined( VMS ) && !defined( OLD_EVENT_LOOP ) && ( __VMS_VER < 70000000 )
+#if USE_XVMSUTILS
 #if 0
 #include "../xvmsutils/unix_types.h"
 #include "../xvmsutils/unix_time.h"
@@ -196,10 +180,32 @@ static char sccsid[] = "@(#)xlock.c	3.11 96/09/20 xlockmore";
 #include <X11/unix_types.h>
 #include <X11/unix_time.h>
 #endif
-#endif
-#ifdef DT_SAVER
+#endif /* USE_XVMSUTILS */
+#ifdef USE_DTSAVER
 #include <X11/Intrinsic.h>
 #include <Dt/Saver.h>
+#endif
+#if defined( __hpux ) || defined( __apollo )
+#include <X11/XHPlib.h>
+#endif
+
+#if ( HAVE_SYSLOG_H && defined( USE_SYSLOG ))
+#include <pwd.h>
+#include <grp.h>
+#include <syslog.h>
+#endif
+
+#if ( __VMS_VER >= 70000000 )
+#include <lib$routines.h>
+#include <mail$routines.h>
+#include <maildef.h>
+struct itmlst_3 {
+	unsigned short int buflen;
+	unsigned short int itmcode;
+	void       *bufadr;
+	unsigned short int *retlen;
+};
+
 #endif
 
 extern char *getenv(const char *);
@@ -207,12 +213,13 @@ extern void checkResources(void);
 extern void initPasswd(void);
 extern int  checkPasswd(char *);
 
-#ifndef HAS_USLEEP
+
+#ifdef USE_OLD_EVENT_LOOP
 extern int  usleep(unsigned int);
 
 #endif
 
-#if defined( AUTO_LOGOUT ) || defined( LOGOUT_BUTTON )
+#if defined( USE_AUTO_LOGOUT ) || defined( USE_LOGOUT_BUTTON )
 extern void logoutUser(void);
 
 #endif
@@ -220,6 +227,11 @@ extern void logoutUser(void);
 #if 0				/* #if !defined( AIXV3 ) && !defined( __hpux ) && !defined( __bsdi__ ) */
 extern int  select(int, fd_set *, fd_set *, fd_set *, struct timeval *);
 
+#if !defined( __hpux ) && !defined( apollo )
+extern int  select(size_t, int *, int *, int *, const struct timeval *);
+
+#endif
+
 #endif
 extern int  nice(int);
 
@@ -233,26 +245,28 @@ extern float delta3d;
 extern int  delay;
 extern int  batchcount;
 extern int  cycles;
-extern int  nicelevel;
-extern int  timeout;
-extern int  lockdelay;
+extern int  size;
+extern Bool nolock;
+extern Bool inwindow;
+extern Bool inroot;
+extern Bool mono;
 extern Bool allowaccess;
 extern Bool allowroot;
+extern Bool debug;
 extern Bool echokeys;
 extern Bool enablesaver;
 extern Bool grabmouse;
-extern Bool inroot;
 extern Bool install;
-extern Bool debug;
-extern Bool inwindow;
-extern Bool mono;
-extern Bool nolock;
 extern Bool timeelapsed;
 extern Bool usefirst;
-extern Bool use3d;
 extern Bool verbose;
+extern int  nicelevel;
+extern int  lockdelay;
+extern int  timeout;
+extern Bool use3d;
 
 extern char *fontname;
+extern char *messagefont;
 extern char *background;
 extern char *foreground;
 extern char *text_name;
@@ -267,27 +281,30 @@ extern char *right3d;
 extern char *left3d;
 extern char *both3d;
 
-#if defined( HAS_RPLAY ) || defined( VMS_PLAY ) || defined( DEF_PLAY )
+#ifdef USE_SOUND
 extern char *locksound;
 extern char *infosound;
 extern char *validsound;
 extern char *invalidsound;
+extern void play_sound(char *string);
+extern Bool sound;
 
 #endif
-#ifdef AUTO_LOGOUT
+#ifdef USE_AUTO_LOGOUT
 extern int  forceLogout;
 
 #endif
 
-#ifdef LOGOUT_BUTTON
+#ifdef USE_LOGOUT_BUTTON
 extern int  enable_button;
+extern int  logoutButton;
 extern char *logoutButtonLabel;
 extern char *logoutButtonHelp;
 extern char *logoutFailedString;
 
 #endif
 
-#ifdef DT_SAVER
+#ifdef USE_DTSAVER
 extern Bool dtsaver;
 
 #endif
@@ -299,12 +316,13 @@ static int  screens;		/* number of screens */
 static Window win[MAXSCREENS];	/* window used to cover screen */
 static Window icon[MAXSCREENS];	/* window used during password typein */
 
-#ifdef LOGOUT_BUTTON
+#ifdef USE_LOGOUT_BUTTON
 static Window button[MAXSCREENS];
 
 #endif
 static Window root[MAXSCREENS];	/* convenience pointer to the root window */
 static GC   textgc[MAXSCREENS];	/* graphics context used for text rendering */
+static GC   msgtextgc[MAXSCREENS];	/* graphics context for message rendering */
 static unsigned long fgcol[MAXSCREENS];		/* used for text rendering */
 static unsigned long bgcol[MAXSCREENS];		/* background of text screen */
 static unsigned long nonecol[MAXSCREENS];
@@ -313,6 +331,8 @@ static unsigned long leftcol[MAXSCREENS];
 static unsigned long bothcol[MAXSCREENS];	/* Can change with -install */
 static int  iconx[MAXSCREENS];	/* location of left edge of icon */
 static int  icony[MAXSCREENS];	/* location of top edge of icon */
+static int  msgx[MAXSCREENS];	/* location of left edge of message */
+static int  msgy[MAXSCREENS];	/* location of top edge of message */
 static Cursor mycursor;		/* blank cursor */
 static Pixmap lockc;
 static Pixmap lockm;		/* pixmaps for cursor and mask */
@@ -320,24 +340,27 @@ static char no_bits[] =
 {0};				/* dummy array for the blank cursor */
 static int  passx, passy;	/* position of the ?'s */
 static int  iconwidth, iconheight;
+static int  timex, timey;	/* position for the times */
 static XFontStruct *font;
+static XFontStruct *msgfont;
 static int  sstimeout;		/* screen saver parameters */
 static int  ssinterval;
 static int  ssblanking;
 static int  ssexposures;
 static long start_time;
+static char *message[MESSAGELINES + 1];		/* Message is stored here */
 
 /* GEOMETRY STUFF */
 static int  sizeconfiguremask;
 static XWindowChanges fullsizeconfigure, minisizeconfigure;
 static int  fullscreen = False;
 
-#if defined( LOGOUT_BUTTON ) || defined( AUTO_LOGOUT )
+#if defined( USE_LOGOUT_BUTTON ) || defined( USE_AUTO_LOGOUT )
 static int  tried_logout = 0;
 
 #endif
 
-#ifdef HAS_RPLAY
+#ifdef USE_SOUND
 static int  got_invalid = 0;
 
 #endif
@@ -351,20 +374,7 @@ static int  got_invalid = 0;
 	Button5MotionMask | ButtonMotionMask | \
 	KeymapStateMask)
 
-#ifdef DEF_PLAY
-void
-system_play(char *string)
-{
-	char        progrun[BUFSIZ];
-
-	(void) sprintf(progrun, "( %s%s ) 2>&1", DEF_PLAY, string);
-	system(progrun);
-}
-
-#endif
-
 /* VARARGS1 */
-#ifndef NONSTDC
 void
 error(char *s1,...)
 {
@@ -375,16 +385,10 @@ error(char *s1,...)
 	s2 = va_arg(vl, char *);
 
 	va_end(vl);
-#else
-void
-error(char *s1, char *s2)
-{
-#endif
 	(void) fprintf(stderr, s1, ProgramName, s2);
 	exit(1);
 }
 
-#ifndef NONSTDC
 void
 warning(char *s1,...)
 {
@@ -396,11 +400,6 @@ warning(char *s1,...)
 	s3 = va_arg(vl, char *);
 
 	va_end(vl);
-#else
-void
-warning(char *s1, char *s2, char *s3)
-{
-#endif
 	(void) fprintf(stderr, s1, ProgramName, s2, s3);
 }
 
@@ -470,7 +469,7 @@ GrabKeyboardAndMouse(void)
 static void
 XChangeGrabbedCursor(Cursor cursor)
 {
-  if (!debug && grabmouse && !inwindow && !inroot)
+	if (!debug && grabmouse && !inwindow && !inroot)
 		(void) XGrabPointer(dsp, win[0], True, AllPointerEventMask,
 		    GrabModeAsync, GrabModeAsync, None, cursor, CurrentTime);
 }
@@ -562,19 +561,23 @@ mode_info(int scrn, Window window, int iconic)
 		MI_WINDOW(mi) = window;
 
 		(void) XGetWindowAttributes(dsp, window, &xgwa);
+
 		MI_WIN_WIDTH(mi) = xgwa.width;
 		MI_WIN_HEIGHT(mi) = xgwa.height;
-		MI_WIN_DEPTH(mi) = xgwa.depth;
-		MI_VISUAL(mi) = xgwa.visual;
 	}
 	MI_WIN_SET_FLAG_STATE(mi, WI_FLAG_ICONIC, iconic);
 
+	MI_WIN_DEPTH(mi) = DisplayPlanes(dsp, scrn);
+	MI_VISUAL(mi) = DefaultVisual(dsp, scrn);
+	MI_COLORMAP(mi) = DefaultColormap(dsp, scrn);
+
 	MI_DELTA3D(mi) = delta3d;
 	MI_PAUSE(mi) = 0;	/* use default if mode doesn't change this */
 
 	MI_DELAY(mi) = delay;	/* globals */
 	MI_BATCHCOUNT(mi) = batchcount;
 	MI_CYCLES(mi) = cycles;
+	MI_SIZE(mi) = size;
 	MI_SATURATION(mi) = saturation;
 
 	return (mi);
@@ -582,7 +585,7 @@ mode_info(int scrn, Window window, int iconic)
 
 
 /* Restore all grabs, reset screensaver, restore colormap, close connection. */
-static void
+void
 finish(void)
 {
 #if 0
@@ -669,11 +672,6 @@ putText(Window w, GC gc, char *string, int bold, int left, int *px, int *py)
 	*py = y;
 }
 
-
-#ifndef LO_BUTTON_TIME
-#define LO_BUTTON_TIME 5
-#endif
-
 static void
 statusUpdate(int isnew, int scr)
 {
@@ -682,27 +680,26 @@ statusUpdate(int isnew, int scr)
 	XWindowAttributes xgwa;
 	static int  last_time;
 
-#ifdef LOGOUT_BUTTON
+#ifdef USE_LOGOUT_BUTTON
 	int         ysave;
 	static      made_button, last_tried_lo = 0;
-	static int  loButton = LO_BUTTON_TIME;
 
-#endif /* LOGOUT_BUTTON */
+#endif /* USE_LOGOUT_BUTTON */
 
 	now = seconds();
 	len = (now - start_time) / 60;
 
-#ifdef LOGOUT_BUTTON
+#ifdef USE_LOGOUT_BUTTON
 	if (tried_logout && !last_tried_lo) {
 		last_tried_lo = 1;
 		isnew = 1;
-#ifdef AUTO_LOGOUT
+#ifdef USE_AUTO_LOGOUT
 		forceLogout = 0;
 #endif
 	}
 	if (isnew)
 		made_button = 0;
-#endif /* LOGOUT_BUTTON */
+#endif /* USE_LOGOUT_BUTTON */
 
 	if (isnew || last_time != len)
 		last_time = len;
@@ -710,21 +707,82 @@ statusUpdate(int isnew, int scr)
 		return;
 
 	(void) XGetWindowAttributes(dsp, win[scr], &xgwa);
-	x = left = iconx[scr];
-	y = xgwa.height - (font->descent + 2);
-#ifdef AUTO_LOGOUT
-	if (forceLogout)
-		y -= (font->ascent + font->descent + 2);
-#endif
-#ifdef LOGOUT_BUTTON
-	if (enable_button) {
-		if (loButton > len) {
-			y -= (font->ascent + font->descent + 2);
+	x = left = timex;
+	y = timey;
+
+	if (timeelapsed) {
+		if (len < 60)
 			(void) sprintf(buf,
-				       "%d minute%s until public logout button appears.    \n",
-			     loButton - len, loButton - len == 1 ? "" : "s");
+#ifdef NL
+				       "%d minute%s op slot.                                        \n",
+				       len, len == 1 ? "" : "n");
+#else
+#ifdef DE
+				       "Seit %d Minute%s gesperrt.                                  \n",
+				       len, len == 1 ? "" : "n");
+#else
+				       "%d minute%s elapsed since locked.                           \n",
+				       len, len == 1 ? "" : "s");
+#endif
+#endif
+		else
+			(void) sprintf(buf,
+#ifdef NL
+				       "%d:%02d uur op slot.                                           \n",
+#else
+#ifdef DE
+				       "Seit %d:%02d Stunden gesperrt.                                 \n",
+#else
+				       "%d:%02d hours elapsed since locked.                            \n",
+#endif
+#endif
+				       len / 60, len % 60);
+		putText(win[scr], textgc[scr], buf, False, left, &x, &y);
+	}
+#ifdef USE_LOGOUT_BUTTON
+	if (enable_button) {
+		if (logoutButton > len) {
+			int         tmp = logoutButton - len;
+
+			if (tmp < 60)
+				(void) sprintf(buf,
+#ifdef NL
+					       "Over %d minute%s verschijnt de " logout " knop.                 \n",
+					       tmp, (tmp == 1) ? "" : "n");
+#else
+#ifdef DE
+					       "In %d Minute%s erscheint der Auslogger.                       \n",
+					       tmp, (tmp == 1) ? "" : "n");
+#else
+					       "%d minute%s until the public logout button appears.           \n",
+					       tmp, (tmp == 1) ? "" : "s");
+#endif
+#endif
+			else
+				(void) sprintf(buf,
+#ifdef NL
+					       "Over %d:%02d uur verschijnt de " logout " knop.               \n",
+#else
+#ifdef DE
+					       "In %d:%02d Stunden erscheint der Auslogger.                 \n",
+#else
+					       "%d:%02d hours until the public logout button appears.       \n",
+#endif
+#endif
+					       tmp / 60, tmp % 60);
 			putText(win[scr], textgc[scr], buf, False, left, &x, &y);
 		} else if (!made_button || (isnew && tried_logout)) {
+			putText(win[scr], textgc[scr],
+#ifdef NL
+				"Schop me eruit                                               \n",
+#else
+#ifdef DE
+				"Werft mich raus                                              \n",
+#else
+				"Kick me out                                                  \n",
+#endif
+#endif
+				False, left, &x, &y);
 			made_button = 1;
 			ysave = y;
 			y = xgwa.height * 0.6;
@@ -755,30 +813,23 @@ statusUpdate(int isnew, int scr)
 			x = left;
 		}
 	}
-#endif /* LOGOUT_BUTTON */
-#ifdef AUTO_LOGOUT
+#endif /* USE_LOGOUT_BUTTON */
+#ifdef USE_AUTO_LOGOUT
 	if (forceLogout) {
-		(void) sprintf(buf, "%d minute%s until auto-logout.    \n",
-			       forceLogout - len,
-			       forceLogout - len == 1 ? "" : "s");
-		putText(win[scr], textgc[scr], buf, False, left, &x, &y);
-	}
-#endif /* AUTO_LOGOUT */
-	if (timeelapsed) {
-		(void) strcpy(buf, "Display has been locked for ");
-		putText(win[scr], textgc[scr], buf, False, left, &x, &y);
+		int         tmp = forceLogout - len;
 
-		if (len < 60)
-			(void) sprintf(buf, "%d minute%s.    \n", len, len == 1 ? "" : "s");
+		if (tmp < 60)
+			(void) sprintf(buf, "%d minute%s until auto-logout.    \n",
+				       tmp, (tmp == 1) ? "" : "s");
 		else
-			(void) sprintf(buf, "%d hour%s %d minute%s.    \n",
-				       len / 60, len / 60 == 1 ? "" : "s",
-				       len % 60, len % 60 == 1 ? "" : "s");
+			(void) sprintf(buf, "%d:%02d hours until auto-logout.  \n",
+				       tmp / 60, tmp % 60);
 		putText(win[scr], textgc[scr], buf, False, left, &x, &y);
 	}
+#endif /* USE_AUTO_LOGOUT */
 }
 
-#ifdef AUTO_LOGOUT
+#ifdef USE_AUTO_LOGOUT
 static void
 checkLogout(void)
 {
@@ -792,9 +843,9 @@ checkLogout(void)
 	}
 }
 
-#endif /* AUTO_LOGOUT */
+#endif /* USE_AUTO_LOGOUT */
 
-#ifndef OLD_EVENT_LOOP
+#ifndef USE_OLD_EVENT_LOOP
 
 /* subtract timer t2 from t1, return result in t3.  Result is not allowed to
    go negative, set to zero if result underflows */
@@ -879,7 +930,7 @@ runMainLoop(int maxtime, int iconscreen)
 				(struct timeval *) &sleep_time);
 #else
 		r = select(fd + 1,
-			(fd_set *) & reads, (fd_set *) NULL, (fd_set *) NULL,
+			   (void *) &reads, (void *) NULL, (void *) NULL,
 			   (struct timeval *) &sleep_time);
 #endif
 
@@ -928,7 +979,7 @@ runMainLoop(int maxtime, int iconscreen)
 	}
 }
 
-#endif /* !OLD_EVENT_LOOP */
+#endif /* !USE_OLD_EVENT_LOOP */
 
 static int
 ReadXString(char *s, int slen)
@@ -945,9 +996,11 @@ ReadXString(char *s, int slen)
 
 	for (screen = 0; screen < screens; screen++)
 		if (thisscreen == screen) {
-			call_init_hook((LockStruct *) NULL, mode_info(screen, icon[screen], True));
+			call_init_hook((LockStruct *) NULL,
+				       mode_info(screen, icon[screen], True));
 		} else {
-			call_init_hook((LockStruct *) NULL, mode_info(screen, win[screen], False));
+			call_init_hook((LockStruct *) NULL,
+				       mode_info(screen, win[screen], False));
 		}
 	statusUpdate(True, thisscreen);
 	bp = 0;
@@ -957,7 +1010,7 @@ ReadXString(char *s, int slen)
 		unsigned long lasteventtime = seconds();
 
 		while (!XPending(dsp)) {
-#ifdef OLD_EVENT_LOOP
+#ifdef USE_OLD_EVENT_LOOP
 			for (screen = 0; screen < screens; screen++)
 				if (thisscreen == screen)
 					call_callback_hook(NULL, mode_info(screen,
@@ -1042,14 +1095,14 @@ ReadXString(char *s, int slen)
 				}
 				break;
 
-#ifdef MOUSE_MOTION
+#ifdef USE_MOUSE_MOTION
 			case MotionNotify:
 #endif
 			case ButtonPress:
 				if (((XButtonEvent *) & event)->window == icon[screen]) {
 					return 1;
 				}
-#ifdef LOGOUT_BUTTON
+#ifdef USE_LOGOUT_BUTTON
 				if (((XButtonEvent *) & event)->window == button[screen]) {
 					XSetFunction(dsp, Scr[screen].gc, GXxor);
 					XSetForeground(dsp, Scr[screen].gc, fgcol[screen]);
@@ -1062,19 +1115,27 @@ ReadXString(char *s, int slen)
 				}
 #endif
 				break;
+
 			case Expose:
 				if (event.xexpose.count != 0)
 					break;
 				/* fall through on last expose event */
 			case VisibilityNotify:
-				/* window was restacked or exposed */
-				if (!debug && !inwindow)
-					XRaiseWindow(dsp, event.xvisibility.window);
-				call_refresh_hook((LockStruct *) NULL,
-				      mode_info(screen, win[screen], False));
-				s[0] = '\0';
-				return 1;
+				/* next line for -geometry */
+				if (event.xvisibility.state != VisibilityUnobscured) {
+					/* window was restacked or exposed */
+					if (!debug && !inwindow)
+						XRaiseWindow(dsp, event.xvisibility.window);
+					call_refresh_hook((LockStruct *) NULL,
+							  mode_info(screen, win[screen], False));
+					s[0] = '\0';
+					return 1;
+				}
+				break;
 			case ConfigureNotify:
+				/* next line for -geometry */
+				if (!fullscreen)
+					break;
 				/* window config changed */
 				if (!debug && !inwindow)
 					XRaiseWindow(dsp, event.xconfigure.window);
@@ -1084,11 +1145,10 @@ ReadXString(char *s, int slen)
 				}
 				s[0] = '\0';
 				return 1;
-
 			case KeymapNotify:
 			case KeyRelease:
 			case ButtonRelease:
-#ifndef MOUSE_MOTION
+#ifndef USE_MOUSE_MOTION
 			case MotionNotify:
 #endif
 			case LeaveNotify:
@@ -1115,8 +1175,9 @@ static int
 getPassword(void)
 {
 	XWindowAttributes xgwa;
-	int         x, y, left, done;
+	int         x, y, left, done, remy;
 	char        buffer[PASSLENGTH];
+	char      **msgp;
 
 	(void) nice(0);
 	if (!fullscreen)
@@ -1155,15 +1216,44 @@ getPassword(void)
 	putText(win[screen], textgc[screen], text_info, False, left, &x, &y);
 	putText(win[screen], textgc[screen], "\n", False, left, &x, &y);
 
+	timex = x;
+	timey = y;
+
+#ifdef USE_AUTO_LOGOUT
+	if (forceLogout) {
+		y += font->ascent + font->descent + 2;
+	}
+#endif
+#ifdef USE_LOGOUT_BUTTON
+	if (enable_button) {
+		y += font->ascent + font->descent + 2;
+	}
+#endif
+	if (timeelapsed) {
+		y += font->ascent + font->descent + 2;
+	}
+	remy = y;
+
+	putText(win[screen], textgc[screen], "\n", False, left, &x, &y);
+
+	y = msgy[screen];
+	if (*message) {
+		for (msgp = message; *msgp; ++msgp) {
+			y += msgfont->ascent + msgfont->descent + 2;
+			XDrawString(dsp, win[screen], msgtextgc[screen],
+				    msgx[screen], y, *msgp, strlen(*msgp));
+		}
+	}
 	XFlush(dsp);
 
-	y += font->ascent + font->descent + 2;
+	y = remy;
 
 	done = False;
 	while (!done) {
-#ifdef HAS_RPLAY
-		if (!got_invalid)
-			rplay_display(infosound);
+#ifdef USE_SOUND
+		if (sound && !got_invalid) {
+			play_sound(infosound);
+		}
 		got_invalid = 0;
 #endif
 		if (ReadXString(buffer, PASSLENGTH))
@@ -1185,7 +1275,7 @@ getPassword(void)
 		if (!done && !*buffer) {
 			/* just hit return, and it was not his password */
 			break;
-#ifdef SYSLOG
+#if defined( HAVE_SYSLOG_H ) && defined( USE_SYSLOG )
 		} else if (!done) {
 			/* bad password... log it... */
 			(void) printf("failed unlock attempt on user %s\n", user);
@@ -1197,8 +1287,9 @@ getPassword(void)
 		(void) memset((char *) buffer, 0, sizeof (buffer));
 
 		if (done) {
-#ifdef HAS_RPLAY
-			rplay_display(validsound);
+#ifdef USE_SOUND
+			if (sound)
+				play_sound(validsound);
 #endif
 			if (!fullscreen)
 				XConfigureWindow(dsp, win[screen], sizeconfiguremask,
@@ -1218,15 +1309,16 @@ getPassword(void)
 					       passx, passy - font->ascent,
 					       xgwa.width - passx,
 					       font->ascent + font->descent);
-#ifdef HAS_RPLAY
-			rplay_display(invalidsound);
+#ifdef USE_SOUND
+			if (sound)
+				play_sound(invalidsound);
 			got_invalid = 1;
 #endif
 		}
 	}
 	XChangeGrabbedCursor(mycursor);
 	XUnmapWindow(dsp, icon[screen]);
-#ifdef LOGOUT_BUTTON
+#ifdef USE_LOGOUT_BUTTON
 	XUnmapWindow(dsp, button[screen]);
 #endif
 	if (!fullscreen)
@@ -1262,25 +1354,25 @@ justDisplay(void)
 
 	while (not_done) {
 		while (!XPending(dsp)) {
-#ifdef OLD_EVENT_LOOP
+#ifdef USE_OLD_EVENT_LOOP
 			(void) usleep(onepause ? onepause : delay);
 			onepause = 0;
 			for (screen = 0; screen < screens; screen++)
 				call_callback_hook((LockStruct *) NULL,
 				      mode_info(screen, win[screen], False));
-#ifdef AUTO_LOGOUT
+#ifdef USE_AUTO_LOGOUT
 			checkLogout();
 #endif
 			XSync(dsp, False);
-#else /* !OLD_EVENT_LOOP */
-#ifdef AUTO_LOGOUT
+#else /* !USE_OLD_EVENT_LOOP */
+#ifdef USE_AUTO_LOGOUT
 			if (runMainLoop(30, -1) == 0)
 				break;
 			checkLogout();
 #else
 			(void) runMainLoop(0, -1);
 #endif
-#endif /* !OLD_EVENT_LOOP */
+#endif /* !USE_OLD_EVENT_LOOP */
 		}
 		XNextEvent(dsp, &event);
 
@@ -1330,7 +1422,7 @@ justDisplay(void)
 				}
 				break;
 
-#ifdef MOUSE_MOTION
+#ifdef USE_MOUSE_MOTION
 			case MotionNotify:
 				screen = event_screen(dsp, event.xmotion.root);
 				not_done = False;
@@ -1344,7 +1436,7 @@ justDisplay(void)
 		}
 
 		if (!nolock && lockdelay &&
-        (lockdelay <= (seconds() - start_time))) {
+		    (lockdelay <= (seconds() - start_time))) {
 			timetodie = True;
 			not_done = False;
 		}
@@ -1369,19 +1461,59 @@ sigcatch(int signum)
 	char        buf[512];
 
 	(void) sprintf(buf,
-    "Access control list restored.\n%%s: caught signal %d while running %s mode.\n",
-	  signum, name);
+		       "Access control list restored.\n%%s: caught signal %d while running %s mode.\n",
+		       signum, name);
 	finish();
 	error(buf);
 }
 
+#if defined( HAVE_SYSLOG_H ) && defined( USE_SYSLOG )
+static void
+syslogStart(void)
+{
+	struct passwd *pw;
+	struct group *gr;
+
+	pw = getpwuid(getuid());
+	gr = getgrgid(getgid());
+
+	(void) openlog(ProgramName, LOG_PID, LOG_AUTH);
+	/* (void) openlog(ProgramName, 0, SYSLOG_FACILITY); */
+	syslog(LOG_INFO, "Start: %s, %s, %s",
+	       pw->pw_name, gr->gr_name, XDisplayString(dsp));
+}
 
 static void
-lockDisplay(void)
+syslogStop(void)
 {
-#ifdef HAS_RPLAY
-	if (!inwindow && !inroot)
-		rplay_display(locksound);
+	struct passwd *pw;
+	struct group *gr;
+	int         seconds, minutes;
+
+	seconds = time((time_t *) 0) - start_time;
+	minutes = seconds / 60;
+	seconds %= 60;
+
+	pw = getpwuid(getuid());
+	gr = getgrgid(getgid());
+
+	syslog(LOG_INFO, "Stop: %s, %s, %s, %dm %ds",
+	       pw->pw_name, gr->gr_name, XDisplayString(dsp),
+	       minutes, seconds);
+	closelog();
+}
+
+#endif
+
+static void
+lockDisplay(Bool do_display)
+{
+#if defined( HAVE_SYSLOG_H ) && defined( USE_SYSLOG )
+	syslogStart();
+#endif
+#ifdef USE_SOUND
+	if (sound && !inwindow && !inroot)
+		play_sound(locksound);
 #endif
 	if (!allowaccess) {
 #if defined( SYSV ) || defined( SVR4 )
@@ -1429,19 +1561,116 @@ lockDisplay(void)
 	XHPDisableReset(dsp);
 #endif
 	do {
-		(void) justDisplay();
+		if (do_display)
+			(void) justDisplay();
+		else
+			do_display = True;
 	} while (getPassword());
 #if defined( __hpux ) || defined( __apollo )
 	XHPEnableReset(dsp);
 #endif
 }
 
+static void
+read_message()
+{
+	FILE       *msgf;
+	char        buf[121];
+	char       *home = getenv("HOME");
+  char       *buffer;
+  int         i, cr;
+
+  if (!home)
+    home = "";
+
+#if ( __VMS_VER >= 70000000 )
+	buffer = (char *) malloc(255);
+
+#else
+	buffer = (char *) malloc(strlen(home) + 32);
+
+#endif
+
+#ifdef VMS
+	(void) sprintf(buffer, "%s%s", home, ".xlockmessage");
+#else
+	(void) sprintf(buffer, "%s/%s", home, ".xlockmessage");
+#endif
+	msgf = fopen(buffer, "r");
+	if (msgf == NULL) {
+#ifdef VMS
+		(void) sprintf(buffer, "%s%s", home, ".plan");
+#else
+		(void) sprintf(buffer, "%s/%s", home, ".plan");
+#endif
+		msgf = fopen(buffer, "r");
+	}
+	if (msgf == NULL) {
+#ifndef VMS
+		(void) sprintf(buffer, "%s/%s", home, ".signature");
+#else
+#if ( __VMS_VER >= 70000000 )
+/* Get signature file for VMS 7.0 and higher */
+		char       *buffer1 = (char *) malloc(256);
+		unsigned int ival;
+		unsigned long int mail_context = 0;
+		unsigned short int buflen, buflen1;
+		struct itmlst_3 item[2], itm_d[1];
+
+		itm_d[0].buflen = 0;
+		itm_d[0].itmcode = 0;
+		item[0].buflen = 255;
+		item[0].itmcode = MAIL$_USER_SIGFILE;
+		item[0].bufadr = buffer1;
+		item[0].retlen = &buflen1;
+		item[1].buflen = 255;
+		item[1].itmcode = MAIL$_USER_FULL_DIRECTORY;
+		item[1].bufadr = buffer;
+		item[1].retlen = &buflen;
+		item[2].buflen = 0;
+		item[2].itmcode = 0;
+		ival = mail$user_begin(&mail_context, itm_d, item);
+		(void) mail$user_end(&mail_context, itm_d, itm_d);
+		(void) strncat(buffer, buffer1, buflen1);
+		(void) free((void *) buffer1);
+#else
+		(void) sprintf(buffer, "%s%s", home, ".signature");
+#endif
+#endif
+		msgf = fopen(buffer, "r");
+	}
+	if (msgf != NULL) {
+		for (i = 0; i < MESSAGELINES; i++) {
+			if (fgets(buf, 120, msgf)) {
+				cr = strlen(buf) - 1;
+				if (buf[cr] == '\n') {
+					buf[cr] = '\0';
+				}
+				message[i] = (char *) malloc(strlen(buf) + 1);
+				(void) strcpy(message[i], buf);
+			}
+		}
+		message[i] = NULL;
+		(void) fclose(msgf);
+	} else {
+		message[0] = NULL;
+	}
+	(void) free((void *) buffer);
+	buffer = NULL;
+}
+
 int
 main(int argc, char **argv)
 {
 	XSetWindowAttributes xswa;
 	XGCValues   xgcv;
 	XColor      nullcolor;
+	char      **msgp;
+	int         tmp;
+
+#ifdef OSF1_ENH_SEC
+	set_auth_parameters(argc, argv);
+#endif
 
 	ProgramName = strrchr(argv[0], '/');
 	if (ProgramName)
@@ -1449,28 +1678,19 @@ main(int argc, char **argv)
 	else
 		ProgramName = argv[0];
 
-#ifdef OSF1_ENH_SEC
-	set_auth_parameters(argc, argv);
-#endif
-
-#ifdef SYSLOG
-	(void) openlog(ProgramName, LOG_PID, LOG_AUTH);
-#endif
-
-	start_time = time((long *) 0);
+	start_time = time((time_t *) 0);
 	SRAND(start_time);	/* random mode needs the seed set. */
 
 	getResources(argc, argv);
 
-	checkResources();
-
 	initPasswd();
 
-	if (geteuid() == 0 && getuid() != 0) {
-		/* revoke root privs */
-		(void) setgid(getgid());
-		(void) setuid(getuid());
-	}
+	/* revoke root privs, if there were any */
+	(void) setgid(getgid());
+	(void) setuid(getuid());
+
+	checkResources();
+
 	font = XLoadQueryFont(dsp, fontname);
 	if (font == NULL) {
 		(void) fprintf(stderr, "%s: can not find font: %s, using %s...\n",
@@ -1512,11 +1732,22 @@ main(int argc, char **argv)
 			minisizeconfigure.height = flags & HeightValue ? h : iconheight;
 		}
 	}
+
+	msgfont = XLoadQueryFont(dsp, messagefont);
+	if (msgfont == NULL) {
+		(void) fprintf(stderr, "%s: can't find font: %s, using %s...\n",
+			       ProgramName, messagefont, FALLBACK_FONTNAME);
+		msgfont = XLoadQueryFont(dsp, FALLBACK_FONTNAME);
+		if (msgfont == NULL)
+			error("%s: can't even find %s!!!\n", FALLBACK_FONTNAME);
+	}
+	read_message();
+
 	screens = ScreenCount(dsp);
 	if (screens > MAXSCREENS)
 		error("%s: can only support %d screens.\n", MAXSCREENS);
 
-#ifdef DT_SAVER
+#ifdef USE_DTSAVER
 	/* The CDE Session Manager provides the windows for the screen saver
 	   to draw into. */
 	if (dtsaver) {
@@ -1634,7 +1865,7 @@ main(int argc, char **argv)
 		xswa.override_redirect = True;
 		xswa.background_pixel = BlackPixel(dsp, screen);
 		xswa.event_mask = KeyPressMask | ButtonPressMask |
-#ifdef MOUSE_MOTION
+#ifdef USE_MOUSE_MOTION
 			MotionNotify |
 #endif
 			VisibilityChangeMask | ExposureMask | StructureNotifyMask;
@@ -1645,11 +1876,11 @@ main(int argc, char **argv)
 		if (inroot) {
 			win[screen] = root[screen];
 			XChangeWindowAttributes(dsp, win[screen], CWBackPixel, &xswa);
-      /* this gives us these events from the root window */
-      XSelectInput(dsp, win[screen], VisibilityChangeMask | ExposureMask);
+			/* this gives us these events from the root window */
+			XSelectInput(dsp, win[screen], VisibilityChangeMask | ExposureMask);
 		} else
 #endif
-#ifdef DT_SAVER
+#ifdef USE_DTSAVER
 		if (!dtsaver)
 #endif
 		{
@@ -1682,19 +1913,41 @@ main(int argc, char **argv)
 			       XA_WM_HINTS, XA_WM_HINTS, 32, PropModeReplace,
 					(unsigned char *) &xwmh, sizeof (xwmh) / sizeof (int));
 		}
+		/*if (mono || CellsOfScreen(scr) <= 2) {
+		   Scr[screen].pixels[0] = fgcol[screen];
+		   Scr[screen].pixels[1] = bgcol[screen];
+		   Scr[screen].npixels = 2;
+		   } else */
+		fixColormap(dsp, win[screen], screen, saturation,
+			    mono, install, inroot, inwindow, verbose);
 
-		if (mono || CellsOfScreen(scr) <= 2) {
-			Scr[screen].pixels[0] = fgcol[screen];
-			Scr[screen].pixels[1] = bgcol[screen];
-			Scr[screen].npixels = 2;
-		} else
-			fixColormap(dsp, win[screen], screen, saturation,
-				    install, inroot, inwindow, verbose);
+		if (debug) {
+			iconx[screen] = (DisplayWidth(dsp, screen) - 100 -
+					 MAX(512, XTextWidth(font, text_info, strlen(text_info)))) / 2;
+			icony[screen] = (DisplayHeight(dsp, screen) - 100) / 6;
+		} else {
+			iconx[screen] = (DisplayWidth(dsp, screen) -
+					 MAX(512, XTextWidth(font, text_info, strlen(text_info)))) / 2;
+			icony[screen] = DisplayHeight(dsp, screen) / 6;
+		}
 
-		iconx[screen] = (DisplayWidth(dsp, screen) -
-			 XTextWidth(font, text_info, strlen(text_info))) / 2;
 
-		icony[screen] = DisplayHeight(dsp, screen) / 6;
+		msgx[screen] = msgy[screen] = 0;
+		for (msgp = message; *msgp; ++msgp) {
+			tmp = XTextWidth(msgfont, *msgp, strlen(*msgp));
+			if (tmp > msgx[screen])
+				msgx[screen] = tmp;
+			++msgy[screen];
+		}
+		if (debug) {
+			msgx[screen] = (DisplayWidth(dsp, screen) - 100 - msgx[screen]) / 2;
+			msgy[screen] = DisplayHeight(dsp, screen) - 100 -
+				(msgy[screen] + 4) * (msgfont->ascent + msgfont->descent + 2);
+		} else {
+			msgx[screen] = (DisplayWidth(dsp, screen) - msgx[screen]) / 2;
+			msgy[screen] = DisplayHeight(dsp, screen) -
+				(msgy[screen] + 4) * (msgfont->ascent + msgfont->descent + 2);
+		}
 
 		xswa.border_pixel = WhitePixel(dsp, screen);
 		xswa.background_pixel = BlackPixel(dsp, screen);
@@ -1709,7 +1962,7 @@ main(int argc, char **argv)
 						 InputOutput, CopyFromParent,
 						     CIMASK, &xswa);
 
-#ifdef LOGOUT_BUTTON
+#ifdef USE_LOGOUT_BUTTON
 		{
 			char        buf[1024];
 			int         w, h;
@@ -1741,6 +1994,9 @@ main(int argc, char **argv)
 		xgcv.background = bgcol[screen];
 		textgc[screen] = XCreateGC(dsp, win[screen],
 				GCFont | GCForeground | GCBackground, &xgcv);
+		xgcv.font = msgfont->fid;
+		msgtextgc[screen] = XCreateGC(dsp, win[screen],
+				GCFont | GCForeground | GCBackground, &xgcv);
 	}
 	lockc = XCreateBitmapFromData(dsp, root[0], no_bits, 1, 1);
 	lockm = XCreateBitmapFromData(dsp, root[0], no_bits, 1, 1);
@@ -1757,7 +2013,7 @@ main(int argc, char **argv)
 	}
 	if (!grabmouse || inwindow || inroot)
 		nolock = 1;
-  else if (!debug)
+	else if (!debug)
 		GrabKeyboardAndMouse();
 	(void) nice(nicelevel);
 
@@ -1766,15 +2022,16 @@ main(int argc, char **argv)
 	if (nolock) {
 		(void) justDisplay();
 	} else if (lockdelay) {
-		if (justDisplay())
-			lockDisplay();
-		else
+		if (justDisplay()) {
+			lockDisplay(False);
+		} else
 			nolock = 1;
-	} else
-		lockDisplay();
-
-#ifdef SYSLOG
-	closelog();
+	} else {
+		lockDisplay(True);
+	}
+#if defined( HAVE_SYSLOG_H ) && defined( USE_SYSLOG )
+	if (!nolock)
+		syslogStop();
 #endif
 
 	finish();
